<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\Components;

use App\Model\Enum\MessageType;
use Nette\InvalidStateException;
use Nette\Security\User;
use Nette\Utils\ArrayHash;
use Nette\Forms\Container;
use Nette\Application\UI\Control;
use Nette\Application\UI\Form;
use App\Forms\BaseFormFactory;
use Nette\Forms\Controls\Checkbox;
use App\Model\UserManager;
use App\Model\PersonManager;
use App\Services\FormatHelperService;
use Nette\Database\UniqueConstraintViolationException;

/**
 * Class PersonFormControl
 * @package App\Components
 * 
 * @author Jan Vodák <jan at ideatech.cz>
 */
class PersonFormControl extends Control {

    /** Cesta k souboru šablony pro tuto komponentu. */
    const TEMPLATE = '/../templates/components/PersonForm/personForm.latte';

    /** @var User Uživatel. */
    private $user;

    /** @var PersonManager Instance třídy modelu pro práci s osobami. */
    private $personManager;

    /** @var BaseFormFactory Vlastní definovaná továrnička na formuláře, která se používá skoro vždy. */
    private $formFactory;

    /** @var UserManager Instance třídy modelu pro práci s uživateli. */
    private $userManager;

    /**
     * @var FormatHelperService Pomocná služba pro formátování data, jména adresy atd.
     */
    private $formatHelperService;

    /** @var null|string Cesta k šabloně formulářů; pokud je null, použije výchozí Nette vykreslování. */
    public $formPath = null;

    /** @var array Pole funkcí volajících se při úspěšném odeslání formuláře. */
    public $onSuccess = [];

    /**
     * Konstruktor s injectovanými třídami uživatele, modelu pro práci s osobami, továrničky na formuláře a modelu pro
     * práci s uživateli.
     * @param User                  $user          automaticky injektovaná třída uživatele
     * @param PersonManager         $personManager automaticky injektovaná třída modelu pro práci s osobami
     * @param BaseFormFactory       $formFactory   automaticky injektovaná továrnička na formuláře
     * @param UserManager           $userManager   automaticky injektovaná třída modelu pro práci s uživateli
     * @param FormatHelperService   $formatHelperService automaticky injektovaná třída modelu pro formátování a generování hesla
     */
    public function __construct(User $user, PersonManager $personManager, BaseFormFactory $formFactory, UserManager $userManager, FormatHelperService $formatHelperService) {
        parent::__construct();
        $this->user = $user;
        $this->personManager = $personManager;
        $this->formFactory = $formFactory;
        $this->userManager = $userManager;
        $this->formatHelperService = $formatHelperService;
    }

    /**
     * Getter pro uživatele.
     * @return User uživatel
     */
    private function getUser() {
        return $this->user;
    }

    /** Vykreslí komponentu formuláře pro správu osob. */
    public function render($userID = null) {
        // Nastaví šablonu komponenty.
        $this->template->setFile(__DIR__ . self::TEMPLATE);

        if ($userID) {
            if (!$this->getUser()->isLoggedIn() || (!$this->getUser()->isInRole('admin') && !$this->getUser()->isInRole('superadmin'))) {
                throw new InvalidStateException('Editovat jinou osobu smí pouze administrátor.');
            }

            // Pokud se předalo výchozí personID - bude se editovat cizí profil
            $person = $this->personManager->getPerson($this->personManager->getPersonId($userID));
        } else {
            // Pokud není zadáno konkrétní perdonID, a pokud je uživatel přihlášený - bude editovat svůj účet
            if ($this->getUser()->isLoggedIn() && ($id = $this->getUser()->getId())) {
                $person = $this->personManager->getPerson($this->personManager->getPersonId($id));
            }else{
                $person = null;
            }
        }

        // Pokud se nasel uzivatel pro editace - vyplním se výchozí údaje pro editaci
        if ($person) {
            /** @var Container $personDetail */
            $personDetail = $this['personForm']['person_detail'];
            $personDetail->setDefaults($person->person_detail);

            /** @var Container $address */
            $address = $this['personForm']['address'];
            $address->setDefaults($person->address);

            if (isset($person->delivery_address_id)) {
                /** @var Container $deliveryAddress */
                $deliveryAddress = $this['personForm']['delivery_address'];
                $deliveryAddress->setDefaults($person->delivery_address);

                /** @var Checkbox $ommitDeliveryAddress */
                $ommitDeliveryAddress = $this['personForm']['omit_delivery_address'];
                $ommitDeliveryAddress->setValue(false);
            }

            if (($this->getUser()->isInRole('admin') || $this->getUser()->isInRole('superadmin')) && $person->bank_account) {
                $bankAccount = $this['personForm']['bank_account'];
                $bankAccount->setDefaults($person->bank_account);

                $bankAccount->setDefaults(['registry_entry' => $person->person_detail->registry_entry]);
            }
        }

        // Předává parametry do šablony.
        $this->template->formPath = $this->formPath;
        // Vykreslí komponentu.
        $this->template->render();
    }

    /**
     * Vykreslí komponentu formuláře pro editaci osoby s danými údaji. Umožní editovat osobu jinou, než aktuálně přihlášenou
     * Měním údaje osoby v objednávce, aniž bych změnil údaje zákazníkovi odsud bez jeho vědoví
     * Je lepší uložit nové údaje metodou savePerson(). 
     * Bude potřeba vytvořit jednotlivé části osoby znovu, nastavit jim změněné hodnoty a ty poté nastavit objednávce. 
     * Funkce savePerson také vytvoří kompletně novou osobu, což v tomto případě není potřeba. 
     * Řešení bez přidaných parametrů metodě - vytvořenou osobu po použití odstranit. 
     * 
     * ID osoby se nastavuje na true, čímž se dává najevo, že se jedná o editaci jiné osoby než aktuálně přihlášené. 
     * Je potřeba vytvořit nové údaje osoby na základě těch starých. 
     * Ty se poté přiřadí pouze konkrétní objednávce, tj. budou na faktuře a původní osoba zůstane netknutá. 
     * 
     * @param array|Traversable $personDetail    detail osoby
     * @param array|Traversable $address         adresa osoby
     * @param array|Traversable $deliveryAddress dodací adresa osoby
     */
    public function renderManage($personDetail, $address, $deliveryAddress) {
        if (!$this->getUser()->isLoggedIn() || (!$this->getUser()->isInRole('admin') && !$this->getUser()->isInRole('superadmin'))) {
            throw new InvalidStateException('Editovat jinou osobu smí pouze administrátor.');
        }

        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE);
        // Schová z formuláře možnost pro editaci bankovních údajů jiné osoby.
        $this['personForm']->removeGroup('Platební údaje');

        // Nastaví ID editované osoby.
        $this['personForm']->setDefaults(['person_id' => true]);

        // Vyplníme výchozí údaje editované osoby do formuláře.
        /** @var Container $personDetailContainer */
        $personDetailContainer = $this['personForm']['person_detail'];
        $personDetailContainer->setDefaults($personDetail);

        /** @var Container $addressContainer */
        $addressContainer = $this['personForm']['address'];
        $addressContainer->setDefaults($address);

        /** @var Container $deliveryAddressContainer */
        $deliveryAddressContainer = $this['personForm']['delivery_address'];
        $deliveryAddressContainer->setDefaults($deliveryAddress);

        if ($address[PersonManager::COLUMN_ADDRESS_ID] != $deliveryAddress[PersonManager::COLUMN_ADDRESS_ID]) {
            /** @var Checkbox $ommitDeliveryAddress */
            $ommitDeliveryAddress = $this['personForm']['omit_delivery_address'];
            $ommitDeliveryAddress->setValue(false);
        }

        // Předává parametry do šablony.
        $this->template->formPath = $this->formPath;
        $this->template->render(); // Vykreslí komponentu.
    }

    /** Vykreslí komponentu formuláře pro správu osob s povinnou registrací nového uživatele. */
    public function renderForceAccount() {
        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE);

        if (!$this->getUser()->isLoggedIn()) {
            // Pokud není uživatel přihlášený - automaticky vynutíme povinnost registrace.
            // Pokud přihlášený je - vytvářím ho v adminsitraci - heslo se generuje samo ve vlastním zpracování v presenteru
            /** @var Checkbox $createAccount */
            $createAccount = $this['personForm']['create_account'];
            $createAccount->setValue(true)
                    ->setRequired()
                    ->setOption('display', false);
        }

        // Předává parametry do šablony.
        $this->template->formPath = $this->formPath;
        // Vykreslí komponentu.
        $this->template->render();
    }

    /**
     * Vytváří a vrací komponentu formuláře pro správu osoby.
     * addGroup() a addContainer() = to dva samostatné prvky, ale v tomto případě trochu provázané
     * -> vyplývá z vlastnosti, že kontejner může patřit jen do jedné konkrétní skupiny.
     * skupiny budou využity při vykreslování formuláře a kontejnery pro efektivní extrakci dat
     * addCondition() -  přidá podmínku, např. validuj daný prvek pokud je vůbec vyplněn (Form::FILLED).
     * addConditionOn() - přidá podmínku, která je navázaná na jiný prvek formuláře, 
     * např. pokud je jiný prvek vyplněn, tento prvek má být také vyplněn. použito u jména a příjmení vs. název firmy 
     * nebo u dodací adresy, tj. pokud je zaškrtnuto, že je jiná, než běžná adresa, rázem jsou její příslušná pole povinná.
     * elseCondition() - Validační pravidla uvedená za touto metodou se aplikují v opačném případě, než praví předcházející podmínka.
     * addRule() - přidá k prvku formuláře validační pravidlo, např. že musí obsahovat celé číslo (Form::INTEGER).
     * setOmitted() - vynechá prvek při získávání hodnot z formuláře.
     * 
     * @return Form formulář pro správu osoby
     */
    protected function createComponentPersonForm() {
        $countries = $this->personManager->getCountries();

        $bankCodes = [];
        $codes = $this->personManager->getAllRows(PersonManager::TABLE_BANK_CODE_NAME);
        foreach ($codes as $code) {
            $bankCodes[$code->{PersonManager::COLUMN_BANK_CODE}] = $code->{PersonManager::COLUMN_BANK_NAME} . ' - ' . $code->{PersonManager::COLUMN_BANK_CODE};
        }

        $form = $this->formFactory->create();

        //$form->getElementPrototype()->novalidate = 'novalidate';
        // Skryté pole pro případné ID osoby.
        $form->addHidden('person_id');
        //Skryté pole pro případné ID uživatele - pokud vytvářím nového uživatele z adminu, toto mi pomáhá při zpracování formu
        $form->addHidden('user_id');

        // Registrace.
        if (!$this->getUser()->isLoggedIn()) {
            $form->addCheckbox('create_account', 'Vytvořit zákaznický účet')->setHtmlId('create_account');
            $form->addGroup('Zákaznický účet')->setOption('id', 'create-account-controls');
            $form->addPassword('password', 'Heslo')
                    ->setAttribute('title', 'Zadejte heslo alespoň na 6 znaků.')
                    ->addConditionOn($form['create_account'], Form::EQUAL, true)
                    ->setRequired()
                    ->addRule(Form::PATTERN, 'Heslo nesmí obsahovat diakritiku a musí být dlouhé alespoň 6 znaků.', '.{6,}');
            $form->addPassword('password_repeat', 'Heslo znovu')
                    ->setOmitted()
                    ->setAttribute('title', 'Zadejte heslo alespoň na 6 znaků.')
                    ->addConditionOn($form['create_account'], Form::EQUAL, true)
                    ->setRequired()
                    ->addRule(Form::EQUAL, 'Hesla nesouhlasí.', $form['password']);
        }

        // Detail.
        $form->addGroup('Osoba');
        $personDetail = $form->addContainer('person_detail');
        $personDetail->addText('email', 'Email')->setType('email')->addRule(Form::EMAIL)->setRequired();
        $firstName = $personDetail->addText('first_name', 'Jméno');
        $lastName = $personDetail->addText('last_name', 'Příjmení');
        $personDetail->addText('company_name', 'Společnost');
        // Setování podmínek pro konkrétní prvky formu
        $firstName->addConditionOn($personDetail['company_name'], Form::BLANK)->setRequired()
                ->elseCondition()->addRule(Form::BLANK, 'Osoba může být fyzická (Jméno a Příjmení), nebo právnická (Společnost).');
        $lastName->addConditionOn($personDetail['company_name'], Form::BLANK)->setRequired()
                ->elseCondition()->addRule(Form::BLANK, 'Osoba může být fyzická (Jméno a Příjmení), nebo právnická (Společnost).');
        $personDetail->addText('identification_number', 'IČ');
        $personDetail->addText('tax_number', 'DIČ');

        // Kontaktní údaje - součástí detailu.
        $personDetail->setCurrentGroup($form->addGroup('Kontaktní údaje'));
        $personDetail->addText('phone', 'Telefon');
        $personDetail->addText('fax', 'Fax');

        // Adresa.
        $form->addGroup('Adresa');
        $address = $form->addContainer('address');
        $address->addText('street', 'Ulice')->setRequired();
        $address->addText('registry_number', 'Číslo popisné')->setRequired();
        $address->addText('house_number', 'Číslo orientační');
        $address->addText('city', 'Město')->setRequired();
        $address->addText('zip', 'PSČ')->setRequired();
        $address->addSelect('country_id', 'Stát', $countries)->setRequired();
        //$address->addHidden('country_id', 1);
        // Checkbox pro možnost buď vyplnit nebo nevyplnit dodací adresu.
        $form->addCheckbox('omit_delivery_address', 'Dodací adresa se shoduje s fakturační')
                ->setDefaultValue(true)->setHtmlId('omit_delivery_address');

        // Dodací adresa.
        $form->addGroup('Doručovací adresa')->setOption('id', 'delivery-address-controls');
        $deliveryAddress = $form->addContainer('delivery_address');
        $deliveryAddress->addText('street', 'Ulice');
        $deliveryAddress->addText('registry_number', 'Číslo popisné')
                ->addConditionOn($form['omit_delivery_address'], Form::EQUAL, false)->setRequired();
        $deliveryAddress->addText('house_number', 'Číslo orientační');
        $deliveryAddress->addText('city', 'Město')
                ->addConditionOn($form['omit_delivery_address'], Form::EQUAL, false)->setRequired();
        $deliveryAddress->addText('zip', 'PSČ')
                ->addConditionOn($form['omit_delivery_address'], Form::EQUAL, false)->setRequired();
        $deliveryAddress->addSelect('country_id', 'Stát', $countries)
                ->addConditionOn($form['omit_delivery_address'], Form::EQUAL, false)->setRequired();
        //$deliveryAddress->addHidden('country_id', 1);
        if ($this->getUser()->isLoggedIn()) {
            if ($this->getUser()->isInRole('admin') || $this->getUser()->isInRole('superadmin')) {
                // Bankovní údaje, pokud je uživatel administrátor.
                $form->addGroup('Platební údaje');
                $bankAccount = $form->addContainer('bank_account');
                $bankAccount->addSelect('bank_code', 'Kód banky', $bankCodes)
                        ->setPrompt('Zvolte typ banky');
                $bankAccount->addText('account_number', 'Číslo účtu')->setType('number')
                        ->addCondition(Form::FILLED)->addRule(Form::INTEGER);
                $bankAccount->addTextArea('registry_entry', 'Spisová značka');

                $form->setCurrentGroup();
                $form->addSubmit('send', 'Potvrdit')
                    ->setAttribute('id', 'changeUserDetailSubmit');
            } else {
                $form->setCurrentGroup();
                $form->addSubmit('register', 'Uložit')
                    ->setAttribute('id', 'changeUserDetailSubmit');
            }
        } else {
            $form->setCurrentGroup();
            $form->addSubmit('register', 'Potvrdit')
                ->setAttribute('id', 'registerSubmit');
        }


        $form->onSuccess[] = [$this, 'processForm'];
        return $form;
    }

    /**
     * Funkce se vykonává při úspěšném odeslání formuláře pro správu osob - jak správně uložit data.
     * Uloží novou / edituje starou osobu a případně k ní registruje daného uživatele.
     * I když se uživatel nebude registrovat - jednorázový nákup -> stejně se jeho údaje uloži do tabulky PERSON a do dalších přidružených
     * @param Form      $form   formulář pro správu osob
     * @param ArrayHash $values odeslané hodnoty formuláře
     */
    public function processForm(Form $form, ArrayHash $values) {
        $userId = null;
        $personId = null;
        $password = null;

        if (!empty($values->create_account) || (empty($values->person_id) && $this->getUser()->isLoggedIn() && !empty($values->user_id))) {
            // Registrace nového uživatele
            // Pokud chci registrovat účet - podmínka z formu, nebo ho chci vytvořit z adminu - druhá část podmíky - je zadán vyplňovatel
            // Heslo: pokud ho chci vytvořit z adminu - generování hesla, jinak heslo z formu od uživatele
            $password = !empty($values->user_id) ? $this->formatHelperService->generatePassword(10) : $values->password;

            try {
                $userId = $this->userManager->register($values->person_detail->email, $password);
            } catch (UniqueConstraintViolationException $ex) {
                $this->presenter->flashMessage('Uživatel s tímto loginem již existuje.', MessageType::ERROR);
                $this->presenter->redirect('this');
            }
        } else if (empty($values->person_id) && $this->getUser()->isLoggedIn() && empty($values->user_id)) {
            // Změna údajů přihlášeného uživatele (edituje sám sebe) - pokud není personID, je přihlášen a není userID editora.
            $userId = $this->getUser()->getId();
            $personId = $this->personManager->getPersonId($userId);
            try {
                $this->userManager->changeUserEmail($userId, $values->person_detail->email);
            } catch (UniqueConstraintViolationException $ex) {
                $this->presenter->flashMessage('Uživatel s tímto loginem již existuje.', MessageType::ERROR);
                $this->presenter->redirect('this');
            }
        } else if (!empty($values->person_id) && $this->getUser()->isLoggedIn() && !empty($values->user_id)) {
            // Změna údajů přihlášeného uživatele (edituje admin někoho) - pokud je personID a userID editované osoby a editor je přihlášen
            $userId = $values->user_id;
            $personId = $this->personManager->getPersonId($userId);
            try {
                $this->userManager->changeUserEmail($userId, $values->person_detail->email);
            } catch (UniqueConstraintViolationException $ex) {
                $this->presenter->flashMessage('Uživatel s tímto loginem již existuje.', MessageType::ERROR);
                $this->presenter->redirect('this');
            }
        }
        //// Jinak vytváříme novou osobu bez uživatelského účtu.

        /** @var ArrayHash $personDetail */
        $personDetail = $values->person_detail;
        $bankAccount = null;

        if ($this->getUser()->isInRole('admin') || $this->getUser()->isInRole('superadmin')) {
            // Spisovná značká - patří do detailu osoby. Jsou to ty kecy, že je firma zapsána do... Nepatří do tabulky bank_account
            /** @var Container $bankAccountContainer */
            $bankAccountContainer = $form['bank_account'];
            if (array_filter($bankAccountContainer->getValues(true))) {
                /** @var ArrayHash $bankAccount */
                $bankAccount = $values->bank_account;
                $personDetail->registry_entry = $bankAccount->registry_entry;
                $bankAccount->offsetUnset('registry_entry');
            }
        }

        try {
            // Pokusíme se uložit osobu.
            $personId = $this->personManager->savePerson(
                    $personDetail, $values->address, $personId, $values->omit_delivery_address ? null : $values->delivery_address, $bankAccount,
                    // Pokud je uživatel přihlášený, testuji jeslti jsem vytvořil uživatele - pokud ano, předám id, jinak id přihlášeného, jinak userID
                    $this->getUser()->isLoggedIn() ? $userId ? $userId : $this->getUser()->getId() : $userId
            );

            // Přidáme do dále předávaných hodnot nově získané hodnoty.
            $values->user_id = $userId;
            $values->person_id = $personId;
            $values->password = $password;

            // Zavoláme všechny další funkce pro úspěšné odeslání formuláře s příslušnými parametry - budu si je definovat v presenteru.
            foreach ($this->onSuccess as $successFunction) {
                $successFunction($form, $values);
            }
        } catch (UniqueConstraintViolationException $ex) {
            $form->addError('Uživatel s tímto jménem je již zaregistrovaný.');
        }
    }

    /* Pomocná funkce pro nasetování defaultních hodnot z venku - z venku ju volám a tady hodnoty setuji */

    public function setDefaults(array $values) {
        $this['personForm']->setDefaults($values);
    }

}

/**
 * Továrnička pro vytvoření komponenty formuláře pro správu osob s využitím autowiringu.
 * @package App\Components
 */
interface IPersonFormFactory {

    /**
     * Vrací novou komponentu formuláře pro správu osob s automaticky injectovanými závislostmi.
     * @return PersonFormControl komponenta formuláře pro správu osob
     */
    public function create();
}
