<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\Components;

use Nette\Application\UI\Control;
use Nette\Utils\ArrayHash;
use Nette\Security\User;
use App\EshopModule\Model\DiscountLevelsManager;
use App\EshopModule\Model\CurrencyManager;

/**
 * Pomocná komponenta pro vykreslování ceny produktů, tak i v košíku.
 * Jediné místo, kde se komponenta nevyužívá je OrderManager - completeOrder a getOrderSummary - pozor na úpravu
 * 
 * Použití: automatiky se vytvoří se všemi závislostmi injectovanými pomocí DI klasicky v kontruktoru viz IPriceControlFactory
 * @package App\Components
 *  
 * @author Jan Vodák <jan at ideatech.cz>
 */
class PriceControl extends Control {

    /** Cesta k souborům šablon pro tuto komponentu. */
    const TEMPLATE_PATH = '/../templates/components/Price/';

    /** @var string Měna, ve které se má cena vykreslit. */
    private $currency;

    /** @var DiscountLevelsManager */
    private $discountLevelManager;

    /** @var CurrencyManager */
    private $currencyManager;

    /**
     * Konstruktor komponenty pro vykreslení ceny produktu.
     * @param DiscountLevelsManager $discountLevelManager automaticky injektovaná třída pro slevy
     */
    public function __construct(DiscountLevelsManager $discountLevelManager, CurrencyManager $currencyManager) {
        parent::__construct();
        $this->discountLevelManager = $discountLevelManager;
        $this->currencyManager = $currencyManager;
        $this->currency = $this->currencyManager->getActualCurrency();
    }

    /**
     * Výchozí metoda pro vykreslení komponenty - hlavní využití ve frontEndu
     * Zformátuje cenu a připočte k ní DPH podle toho, zda dotyčný provozovatel jeho plátce, což se stejně jako jeho výši dozví z
     * účetního nastavení. Pokud není zadáno, bere se vždy cena bez DPH.
     * @param float  $price     cena bez DPH
     * @param null|ArrayHash    $settings účetní nastavení nebo null, pokud se má pouze zformátovat cena bez DPH
     * @param null|User $user   aktuálně přihlášený uživatel
     * @param null|ArrayHash    $currency požadovaná měna a její kurz. 
     *                          Může se stát, že na některých místech budu potřeba počítat s jinou měnou, než která je momentálně nastavena
     *                          Hlavně v administraci ve správě objednávek - počítá se s tou měnou, ve které byla obednávka objednána
     * @param null|int          $aditionalPrice Pro počítání slev z celkového součtu objednávky - slevy nepočítám pro dopravy a platby
     *                          Vypočítám cenu + DPH + uživatelské slevy a nakonec k tomu přičtu částku, které se slevy netýkají
     */
    public function render($price, $settings = null, User $user = null, $currency = null, $aditionalPrice = null, $precision = null) {
        // Pokud je nastavený user získám si jeho id
        $userId = ($user && $user->getIdentity() && $user->isLoggedIn()) ? $user->getIdentity()->getId() : null;

        // Provolání metody pro vytvoření šabloný
        $this->fillTemplate($price, $settings, $userId, $currency, $aditionalPrice, $precision);
    }

    /**
     * Výchozí metoda pro vykreslení komponenty - hlavní využití v backendu, nemusím řešit ověření přihlášeného uživatele
     * @param float  $price     cena bez DPH
     * @param null|ArrayHash    $settings účetní nastavení nebo null, pokud se má pouze zformátovat cena bez DPH
     * @param null|User $userId id uživatele
     * @param null|ArrayHash    $currency požadovaná měna a její kurz.                
     * @param null|int          $aditionalPrice Pro počítání slev z celkového součtu objednávky - slevy nepočítám pro dopravy a platby
     */
    public function renderPriceForBackend($price, $settings = null, $userId = null, $currency = null, $aditionalPrice = null) {
        // Provolání metody pro vytvoření šabloný
        $this->fillTemplate($price, $settings, $userId, $currency, $aditionalPrice);
    }

    /**
     * Vrátí slevu podle staré a nové ceny ve formátu např.: "-20%"
     * Využívám i pro jiné porovnání, než jsou slevy - například v adminu porovnání objednávek a počtu nových zákazníků
     * @param float $oldPrice původní cena
     * @param float $newPrice nová cena
     */
    public function renderSale($oldPrice, $newPrice) {
        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE_PATH . 'sale.latte');
        // Předává parameter do šablony.
        $this->template->sale = $oldPrice != 0 ? round(abs($oldPrice - $newPrice) / ($oldPrice / 100)) : round(($newPrice - 0)*100);
        //Ještě porovnám, jeslti je nová cena nižší a potom i podle toho doplním známenko 
        $this->template->oldPrice = $oldPrice;
        $this->template->newPrice = $newPrice;
        // Vykreslí komponentu.
        $this->template->render();
    }

    /**
     * Pomocná proměnná pro vytvoření a naplnění šablony
     * @param float  $price     cena bez DPH
     * @param null|ArrayHash    $settings účetní nastavení nebo null, pokud se má pouze zformátovat cena bez DPH
     * @param null|User $userId id uživatele
     * @param null|ArrayHash    $currency požadovaná měna a její kurz.                
     * @param null|int          $aditionalPrice Pro počítání slev z celkového součtu objednávky - slevy nepočítám pro dopravy a platby
     */
    private function fillTemplate($price, $settings = null, $userId = null, $currency = null, $aditionalPrice = null, $precision = 2) {
        // Pokud je požadována speciální měna, použiju speciální. Jinak si hledám tu, která je momentálně uživatelem zvolena
        $currencyUsed = $currency ? $this->currencyManager->getCurrency($currency) : $this->currency;

        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE_PATH . 'price.latte');
        // Vypočítám cenu a vrátím ji do šablony. Předám ji částku, měnu a kurz, settings pro DPH a id uživatele pro slevy
        $this->template->price = $this->discountLevelManager->calculatePrice($price, $currencyUsed, $settings, $userId, $aditionalPrice);
        // Vložení měny pro výpis
        $this->template->currency = $currencyUsed[CurrencyManager::COLUMN_CURRENCY_SYMBOL];
        $this->template->precision = $precision;

        // Vykreslí komponentu.
        $this->template->render();
    }

}

/**
 * Interface pro vytvoření pomocné komponenty (PriceControl) pro vykreslování ceny s využitím autowiringu.
 * Autowiring spočívá v implementaci pomocí interface a definici v configu, kde si v settings volám setr a nastavují měnu
 * Automatiké vytvoření se všemi závislostmi injectovanými pomocí DI klasicky v kontruktoru + přes metodu createComponent a zavolání create
 * @package App\Components
 */
interface IPriceControlFactory {

    /**
     * Vrací novou pomocnou komponentu pro vykreslování ceny s automaticky inejectovanými závislostmi.
     * @return PriceControl pomocná komponenta pro vykreslování ceny
     */
    public function create();
}
