<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\Forms;

use App\Model\Enum\MessageType;
use Nette\Application\UI\Form;
use Nette\Object;
use Nette\Security\AuthenticationException;
use Nette\Security\User;
use Nette\Utils\ArrayHash;

/**
 * Class UserForms Továrnička na tvorbu formuláře pro uživatelské přihlášení
 * @package App\Forms
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class UserForms extends Object {

    /** @var User Uživatel. */
    private $user;

    /** @var BaseFormFactory Továrnička na formuláře. */
    private $formFactory;

    /** @var array Pole funkcí volajících se při úspěšném odeslání formuláře. */
    public $onSuccess = [];

    /**
     * Konstruktor s injektovanou třidou uživatele.
     * @param User $user automaticky injektovaná třída uživatele
     * @param BaseFormFactory $baseFormFactory automaticky injektovaná továrnička na formuláře se společným nastavením
     */
    public function __construct(User $user, BaseFormFactory $baseFormFactory) {
        $this->user = $user;
        $this->formFactory = $baseFormFactory;
    }

    /**
     * Vrací formulář se společným základem.
     * @param null|Form $form formulář, který se má rozšířit o společné prky, nebo null, pokud se má vytvořit nový formulář
     * @return Form formulář se společným základem
     */
    private function createBasicForm(Form $form = null) {
        $form = $form ? $form : $this->formFactory->create();
        $form->addText('email', 'Email')->setType('email')->addRule(Form::EMAIL)->setRequired();
        $form->addPassword('password', 'Heslo')->setRequired();
        return $form;
    }

    /**
     * Vrací komponentu formuláře s přihlašovacími prvky a zpracování přihlašování podle uživatelských instrukcí.
     * @param null|ArrayHash $instructions uživatelské instrukce pro zpracování registrace
     * @param null|Form $form  = komponenta formuláře, která se má rozšířit o přihlašovací prvky, nebo null, pokud se má vytvořit nový formulář
     * @return Form komponenta formuláře s přihlašovacími prky
     */
    public function createLoginForm($instructions = null, Form $form = null) {
        $form = $this->createBasicForm($form);
        $form->addSubmit('submit', 'Přihlásit');

        $form->onSuccess[] = function (Form $form) use ($instructions) {
            $this->login($form, $instructions);
        };

        return $form;
    }

    /**
     * Přihlašuje a případně registruje uživatele.
     * @param Form $form - formulář, ze kterého se metoda volá
     * @param null|ArrayHash $instructions uživatelské instrukce
     * @param bool $register - registruj nového uživatele
     */
    private function login($form, $instructions) {
        // Získej presenter ve kterém je formulář umístěn.
        $presenter = $form->getPresenter();
        // Extrakce hodnot z formuláře.
        $values = $form->getValues();
        try {
            // Zkusíme se přihlásit.
            $this->user->login($values->email, $values->password);
            // Pokud jsou zadány uživatelské instrukce a formulář je umístěn v presenteru.
            if ($instructions && $presenter) {
                // Pokud instrukce obsahují zprávu, tak ji pošli do příslušného presenteru.
                if (isset($instructions->message)) {
                    $presenter->flashMessage($instructions->message,
                            // Pokud instrukce obsahují typ zprávy, tak ho použij, jinak použij výchozí informativní typ.
                            isset($instructions->msg_type) ? $instructions->msg_type : MessageType::INFO
                    );
                }
                
                // Rozšíření zpracování odeslání formuláře -  bude používáno v presenterech - zde ho volám ještě před redirectem.
                foreach ($this->onSuccess as $successFunction) {
                    $successFunction($form, $values);
                }
                
                // Pokud instrukce obsahují přesměrování, tak ho proveď na příslušném presenteru.
                if (isset($instructions->redirection)) {
                    $presenter->redirect($instructions->redirection);
                }
            }
        } catch (AuthenticationException $ex) {
            // Přihlášení selhalo.
            if ($presenter) {
                // Pokud je formulář v presenteru - pošli tam chybovou zprávu.
                $presenter->flashMessage($ex->getMessage(), MessageType::ERROR);
                // Proveď přesměrování.
                $presenter->redirect('this');
            } else {
                // Jinak přidej chybovou zprávu alespoň do samotného formuláře.
                $form->addError($ex->getMessage());
            }
        }
    }

}
