<?php

/**
 * Základní abstraktní třída modelu pro všechny modely aplikace.
 * Předává přístup k práci s databází a zamezuje duplicitnímu kódu.
 * To se docílí tím, že všechny ostatní modely budou potomek tohoto modelu a získají tím přástup k DB.
 * 
 * @package App\Model
 * @author Jan Vodák <jan at ideatech.cz>
 */

namespace App\Model;

use App\Model\Enum\Status;
use Nette\Database\Context;
use Nette\Database\Table\Selection;
use Nette\Object;
use Nette\Database\ForeignKeyConstraintViolationException;

abstract class BaseManager extends Object {

    /** Konstanty pro statusy */
    const
            COLUMN_STATUS = 'status';

    /** @var Context Instance třídy pro práci s databází. */
    protected $database;

    /**
     * Konstruktor s injektovanou třídou pro práci s databází.
     * @param Context $database automaticky injektovaná třída pro práci s databází
     */
    public function __construct(Context $database) {
        $this->database = $database;
    }

    /**
     * Vrátí prvek se všemi informace podle zadaného id
     * @param string   $tableName název tabulky
     * @param string   $idName název idecka pro konrtétní tabulku
     * @param int      $id id hledaného prvku
     * @return IRow    kornétní prvek se všemi infromacemi
     */
    public function getById($tableName, $idName, $id) {
        return $this->database->table($tableName)
                        ->where($idName, $id)
                        ->fetch();
    }

    /**
     * Vrátí všechny řádky se všemi informace podle zadané podmínky
     * @param string   $tableName název tabulky
     * @param string   $idName název idecka pro konrtétní tabulku
     * @param int      $id id hledaného prvku
     * @return Selection    kornétní prvek se všemi infromacemi
     */
    public function getAllRowsById($tableName, $idName, $id) {
        return $this->database->table($tableName)
                        ->where($idName, $id)
                        ->fetchAll();
    }

    /**
     * Vrátí první nalezený řádek se všemi informace podle zadaných podmínek v poli key => value, mezi podmínkami je AND
     * @param string        $tableName název tabulky
     * @param ArrayHash     $condiions název idecka pro konrtétní tabulku
     * @return IRow    kornétní prvek se všemi infromacemi
     */
    public function getRowByMoreConditons($tableName, $condiions) {
        $selection = $this->database->table($tableName);

        foreach ($condiions as $key => $value) {
            $selection->where($key, $value);
        }

        return $selection->fetch();
    }

    /**
     * Vrátí všechny nalezené řádky se všemi informacemi podle zadaných podmínek v poli key => value, mezi podmínkami je AND
     * @param string        $tableName název tabulky
     * @param ArrayHash     $condiions název idecka pro konrtétní tabulku
     * @return Selection    kornétní prvek se všemi infromacemi
     */
    public function getRowsByMoreConditons($tableName, $condiions) {
        $selection = $this->database->table($tableName);

        foreach ($condiions as $key => $value) {
            $selection->where($key, $value);
        }

        return $selection->fetchAll();
    }

    /**
     * Vrátí publikovaný konrétní prvek se všemi informace podle zadaného id
     * @param string   $tableName název tabulky
     * @param string   $idName název idecka pro konrtétní tabulku
     * @param int      $id id hledaného prvku
     * @return IRow    kornétní prvek se všemi infromacemi
     */
    public function getPublishedRowByCondition($tableName, $idName, $id) {
        return $this->database->table($tableName)
                        ->where(self::COLUMN_STATUS, Status::PUBLISHED)
                        ->where($idName, $id)
                        ->fetch();
    }

    /**
     * Vrátí všechny publikované řádky se všemi informace podle zadané podmínky
     * @param string        $tableName název tabulky
     * @param string        $columnName název omezujícího sloupce pro konrtétní tabulku
     * @param int           $columnValue omezující hodnota pro podmínku
     * @return Selection    Všechny rádky splňující základní podmínky - viditelné a omezení
     */
    public function getAllPublishedRowsByCondition($tableName, $columnName, $columnValue) {
        return $this->database->table($tableName)
                        ->where($columnName, $columnValue)
                        ->where(self::COLUMN_STATUS, Status::PUBLISHED)
                        ->fetchAll();
    }

    /**
     * Vrátí všechny  položky z dané tabulky - bez podmínky
     * @param string   $tableName název tabulky
     * @param null|int $limit limit počtu výsledků, defaultně je null - bez limitu
     * @return Selection všechny položky z tabulky, které se na v dané tabulce budou vyskytovat
     */
    public function getAllRows($tableName, $limit = null) {
        return $this->database->table($tableName)
                        ->limit($limit);
    }

    /**
     * Vrátí všechny položky z dané tabulky, které jsou publikované (nejsou skryté ani smazané)
     * @param string   $tableName název tabulky
     * @param null|int $limit limit počtu výsledků, defaultně je null - bez limitu
     * @return Selection všechny položky z tabulky, které se na v systému budou vyskytovat a jsou viditelné
     */
    public function getAllPublished($tableName, $limit = null) {
        return $this->database->table($tableName)
                        ->where(self::COLUMN_STATUS, Status::PUBLISHED)
                        ->limit($limit);
    }

    /**
     * Uloží hodnoty do databáze. Potomci můžou volat metodu ve svých managerech.
     * Testuji, pokud je zadané v hodnotách id, když není použiji insert, pokud je update 
     * @param string   $tableName název tabulky
     * @param string   $idName název idecka pro konrtétní tabulku
     * @param array    $values hodnoty, které mají být vloženy do databáze
     * @return IRow    kornétní prvek se všemi infromacemi
     */
    public function saveValues($tableName, $idName, $values) {
        // Pokud je zadáno ID, updatuji
        if (isset($values[$idName])) {
            // Testuji, jestli ID není prázdný string - v tom případě odstraním ID a vkládám nový záznam
            if ($values[$idName] == '') {
                $values->offsetUnset($idName);
                // Pokud ID zadané není, insertuji
                return $this->database->table($tableName)
                                ->insert($values);
            } else {
                return $this->database->table($tableName)
                                ->where($idName, $values[$idName])
                                ->update($values);
            }
        } else {
            // Pokud ID zadané není, insertuji
            return $this->database->table($tableName)
                            ->insert($values);
        }
    }

    /**
     * Odstraní danou položku úplně z databáze, bez možnosti skrýt.
     * @param string   $tableName název tabulky
     * @param string   $idName název idecka pro konrtétní tabulku
     * @param int      $id id hledaného prvku
     */
    public function remove($tableName, $idName, $id) {
        $this->database->table($tableName)->where($idName, $id)->delete();
    }

    /**
     * Pokusí se odstranit danou položku.
     * Pokud ji ještě nikdo nedal do koše, nastavím status na delete.
     * Pokud ji někdo už do koše dal, smažu ji.
     * Pokud se ji nepodařilo smazat, nastavím ji status na hidden.
     * @param string   $tableName název tabulky
     * @param string   $idName název idecka pro konrtétní tabulku
     * @param int      $id id hledaného prvku
     * @return IRow    kornétní prvek se všemi infromacemi
     */
    public function clean($tableName, $idName, $id) {
        $element = $this->getById($tableName, $idName, $id);

        //Zjistí, jestli se už někdo měnu pokusil smazat - dal ji do koše
        if ($element[self::COLUMN_STATUS] == Status::DELETED) {
            // Odstraní úplně z databáze prvek, pokud jej někdo dal už do koše.
            try {
                // Pokusí se prvek odstranit, pokud není někde navázaný.
                $row = $this->database->table($tableName)
                        ->where($idName, $id)
                        ->delete();
            } catch (ForeignKeyConstraintViolationException $ex) {
                // Jinak ho ponechám jako skrytý
                $row = $this->database->table($tableName)
                        ->where($idName, array($id))
                        ->update(array(self::COLUMN_STATUS => Status::HIDDEN));
            }
        } else {
            // Prvek se nikdo nepokusil smazat, takže mu dám status na smazaný - tím je v koši a potom se může někdo pokusit ji smazat
            $row = $this->database->table($tableName)
                    ->where($idName, $id)
                    ->update(array(self::COLUMN_STATUS => Status::DELETED));
        }

        return $row;
    }

}
