<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\Model;

use App\Model\BaseManager;
use App\Model\UserManager;
use Nette\Database\ForeignKeyConstraintViolationException;
use Nette\Database\Table\ActiveRow;
use Nette\Database\Table\IRow;
use Nette\Utils\ArrayHash;
use Nette\Database\Context;

/**
 * Spravuje osoby v systému.
 * @package App\Model
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class PersonManager extends BaseManager {

    /** Konstanty pro manipulaci s modelem - table person. */
    const
            TABLE_NAME = 'person',
            COLUMN_ID = 'person_id',
            COLUMN_PERSON_DETAIL_ID = 'person_detail_id',
            COLUMN_ADDRESS_ID = 'address_id',
            COLUMN_DELIVERY_ADDRESS_ID = 'delivery_address_id',
            COLUMN_BANK_ACCOUNT_ID = 'bank_account_id',
            COLUMN_USER_ID = 'user_id';

    /** Konstanty pro manipulaci s modelem - table person_detail. */
    const
            TABLE_DETAIL_NAME = 'person_detail',
            COLUMN_FIRST_NAME = 'first_name',
            COLUMN_LAST_NAME = 'last_name',
            COLUMN_COMPANY_NAME = 'company_name',
            COLUMN_TAX_NUMBER = 'tax_number',
            COLUMN_IDENTIFICATION_NUMBER = 'identification_number',
            COLUMN_EMAIL = 'email',
            COLUMN_TOTAL_AMOUNT = 'total_amount';

    /** Konstanty pro manipulaci s modelem - table address. */
    const
            TABLE_ADDRESS_NAME = 'address',
            COLUMN_ADDRESS_STREET = 'street',
            COLUMN_ADDRESS_REGISTRY_NUMBER = 'registry_number',
            COLUMN_ADDRESS_HOUSE_NUMBER = 'house_number',
            COLUMN_ADDRESS_CITY = 'city',
            COLUMN_ADDRESS_ZIP = 'zip';

    /** Konstanty pro manipulaci s modelem - table country. */
    const
            TABLE_COUNTRY_NAME = 'country',
            COLUMN_COUNTRY_ID = 'country_id',
            COLUMN_COUNTRY_TITLE = 'title';

    /** Konstanty pro manipulaci s modelem - table bank_account. */
    const
            TABLE_BANK_ACCOUNT_NAME = 'bank_account';

    /** Konstanty pro manipulaci s modelem - table bank_code. */
    const
            TABLE_BANK_CODE_NAME = 'bank_code',
            COLUMN_BANK_NAME = 'bank_name',
            COLUMN_BANK_CODE = 'bank_code';

    /** @var UserManager Model pro správu uživatelů. */
    private $userManager;

    /**
     * DI setter modelu pro správu uživatelů.
     * @param UserManager $userManager automaticky injectovaná třída modelu pro správu uživatelů
     */
    public function injectUserManager(UserManager $userManager) {
        $this->userManager = $userManager;
    }

    /**
     * Vrátí tabulku jako zdroj pro DataGrid
     * @return Table\Selection
     */
    public function getTable() {
        return $this->database->table(self::TABLE_NAME);
    }

    /**
     * Vrátí ID osoby, aktuálně přiřazené k danému uživatelskému účtu.
     * @param int $userId ID uživatele
     * @return null|int ID osoby nebo null, pokud taková osoba neexistuje
     */
    public function getPersonId($userId) {
        $row = $this->database->table(self::TABLE_NAME)
                        ->where(self::COLUMN_USER_ID, $userId)->fetch();
        return $row ? (int) $row->getPrimary() : null;
    }

    /**
     * Vrací osobu podle jejího ID.
     * @param int $personId ID osoby
     * @return bool|mixed|IRow osobu s daným ID nebo false, pokud osoba sa daným ID neexistuje
     */
    public function getPerson($personId) {
        return $this->database->table(self::TABLE_NAME)->select('*')->where(self::COLUMN_ID, $personId)->fetch();
    }

    /**
     * Vrací ID user podle ID osoby
     * @param int $personId ID osoby
     * @return null|int ID usera nebo null, pokud takový user neexistuje
     */
    public function getUserIdByPersonId($personId) {
        $row = $this->database->table(self::TABLE_NAME)
                        ->where(self::COLUMN_ID, $personId)->fetch();
        return $row ? (int) $row[self::COLUMN_USER_ID] : null;
    }

    /**
     * Sestaví název osoby z jejího jména, příjmení a názvu firmy.
     * @param IRow $person_detail detailní informace o osobě (IRow je radek z tabulky)
     * @return string název osoby
     */
    public static function getName(IRow $person_detail) {
        $name = '';

        if (isset($person_detail[self::COLUMN_FIRST_NAME]))
            $name .= $person_detail[self::COLUMN_FIRST_NAME];

        if (isset($person_detail[self::COLUMN_LAST_NAME]))
            $name .= ' ' . $person_detail[self::COLUMN_LAST_NAME];

        if (isset($person_detail[self::COLUMN_COMPANY_NAME]))
            $name .= ' ' . $person_detail[self::COLUMN_COMPANY_NAME];

        return $name;
    }

    /**
     * Vrací seznam účetních pro výběr - vybírá jen z uživatelů, kteří jsou adminové.
     * @return array pole účetních, kde primární klíc je ID detailu osoby účetního a hodnota je jeho kompletní jméno
     */
    public function getAccountants() {
        $admins = $this->userManager->getAdmins();
        $accountants = array();

        /** @var ActiveRow $admin */
        foreach ($admins as $admin) {
            //spojuji zaznamy z tabuly user pro adminy s udaji z tabulky person
            foreach ($admin->related(self::TABLE_NAME) as $person) {
                /** @var ActiveRow $accountant */
                $accountant = $person->ref(self::TABLE_DETAIL_NAME);
                //takze to je to pole - index je idecko a hodnota je jmeno
                $accountants[$accountant->getPrimary()] = self::getName($accountant);
            }
        }

        return $accountants;
    }

    /**
     * Vrací seznam dodavatelů pro výběr - vybírá jen z uživatelů, kteří jsou adminové.
     * @return array pole dodavatelů, kde primární klíc je ID osoby dodavatele a hodnota je jeho kompletní jméno
     */
    public function getSellers() {
        $admins = $this->userManager->getAdmins();
        $sellers = array();

        /** @var ActiveRow $admin */
        foreach ($admins as $admin) {
            foreach ($admin->related(self::TABLE_NAME) as $seller) {
                //takze to je to pole - index je idecko a hodnota je jmeno
                $sellers[$seller->getPrimary()] = self::getName($seller->ref(self::TABLE_DETAIL_NAME));
            }
        }

        return $sellers;
    }

    /**
     * Vrátí státy - určené pro formuláře.
     * @return array státy
     */
    public function getCountries() {
        return $this->database->table(self::TABLE_COUNTRY_NAME)
                        ->select(self::COLUMN_COUNTRY_ID . ', ' . self::COLUMN_COUNTRY_TITLE)
                        ->order(self::COLUMN_COUNTRY_TITLE)
                        ->fetchPairs(self::COLUMN_COUNTRY_ID, self::COLUMN_COUNTRY_TITLE);
    }

    /**
     * Vrátí informace o bance s daným kódem - kod je uložen u čísla účtu konkrétního uživatele (lépe řečeno uživatel ma idecko uctu).
     * @param int $bankCode kód banky
     * @return bool|mixed|IRow informace o bance s daným kódem
     */
    public function getBank($bankCode) {
        return $this->database->table(self::TABLE_BANK_CODE_NAME)->where(self::COLUMN_BANK_CODE, $bankCode)->fetch();
    }

    /**
     * Vrátí celkou utracenou částku zákazníka.
     * V tabulce person najdu podle zadaného userID utracenou částku - sleva pro registrované uživatele. 
     * Uchovávám v person místo user, protože v případě použití adminu bez eshopu, by tato hodnota byla v user zbytečná viz setTotalAmount
     * @param int $userID id uživatele
     * @return int celková utracená částka
     */
    public function getUserTotalAmount($userID) {
        // V return se musím odkázat na total_amount, jinak to vrátí active row
        $amount = $this->database->table(self::TABLE_NAME)
                ->select(self::COLUMN_TOTAL_AMOUNT)
                ->where(self::COLUMN_USER_ID, $userID)
                ->fetch();

        return $amount[self::COLUMN_TOTAL_AMOUNT];
    }

    /**
     * Změní celkovou utracenou částku dané osoby v e-shopu.
     * Částku si pamatuji pro využívání slevového systému pro zákanzíky - určím, v jaké slevové hladině se nachází.
     * @param int    $personId id osoby (person_id)
     * @param int    $amount  utracená částka
     * @param string $operation operace, která se bude při změně celkové utracené částky provádět
     */
    public function setTotalAmount($personId, $amount, $operation) {
        $this->database->table(self::TABLE_NAME)
                ->where(self::COLUMN_ID, $personId)
                ->update([self::COLUMN_TOTAL_AMOUNT => Context::literal(self::COLUMN_TOTAL_AMOUNT . $operation . '?', $amount)]);
    }

    /**
     * Uloží osobu (vytvoří novou nebo upravuje existující podle toho, zda je zadaný klíč person_id).
     * Jednotlivé součásti osoby jsou vždy vytvořené znovu a staré jsou vymazány, pokud nejsou použity v existujících
     * objednávkách.
     * @param array|ArrayHash      $personDetail    detail osoby
     * @param array|ArrayHash      $address         adresa osoby
     * @param null|int             $personId        ID osoby
     * @param null|array|ArrayHash $deliveryAddress dodací adresa osoby
     * @param null|array|ArrayHash $backAccount     platební údaje
     * @param null|int             $userId          ID uživatele, ke kterému se má vytvořená osoba přiřadit
     * @return null|int ID osoby nebo null, pokud se nepovedlo vytvořit novou osobu
     */
    public function savePerson($personDetail, $address, $personId = null, $deliveryAddress = null, $backAccount = null, $userId = null) {
        // Reprezentace kompletní osoby
        $person = new ArrayHash();

        /*
         * Nastavení určitých hodnot na null, protože z formuláře při nevyplnění položky dorazí prázdný string
         * ale je potřeba v takovýchto hodnotách explicitně null. 
         */
        if (empty($personDetail[self::COLUMN_COMPANY_NAME])) {
            $personDetail[self::COLUMN_COMPANY_NAME] = null;
        }
        if (empty($personDetail[self::COLUMN_FIRST_NAME])) {
            $personDetail[self::COLUMN_FIRST_NAME] = null;
        }
        if (empty($personDetail[self::COLUMN_LAST_NAME])) {
            $personDetail[self::COLUMN_LAST_NAME] = null;
        }
        if (empty($personDetail[self::COLUMN_IDENTIFICATION_NUMBER])) {
            $personDetail[self::COLUMN_IDENTIFICATION_NUMBER] = null;
        }
        if (empty($address[self::COLUMN_ADDRESS_HOUSE_NUMBER])) {
            $address[self::COLUMN_ADDRESS_HOUSE_NUMBER] = null;
        }

        /*
         * v metodě se vol8 několik databázových dotazů, její obsah se provede v transakci. 
         * CProběhne buď kompletně nebo vůbec, jinak by v databázi zbyly "zombie" řádky
         */
        $this->database->beginTransaction();

        // Detail.
        $row = $this->database->table(self::TABLE_DETAIL_NAME)->insert($personDetail);
        $person[self::COLUMN_PERSON_DETAIL_ID] = $row ? $row->getPrimary() : null;

        // Adresa.
        $row = $this->database->table(self::TABLE_ADDRESS_NAME)->insert($address);
        $person[self::COLUMN_ADDRESS_ID] = $row ? $row->getPrimary() : null;

        // Dodací adresa, pokud je vyplněna.
        if (isset($deliveryAddress)) {
            $row = $this->database->table(self::TABLE_ADDRESS_NAME)->insert($deliveryAddress);
            $person[self::COLUMN_DELIVERY_ADDRESS_ID] = $row ? $row->getPrimary() : null;
        }

        // Platební údaje.
        if (isset($backAccount)) { // Pokud je účet vyplněný, vložíme nový.
            $row = $this->database->table(self::TABLE_BANK_ACCOUNT_NAME)->insert($backAccount);
            $person[self::COLUMN_BANK_ACCOUNT_ID] = $row ? $row->getPrimary() : null;
        }

        // Osoba.
        if (is_null($personId)) {
            // Vkládá novou
            $row = $this->database->table(self::TABLE_NAME)->insert($person);
            $personId = $row ? $row[self::COLUMN_ID] : null;

            // Přiřazení osoby k určitému účtu.
            if ($userId) {
                $this->database->table(self::TABLE_NAME)
                        ->where(self::COLUMN_ID, $personId)
                        ->update([self::COLUMN_USER_ID => $userId]);
            }
        } else {
            // Upravuje existující
            // Načtení původní osoby.
            $oldPerson = $this->database->table(self::TABLE_NAME)
                            ->where(self::COLUMN_ID, $personId)->fetch();

            // Uložení nové osoby.
            $this->database->table(self::TABLE_NAME)
                    ->where(self::COLUMN_ID, $personId)->update($person);

            // Vyčištění původních záznamů.
            try {
                // Odstranění starého detailu.
                $this->database->table(self::TABLE_DETAIL_NAME)
                        ->where(self::COLUMN_PERSON_DETAIL_ID, $oldPerson[self::COLUMN_PERSON_DETAIL_ID])
                        ->delete();
                // Odstranění starých adres.
                $this->database->table(self::TABLE_ADDRESS_NAME)
                        ->where(self::COLUMN_ADDRESS_ID, [$oldPerson[self::COLUMN_ADDRESS_ID], $oldPerson[self::COLUMN_DELIVERY_ADDRESS_ID]])
                        ->delete();
                // Odstranění starého platebního údaje.
                $this->database->table(self::TABLE_BANK_ACCOUNT_NAME)
                        ->where(self::COLUMN_BANK_ACCOUNT_ID, $oldPerson[self::COLUMN_BANK_ACCOUNT_ID])
                        ->delete();
            } catch (ForeignKeyConstraintViolationException $ex) {
                // Položku se nepodařilo odstranit, jelikož je napojená na objednávky.
            }
        }

        $this->database->commit();

        return $personId;
    }

    /**
     * Odstraní danou osobu.
     * @param int $personId ID osoby
     */
    public function deletePerson($personId) {
        $this->database->table(self::TABLE_NAME)
                ->where(self::COLUMN_ID, $personId)
                ->delete();
    }

}
