<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\Model;

use Nette\Database\Table\ActiveRow;
use Nette\Security\AuthenticationException;
use Nette\Security\IAuthenticator;
use Nette\Security\Identity;
use Nette\Security\Passwords;
use Nette\Database\UniqueConstraintViolationException;

/**
 * Správce uživatelů systému.
 * 
 * @package App\EshopModule\Model
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class UserManager extends BaseManager implements IAuthenticator {

    const
            TABLE_NAME = 'user',
            COLUMN_ID = 'user_id',
            COLUMN_NAME = 'username',
            COLUMN_PASSWORD_HASH = 'password',
            COLUMN_OLD_PASSWORD = 'old_password',
            COLUMN_ROLE = 'role',
            COLUMN_FACEBOOK_ID = 'facebook_id',
            COLUMN_FACEBOOK_TOKEN = 'facebook_token';

    /**
     * Vrátí administrátory.
     * @return array administrátoři
     */
    public function getAdmins() {
        return $this->database->table(self::TABLE_NAME)->where(self::COLUMN_ROLE, 'admin')->fetchAll();
    }

    /**
     * @param $id
     * @return bool|mixed|\Nette\Database\Table\IRow
     */
    public function getUser($id) {
        return $this->database->table(self::TABLE_NAME)->where(self::COLUMN_ID, $id)->fetch();
    }

    /**
     * @param $facebookId
     * @return bool|mixed|\Nette\Database\Table\IRow
     */
    public function findByFacebookId($facebookId) {
        return $this->database->table(self::TABLE_NAME)
                        ->where(self::COLUMN_FACEBOOK_ID, $facebookId)
                        ->fetch();
    }

    public function findByEmail($email) {
        return $this->database->table(self::TABLE_NAME)->where(self::COLUMN_NAME, $email);
    }

    /**
     * @param $facebookId
     * @param $facebookAccessToken
     * @return int
     */
    public function updateFacebookAccessToken($facebookId, $facebookAccessToken) {
        return $this->database->table(self::TABLE_NAME)->where(self::COLUMN_FACEBOOK_ID, $facebookId)->update([self::COLUMN_FACEBOOK_TOKEN => $facebookAccessToken]);
    }

    /**
     * Změní email uživatele.
     * @param int $userId ID uživatele
     * @param string $email nový uživatelský email
     */
    public function changeUserEmail($userId, $email) {
        try {
            $this->database->table(self::TABLE_NAME)->where(self::COLUMN_ID, $userId)->update([self::COLUMN_NAME => $email]);
        } catch (UniqueConstraintViolationException $ex) {
            throw $ex;
        }
    }

    /**
     * Změní uživatelské heslo
     * @param int $values hodnoty z formuláře - stávající heslo, nové heslo
     * @param string $password heslo
     * @return ActiveRow|null
     * @throws AuthenticationException Jestliže došlo k chybě při ověřování, např. špatné heslo nebo uživatelské id.
     */
    public function changePassword($values) {

        /** @var ActiveRow $user */
        $user = $this->database->table(self::TABLE_NAME)
                ->where(self::COLUMN_ID, $values[self::COLUMN_ID])
                ->fetch();

        // Ověření uživatele.
        if (!$user) {
            // Vyhodí výjimku, pokud uživatel neexituje.
            throw new AuthenticationException('Hledaný uživatel neexistuje.', self::IDENTITY_NOT_FOUND);
        }

        // Ověří heslo.
        if (!Passwords::verify($values[self::COLUMN_OLD_PASSWORD], $user[self::COLUMN_PASSWORD_HASH])) {
            // Vyhodí výjimku, pokud je heslo špatně.
            throw new AuthenticationException('Původní heslo nebylo zadáno správně, zkuste to znovu.', self::INVALID_CREDENTIAL);
        } else {
            // Pokud bylo heslo ověřeno - změní uživateli heslo
            $row = $this->database->table(self::TABLE_NAME)
                    ->where(self::COLUMN_ID, $user[self::COLUMN_ID])
                    ->update(array(self::COLUMN_PASSWORD_HASH => Passwords::hash($values[self::COLUMN_PASSWORD_HASH])));
            return $row ? $row : null;
        }
    }

    /**
     * @param $facebookId
     * @param $userInfo
     * @param $userPhoto
     * @return bool|int|\Nette\Database\Table\IRow
     * @todo Naplnit daty personDetail
     */
    public function registerFromFacebook($facebookId, $userInfo, $userPhoto) {
        return $this->database->table(self::TABLE_NAME)->insert([
                    self::COLUMN_ROLE => 'member',
                    self::COLUMN_FACEBOOK_ID => $facebookId
        ]);
    }

    /**
     * Registruje nového uživatele do systému.
     * @param string $email uživatelský email
     * @param string $password heslo
     * @throws DuplicateNameException Jestliže uživatel s daným jménem již existuje.
     * @return ActiveRow|null
     */
    public function register($email, $password) {
        try {
            // Vloží nového uživatele do databáze.
            $row = $this->database->table(self::TABLE_NAME)->insert(array(
                self::COLUMN_NAME => $email,
                self::COLUMN_PASSWORD_HASH => Passwords::hash($password),
            ));
            return $row ? $row->getPrimary() : null;
        } catch (UniqueConstraintViolationException $ex) {
            throw $ex;
        }
    }

    /**
     * Přihlásí uživatele do systému - přihlášení pomocí emaiu, který je uložen jako username.
     * @param array $credentials jméno a heslo uživatele
     * @return Identity identitu přihlášeného uživatele pro další manipulaci
     * @throws AuthenticationException Jestliže došlo k chybě při prihlašování, např. špatné heslo nebo uživatelské jméno.
     */
    public function authenticate(array $credentials) {
        // Extrahuje potřebné parametry.
        list($email, $password) = $credentials;
        // Vykoná dotaz nad databází a vrátí první řádek výsledku nebo false, pokud uživatel neexistuje.
        /** @var ActiveRow $user */
        $user = $this->database->table(self::TABLE_NAME)
                ->where(self::COLUMN_NAME, $email)
                ->fetch();

        // Ověření uživatele.
        if (!$user) {
            // Vyhodí výjimku, pokud uživatel neexituje.
            throw new AuthenticationException('Zadané uživatelský email neexistuje.', self::IDENTITY_NOT_FOUND);
        } elseif (!Passwords::verify($password, $user[self::COLUMN_PASSWORD_HASH])) { // Ověří heslo.
            // Vyhodí výjimku, pokud je heslo špatně.
            throw new AuthenticationException('Zadané heslo není správně.', self::INVALID_CREDENTIAL);
        } elseif (Passwords::needsRehash($user[self::COLUMN_PASSWORD_HASH])) { // Zjistí, jestli heslo potřebuje rehashovat.
            // Rehashuje heslo.
            $user->update(array(self::COLUMN_PASSWORD_HASH => Passwords::hash($password)));
        }

        // Příprava uživatelských dat.
        $userData = $user->toArray(); // Extrahuje uživatelská data.
        unset($userData[self::COLUMN_PASSWORD_HASH]); // Odstraní položku hesla z uživatelských dat (kvůli bezpečnosti).
        // Vrátí novou identitu přihlášeného uživatele.
        return new Identity($user[self::COLUMN_ID], $user[self::COLUMN_ROLE], $userData);
    }

}
