<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\BlogModule\Components;

use App\BlogModule\Model\ArticleManager;
use App\BlogModule\Model\CategoryManager;
use App\Forms\BaseFormFactory;
use App\Model\Enum\MessageType;
use App\Model\UserManager;
use Nette\Application\UI\Control;
use Nette\Application\UI\Form;
use Nette\Database\UniqueConstraintViolationException;
use Nette\Utils\ArrayHash;
use Nette\Utils\DateTime;

/**
 * Komponenta pro obnovení zapomenutého hesla
 * @package App\BlogModule\Components
 *
 * @author Petr Vojáček <petr at ideatech.cz>
 */
class ArticleFormControl extends Control {

    /** Cesta k souborům šablon pro tuto komponentu. */
    const TEMPLATE_PATH = '/../templates/components/ArticleFormControl/';

    /**
     * @var UserManager
     */
    private $userManager;

    /**
     * @var CategoryManager
     */
    private $categoryManager;

    /**
     * @var ArticleManager
     */
    private $articleManager;

    /**
     * @var BaseFormFactory
     */
    private $baseFormFactory;

    /**
     * ArticleFormControl constructor.
     * @param UserManager $userManager
     * @param CategoryManager $categoryManager
     * @param ArticleManager $articleManager
     * @param BaseFormFactory $baseFormFactory
     */
    public function __construct(UserManager $userManager, CategoryManager $categoryManager, ArticleManager $articleManager, BaseFormFactory $baseFormFactory) {
        parent::__construct();
        $this->userManager = $userManager;
        $this->categoryManager = $categoryManager;
        $this->articleManager = $articleManager;
        $this->baseFormFactory = $baseFormFactory;
    }

    /**
     * Výchozí metoda pro vykreslení komponenty
     */
    public function render($formPath = null) {
        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE_PATH . 'form.latte');
        $this->template->formPath = $formPath;
        // Vykreslí komponentu.
        $this->template->render();
    }

    public function createComponentArticleForm() {
        $form = $this->baseFormFactory->create();
        $form->addHidden('article_id');
        $form->addText('title', 'blog.article.form.title')->setRequired();
        $form->addText('url', 'blog.article.form.url')->setRequired();

        /* $form->addDate('date', 'blog.article.form.date', 'Y-m-d')
          ->setRequired('blog.article.flashes.missing-date')
          ->getControlPrototype()
          ->addClass('form-control'); */

        $form->addText('description', 'blog.article.form.description')->setRequired();
        $form->addTextArea('ingredients', 'blog.article.form.ingredients');
        $form->addTextArea('workflow', 'blog.article.form.workflow');

        $form->addMultiSelect('categories', 'blog.article.form.categories', $this->categoryManager->getCategoryLeafs())
                ->setRequired()
                ->getControlPrototype()
                ->addClass('form-control');

        $form->addText('seo_title', 'blog.article.form.meta-title')
                ->getControlPrototype()
                ->addClass('form-control');

        $form->addText('seo_description', 'blog.article.form.meta-description')
                ->getControlPrototype()
                ->addClass('form-control');

        $form->addText('seo_keywords', 'blog.article.form.meta-keywords')
                ->getControlPrototype()
                ->addClass('form-control');


        $form->addSubmit('submit', 'blog.article.form.submit');
        $form->onSuccess[] = function (Form $form, ArrayHash $values) {
            $this->succeeded($form, $values);
        };
        return $form;
    }

    public function succeeded(Form $form, ArrayHash $values) {
        $presenter = $form->getPresenter();
        try {
            $categories = $values->categories;
            $values->offsetUnset('categories');
            if (isset($values->id)) {
                $values->offsetSet('date', new DateTime());
                // Zaznamení akce do DB
                $presenter->logAction('Nový příspěvek na blogu - ' . $values[ArticleManager::COLUMN_URL]);
            } else {
                // Zaznamení akce do DB
                $presenter->logAction('Editace příspěvku v blogu - ' . $values[ArticleManager::COLUMN_URL]);
            }
            $articleId = $this->articleManager->saveArticle($values);
            $this->categoryManager->updateArticleCategories($articleId, $categories);
            $presenter->flashMessage($this->presenter->translator->translate('blog.article.flashes.saved-success'), MessageType::SUCCESS);
            $presenter->redirect('this');
        } catch (UniqueConstraintViolationException $ex) {
            $presenter->flashMessage($this->presenter->translator->translate('blog.article.flashes.constraint-violation'), MessageType::ERROR);
        }
    }

    public function setDefaults(array $values) {
        $this['articleForm']->setDefaults($values);
    }

}

/**
 * Interface IArticleFormFactory
 * @package App\BlogModule\Components
 */
interface IArticleFormFactory {

    /**
     * @return ArticleFormControl
     */
    public function create();
}
