<?php

/**
 * Třída poskytuje metody pro správu článků v redakčním systému.
 * @package App\BlogModule\Model
 *
 * @author Jan Vodák <vodak at ideatech.cz>
 */

namespace App\BlogModule\Model;

use App\Model\BaseManager;
use Nette\Database\Table\IRow;
use Nette\Database\Table\Selection;
use Nette\Utils\ArrayHash;
use Nette\Database\Context;


class ArticleManager extends BaseManager
{
    /** Konstanty pro manipulaci s modelem. */
    const
        TABLE_NAME = 'blog_article',
        COLUMN_ID = 'article_id',
        COLUMN_IMAGE_COUNT = 'images_count',
        COLUMN_URL = 'url';

    /**
     * ArticleManager constructor.
     * @param Context $database
     */
    public function __construct(Context $database)
    {
        parent::__construct($database);
    }

    /**
     * Vrátí tabulku jako zdroj pro DataGrid
     */
    public function getTable()
    {
        return $this->database->table(self::TABLE_NAME);
    }

    /**
     * @param null $category_url
     * @return \Nette\Database\ResultSet
     */
    public function getArticles($category_url = null)
    {
        if ($category_url) {
            return $this->database->query('SELECT blog_article.* FROM blog_article 
              LEFT JOIN blog_article_blog_category ON blog_article.article_id = blog_article_blog_category.article_id 
              LEFT JOIN blog_category ON blog_article_blog_category.category_id = blog_category.category_id 
              WHERE blog_category.url = ? ORDER BY blog_article.article_id DESC', $category_url);
        }
        return $this->database->table(self::TABLE_NAME)->order(self::COLUMN_ID . ' DESC');
    }

    /**
     * @param $category_id
     * @return \Nette\Database\ResultSet
     */
    public function getArticlesByCategoryId($category_id, $order = 'DESC()', $limit = 3)
    {
        return $this->database->query('SELECT blog_article.* FROM blog_article 
              LEFT JOIN blog_article_blog_category ON blog_article.article_id = blog_article_blog_category.article_id 
              LEFT JOIN blog_category ON blog_article_blog_category.category_id = blog_category.category_id 
              WHERE blog_category.category_id = ? ORDER BY ? LIMIT ?', $category_id, $order, $limit);
    }

    /**
     * @param int $limit
     * @return static
     */
    public function getRandomArticles($limit = 3)
    {
        return $this->database->table(self::TABLE_NAME)->order('RAND()')->limit($limit);
    }

    /**
     * @return bool|mixed|IRow
     */
    public function getLastArticle()
    {
        return $this->database->table(self::TABLE_NAME)->order(self::COLUMN_ID . ' DESC')->limit(1)->fetch();
    }

    /**
     * Vrátí článek z databáze podle jeho URL.
     * @param string $url URl článku
     * @return bool|mixed|IRow první článek, který odpovídá URL nebo false při neúspěchu
     */
    public function getArticle($url)
    {
        return $this->database->table(self::TABLE_NAME)->where(self::COLUMN_URL, $url)->fetch();
    }

    /**
     * Uloží článek do systému. Pokud není nastaveno ID, vloží nový, jinak provede editaci.
     * @param array|ArrayHash $article článek
     * @return int  id clanku
     */
    public function saveArticle($article)
    {
        if (!$article[self::COLUMN_ID] || $article[self::COLUMN_ID] == '') {
            unset($article[self::COLUMN_ID]);
            return $this->database->table(self::TABLE_NAME)->insert($article);
        } else {
            $this->database->table(self::TABLE_NAME)->where(self::COLUMN_ID, $article[self::COLUMN_ID])->update($article);
            return $article[self::COLUMN_ID];
        }
    }

    /**
     * Odstraní článek.
     * @param string $url URL článku
     */
    public function removeArticle($url)
    {
        $article = $this->getArticle($url);
        $this->database->table(self::TABLE_NAME)->where(self::COLUMN_URL, $url)->delete();
    }

    /**
     * Vrátí článek podle ID.
     * @param int $articleId - ID článku
     * @param null|string $columns sloupce pro SELECT, nebo null pro všechny sloupce
     * @return bool|mixed|IRow článek
     */
    public function getArticleFromId($articleId, $columns = null)
    {
        return $this->database->table(self::TABLE_NAME)
            ->select($columns ? $columns : '*')
            ->where(self::COLUMN_ID, array($articleId))->fetch();
    }


}
