<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\BlogModule\Model;

use App\Model\BaseManager;
use Nette\Database\Table\IRow;
use Nette\Database\UniqueConstraintViolationException;

/**
 * Správce kategorií pro články.
 * Uživatel zařazuje článek do kategorií - vybírá je v multiple selectu
 * Odtud přijde pole id kategorí, ja je jen nahraji do spojovací tabulky
 * @package App\EshopModule\Model
 *
 * @author Jan Vodák <vodak at ideatech.cz>
 */
class CategoryManager extends BaseManager {

    /** Konstanty pro manipulaci s modelem. */
    const
            TABLE_NAME = 'blog_category',
            COLUMN_ID = 'category_id',
            COLUMN_TITLE = 'title',
            COLUMN_PARENT = 'parent_category_id',
            COLUMN_HIDDEN = 'hidden',
            COLUMN_ORDER = 'order_no',
            COLUMN_URL = 'url';

    /**
     * Vrátí všechny kategorie článků.
     * @return array kategorie článků
     */
    public function getAllCategories() {
        return $this->database->table(self::TABLE_NAME)->order(self::COLUMN_ORDER)->fetchAll();
    }

    /**
     * Vrátí kategorii podle její URL.
     * @param string $url URL kategorie
     * @return bool|mixed|IRow kategorie
     */
    public function getCategory($url) {
        return $this->database->table(self::TABLE_NAME)
                        ->where(self::COLUMN_URL, $url)->fetch();
    }

    /**
     * Vrátí ty kategorie, které v sobě již neobsahují další podkategorie (listy stromu kategorií).
     * Slouží pro výpis v administraci, kam se vlastně dá zařadit produkt - do listů
     * To jsou ty kategorie, jejichž ID se neobjevuje jako hodnota ve sloupci parent_category_id u žádné kategorie v tabulce
     * @return array kategorie, které v sobě již neobsahují další podkategorie (listy stromu kategorií)
     */
    public function getCategoryLeafs() {
        return $this->database->table(self::TABLE_NAME)
                        ->select(self::COLUMN_ID . ', ' . self::COLUMN_TITLE)
                        ->where(self::COLUMN_ID . ' NOT',
                                // Vnořený SELECT.
                                $this->database->table(self::TABLE_NAME)
                                ->select(self::COLUMN_PARENT . ' AS ' . self::COLUMN_ID)
                                ->where(self::COLUMN_PARENT . ' IS NOT NULL')
                        )->fetchPairs(self::COLUMN_ID, self::COLUMN_TITLE);
    }

    /**
     * Vrátí ID kategorií, ve kterých je zařazený daný článek - pro výpis článku.
     * @param int $articleId ID článku
     * @return array ID kategorií, ve kterých je zařazený daný článek
     */
    public function getArticleCategories($articleId) {
        return $this->database->table(ArticleManager::TABLE_NAME . '_' . self::TABLE_NAME)
                        ->select(self::COLUMN_ID)
                        ->where(ArticleManager::COLUMN_ID, array($articleId))
                        ->fetchPairs(null, self::COLUMN_ID);
    }

    /**
     * Vrátí URL adresu první kategorie, do které článek spadá.
     * @param int $articleId ID článku
     * @return string URL adresa první kategorie, do které článek spadá nebo prázdný string, pokud nespadá do žádné
     */
    public function getArticleCategory($articleId) {
        if ($row = $this->database->table(ArticleManager::TABLE_NAME . '_' . self::TABLE_NAME)
                        ->where(ArticleManager::COLUMN_ID, $articleId)->fetch()
        )
            return $row->ref(self::TABLE_NAME)->url;
        else
            return '';
    }

    /**
     * Vrátí kategorie článků v podobě stromu.
     * @param bool $showAll zda chceme zobrazovat i skryté kategorie pro admina, nebo obyč menu bez skrytých
     * @return array kategorie článků v podobě stromu
     */
    public function getCategories($showAll = false) {
        // Získá seznam kategorií z databáze.
        $categories = $this->database->table(self::TABLE_NAME);

        // Schová skyté kategorie.
        if (!$showAll)
            $categories->where('NOT ' . self::COLUMN_HIDDEN);

        // Seřadí výsledek.
        $categories->order(self::COLUMN_ORDER);

        // Vratí výsledný strom.
        return $this->formatTree($categories->fetchAll(), null);
    }

    /**
     * Aktualizuje zařazení článku do kategorií.
     * @param int $articleId    ID článku, který zařazujeme
     * @param array $categories ID kategorií, do kterých má být článek zařazen
     */
    public function updateArticleCategories($articleId, $categories) {
        $this->database->table(ArticleManager::TABLE_NAME . '_' . self::TABLE_NAME)
                ->where(ArticleManager::COLUMN_ID, array($articleId))->delete();

        $rows = array();

        foreach ($categories as $category) {
            $rows[] = array(
                ArticleManager::COLUMN_ID => $articleId,
                self::COLUMN_ID => $category,
            );
        }

        $this->database->table(ArticleManager::TABLE_NAME . '_' . self::TABLE_NAME)->insert($rows);
    }

    /**
     * Metoda rekurzivně naformátuje kategorie z databáze - výsledné pole bude mít stromovou strukturu 
     * V šabloně bude rekurzivně vypsáno pomocí komponent.  Stromovou strukturu pole - vnoření do sebe. 
     * Klíčem je vždy ID kategorie, hodnotou je pole s daty kategorie. Pro podkategorie vytvoříme vždy klíč "subcategories".
     * @param array $categories kategorie pro zformátování - pole všech položek z DB
     * @param int   $parentId   ID rodičovské kategorie
     * @return array kategorie ve stromové podobě
     */
    private function formatTree($categories, $parentId) {
        // Prázdný strom
        $tree = array();

        // Pokusí se najít položky, které patří do rodičovské kategorie ($parentId)
        foreach ($categories as $category) {
            if ($category->parent_category_id == $parentId) {
                // Položku se přidá do nového stromu a rekurzivně se přidá strom podpoložek
                $tree[$category->category_id] = [
                    'category' => $category,
                    'subcategories' => $this->formatTree($categories, $category->category_id)
                ];
            }
        }
        // Vrátíme hotový strom
        return $tree;
    }

    /**
     * Aktualizuje kategorie článků.
     * 
     * Nejjednodušším způsobem uložení změn v kategoriích je všechny kategorie z tabulky odstranit a znovu je vložit
     * Kvůli existenci cizího klíče by odstranění kategorií vyvolalo i odstranění článků do nich vložených.
     * Proto se cizí klíč nejdřív odebere a po provedení změn zase vrátí.
     * 
     * Vymazání a vložení kategorií se provede v tzv. transakci. Pokud se cokoli nepovede 
     * (např. uživatel zadá 2 stejná URL a vyvolá se výjimka), zavoláním rollBack() se vrátí databázi do původního stavu. 
     * Poté se ještě vrátí cizí klíč. To se díky bloku finnaly provede vždy, ať už operace dopadla úspěšně nebo ne.
     * 
     * @param array $categories nové kategorie produktů
     * @return bool true pokud byla operace úspěšná, jinak false
     * @throws UniqueConstraintViolationException Jestliže uživatel zadá dvě kategorie se stejnou URL.
     */
    public function saveCategories($categories) {
        // Zbavíme se cizího klíče, aby odstranění kategorií nevyvolalo i odstranění navázaných produktů.
        $this->database->query('ALTER TABLE '.ArticleManager::TABLE_NAME . '_' . self::TABLE_NAME.' DROP FOREIGN KEY '.ArticleManager::TABLE_NAME . '_' . self::TABLE_NAME . '_ibfk_1');

        // Započneme databázovou transakci.
        $this->database->beginTransaction();

        // Smaže všechny předešlé kategorie.
        $this->database->table(self::TABLE_NAME)->delete();

        try {
            // Vložíme nové kategorie.
            if (($result = (bool) $this->database->table(self::TABLE_NAME)->insert(array_map(function ($category) {
                    return array(
                        self::COLUMN_ID => $category[self::COLUMN_ID],
                        self::COLUMN_URL => $category[self::COLUMN_URL],
                        self::COLUMN_TITLE => $category[self::COLUMN_TITLE],
                        self::COLUMN_ORDER => $category[self::COLUMN_ORDER],
                        self::COLUMN_HIDDEN => $category[self::COLUMN_HIDDEN],
                        self::COLUMN_PARENT => isset($category[self::COLUMN_PARENT]) ? $category[self::COLUMN_PARENT] : null
                    );
                }, $categories)))
            ) {
                // Uplatníme změny v transakci a tím ji ukončíme.
                $this->database->commit();

                // Produkty, jejichž kategorie nyní neexistuje, přesuneme do kategorie nezařazeno.
                $this->database->table(ArticleManager::TABLE_NAME . '_' . self::TABLE_NAME)
                        ->where('NOT ' . self::COLUMN_ID, $this->database->table(self::TABLE_NAME)->select(self::COLUMN_ID))
                        ->update([self::COLUMN_ID => $this->database->table(self::TABLE_NAME)
                            ->where(self::COLUMN_URL, 'nezarazeno')
                            ->where(self::COLUMN_HIDDEN)
                            ->fetch()[self::COLUMN_ID]]);
            } else {
                // Vrátíme změny v transakci a tím ji ukončíme.
                $this->database->rollBack();
            }

            // Vrátíme výsledek operace.
            return $result;
        } catch (UniqueConstraintViolationException $ex) {
            // Vrátíme změny v transakci a tím ji ukončíme.
            $this->database->rollBack();
            throw $ex;
        } finally {
            // Při úspěchu i neúspěchu vrátíme cizí klíč.
            $this->database->query('
                        ALTER TABLE '.ArticleManager::TABLE_NAME . '_' . self::TABLE_NAME.'
                        ADD CONSTRAINT '.ArticleManager::TABLE_NAME . '_' . self::TABLE_NAME . '_ibfk_1 FOREIGN KEY ('.self::COLUMN_ID.') REFERENCES '.self::TABLE_NAME.'('.self::COLUMN_ID.')
                        ON DELETE CASCADE ON UPDATE CASCADE');
        }
    }

}
