<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\CmsModule\Components;

use App\Model\Enum\MessageType;
use Ublaboo\DataGrid\DataGrid;
use Nette\Application\UI\Control;
use App\CmsModule\Model\PageManager;
use Nette\Database\ForeignKeyConstraintViolationException;

/**
 * Komponenta datagridu pro správu stránek
 * @package App\CmsModule\Components
 *
 * @author Petr Vojáček <petr at ideatech.cz>
 */
class PageDataGridControl extends Control {

    /** Cesta k souboru šablony pro tuto komponentu. */
    const TEMPLATE = '/../../../templates/components/DataGrids/dataGrid.latte';

    /** @var PageManager Instance třídy modelu pro práci se stránkami. */
    private $pageManager;

    /**
     * Konstruktor s injectovanými třídami modelu pro práci se stránkami
     * @param PageManager     $pageManager   automaticky injektovaná třída modelu pro práci se stránkami
     */
    public function __construct(PageManager $pageManager) {
        parent::__construct();
        $this->pageManager = $pageManager;
    }

    /** Vykreslí komponentu formuláře pro správu osob. */
    public function render() {
        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE);

        // Vykreslí komponentu.
        $this->template->render();
    }

    public function createComponentDataGrid($name) {
        $grid = new DataGrid($this, $name);
        $grid->setTranslator($this->presenter->translator);

        $grid->setDataSource($this->pageManager->getDatagridSource());
        $grid->setSortable();

        /**
         * First parameter is a callback that will return particular children rows for given parent
         * Second is a column name (or callback), that indicates whether the row has some children or not
         */
        $grid->setTreeView(
                [$this, 'getChildren'], [$this, 'hasChildren']);

        $grid->setSortableHandler('pageDataGrid:sort!');
        $grid->addColumnNumber('position', 'page.page.datagrid.column-position', 'position')->setDefaultHide(true);

        $grid->setDefaultSort(['position' => 'DESC']);
        $grid->addColumnText('title', 'page.page.datagrid.column-name')
                ->setSortable();

        /**
         * Odkaz na editaci
         */
        $grid->addAction('edit', '', ':Cms:Page:edit')
                ->setIcon('pencil');

        /**
         * Mazani
         */
        $grid->addAction('delete', '', 'deletePage!')
                ->setIcon('trash')
                ->setTitle('page.page.datagrid.action-delete')
                ->setClass('btn btn-xs btn-danger ajax')
                ->setConfirm('page.page.datagrid.action-delete-confirm', 'title');

        return $grid;
    }

    public function positionEdited($id, $value) {
        $this->pageManager->savePage([
            'id' => $id,
            'position' => $value
        ]);
    }

    public function handleDeletePage($id) {
        $presenter = $this->getPresenter();
        
        try{
            $this->pageManager->removePage($id);
            // Zaznamení akce do DB
            $presenter->logAction('Smazání stránky s id - ' . $id);
            $presenter->flashMessage($this->presenter->translator->translate('page.page.flashes.deleted-success', ['id' => $id]), MessageType::SUCCESS);
        }  catch (ForeignKeyConstraintViolationException $ex){
            $presenter->flashMessage($this->presenter->translator->translate('page.page.flashes.deleted-error', ['id' => $id]), MessageType::ERROR);
        }
        
        if ($presenter->isAjax()) {
            $presenter->redrawControl("ajaxNotification");
            $presenter->redrawControl('pageSnippet');
        } else {
            $presenter->redirect('this');
        }
    }

    public function handleSort($item_id, $prev_id, $next_id) {
        $presenter = $this->getPresenter();
        $presenter->flashMessage(
                $this->presenter->translator->translate('page.page.flashes.order-success'), MessageType::SUCCESS
        );

        $item = $this->pageManager->getPageById($item_id);
        $prev = $this->pageManager->getPageById($prev_id);
        $next = $this->pageManager->getPageById($next_id);
        $position = $item->position;

        if ($next && $prev) {
            $space = $prev->position - $next->position;
            if ($space > 1) {
                $position = $next->position + 1;
            } elseif ($space == 1) {
                $prevPosition = $prev->position + 1;
                $position = $prev->position;
            } elseif ($space == 0) {
                $prevPosition = $prev->position + 2;
                $position = $prev->position + 1;
            }
        } elseif ($next && $position == 0) {
            if ($next->position < 0) {
                $nextPosition = 0;
                $position = 1;
            } else {
                $position = $next->position + 1;
            }
        } elseif ($prev && $position == 0) {
            $prevPosition = $position + 1;
        } elseif ($prev) {
            $position = 0;
        } elseif ($next) {
            $position = $next->position + 1;
        }

        $this->pageManager->savePage([
            PageManager::COLUMN_ID => $item->{PageManager::COLUMN_ID},
            PageManager::COLUMN_POSITION => $position
        ]);

        if (isset($prevPosition)) {
            $this->pageManager->savePage([
                PageManager::COLUMN_ID => $prev->{PageManager::COLUMN_ID},
                PageManager::COLUMN_POSITION => $prevPosition
            ]);
        }

        if (isset($nextPosition)) {
            $this->pageManager->savePage([
                PageManager::COLUMN_ID => $next->{PageManager::COLUMN_ID},
                PageManager::COLUMN_POSITION => $nextPosition
            ]);
        }

        if ($presenter->isAjax()) {
            $presenter->redrawControl("ajaxNotification");
            $this['dataGrid']->reload();
        } else {
            $presenter->redirect('this');
        }
    }

    public function getChildren($parentId) {
        return $this->pageManager->getChildren($parentId);
    }

    public function hasChildren($parentId) {
        if ($this->pageManager->hasChildren($parentId) > 0) {
            return true;
        } else {
            return false;
        }
    }

}

interface IPageDataGridFactory {

    /**
     * @return PageDataGridControl
     */
    public function create();
}
