<?php

/**
 * Třída poskytuje metody pro správu stránek v redakčním systému.
 * @package App\CmsModule\Model
 *
 * @author Petr Vojáček <petr at ideatech.cz>
 */

namespace App\CmsModule\Model;

use App\Model\BaseManager;
use Nette\Database\Context;
use Nette\Database\Table\IRow;
use Nette\Database\Table\Selection;
use Nette\Utils\ArrayHash;

class PageManager extends BaseManager
{

    /** Konstanty pro manipulaci s modelem. */
    const
        TABLE_NAME = 'page',
        COLUMN_ID = 'id',
        COLUMN_TITLE = 'title',
        COLUMN_URL = 'url',
        COLUMN_SLOGAN = 'slogan',
        COLUMN_TYPE = 'type',
        COLUMN_POSITION = 'position',
        COLUMN_PARENT = 'parent_id',
        COLUMN_MENU = 'menu',
        COLUMN_IMAGE_COUNT = 'images_count',
        COLUMN_PRODUCT = 'product';


    /**
     * Vrátí seznam stránek v databázi.
     * @return Selection seznam stránek
     */
    public function getPages()
    {
        return $this->database->table(self::TABLE_NAME)->order(self::COLUMN_POSITION . ' DESC');
    }

    /**
     * Pokusí se najít stránku, která obsahuje řetězec v url.
     * Používá se pro získání informací o stránce, která není typickou stránkou, ale odkazem na jiný presenter.
     * @param $url
     * @return bool|mixed|IRow
     */
    public function findPageByUrlString($url)
    {
        $url = '%' . $url . '%';
        return $this->database->table(self::TABLE_NAME)->where(self::COLUMN_URL . ' LIKE ?', $url)->limit(1)->fetch();
    }

    /**
     * Vrátí seznam stránek v databázi, které se mají zobrazovat v menu stránek.
     * @return Selection seznam stránek
     */
    public function getPagesToMenu()
    {
        return $this->database->table(self::TABLE_NAME)
            ->where(self::COLUMN_MENU, 1)
            ->order(self::COLUMN_POSITION . ' DESC');
    }

    /**
     * Vrátí stránku z databáze podle její URL.
     * @param string $url URl stránky
     * @return bool|mixed|IRow první stránka, který odpovídá URL nebo false při neúspěchu
     */
    public function getPage($url)
    {
        return $this->database->table(self::TABLE_NAME)->where(self::COLUMN_URL, $url)->fetch();
    }

    /**
     * Vrátí stránku z databáze podle jejího ID.
     * @param int $pageId ID stránky
     * @param null|string $columns sloupce pro SELECT, nebo null pro všechny sloupce
     * @return bool|mixed|IRow první stránka, který odpovídá URL nebo false při neúspěchu
     */
    public function getPageById($pageId, $columns = null)
    {
        return $this->database->table(self::TABLE_NAME)
            ->select($columns ? $columns : '*')
            ->where(self::COLUMN_ID, array($pageId))->fetch();
    }

    /**
     * Uloží stránku do systému. Pokud není nastaveno ID, vloží novou, jinak provede editaci.
     * @param array|ArrayHash $page stránka
     */
    public function savePage($page)
    {
        if ($page[self::COLUMN_POSITION] == '') {
            $page[self::COLUMN_POSITION] = 0;
        }

        if (!$page[self::COLUMN_ID]) {
            $page->offsetUnset(self::COLUMN_ID);
            return $this->database->table(self::TABLE_NAME)->insert($page);
        } else {
            return $this->database->table(self::TABLE_NAME)->where(self::COLUMN_ID, $page[self::COLUMN_ID])->update($page);
        }
    }

    /**
     * Odstraní stránku.
     * @param int $id URL stránky
     */
    public function removePage($id)
    {
        $this->database->table(self::TABLE_NAME)->where(self::COLUMN_ID, $id)->delete();
    }

    /**
     * Vrátí tabulku jako zdroj pro DataGrid
     */
    public function getTable()
    {
        return $this->database->table(self::TABLE_NAME);
    }

    /**
     * Vrátí asociativní pole id => nazev stranky pro pouziti v selectu
     */
    public function getPagesToSelect()
    {
        $pages = $this->getPages();
        return $pages->fetchPairs('id', 'title');
    }

    public function getDatagridSource()
    {
        return $this->database->table(self::TABLE_NAME)->where(self::COLUMN_PARENT, null);
    }

    /**
     * Vrátí podřazené stránky
     * @param $parentId int Id nadřazené stránky
     * @return Selection
     */
    public function getChildren($parentId)
    {
        return $this->getPages()->where(self::COLUMN_PARENT, $parentId);
    }

    public function hasChildren($parentId)
    {
        return $this->getChildren($parentId)->count('*');
    }

    /**
     * @param $productId
     * @return Selection
     */
    public function findPageByProduct($productId)
    {
        return $this->database->table(self::TABLE_NAME)->where(self::COLUMN_PRODUCT, $productId);
    }

}
