<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\CoreModule\Components\DataGrids;

use App\Model\Enum\MessageType;
use App\CoreModule\Model\ModuleManager;
use Ublaboo\DataGrid\DataGrid;
use Nette\Application\UI\Control;

/**
 * Komponenta datagridu pro správu modulů
 * @package App\CoreModule\Components\DataGrids
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class ModuleDatagridControl extends Control {

    /** Cesta k souboru šablony pro tuto komponentu. */
    const TEMPLATE = '/../../../../templates/components/DataGrids/dataGrid.latte';

    /** @var ModuleManager Instance třídy modelu pro práci s moduly. */
    private $moduleManager;

    /**
     * Konstruktor s injectovanými třídami modelu pro práci s moduly
     * @param ModuleManager $moduleManager  třída modelu pro práci s modulz
     */
    public function __construct(ModuleManager $moduleManager) {
        parent::__construct();

        $this->moduleManager = $moduleManager;
    }

    /** Vykreslí komponentu formuláře pro správu osob. */
    public function render() {
        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE);

        // Vykreslí komponentu.
        $this->template->render();
    }

    public function createComponentDataGrid($name) {

        $grid = new DataGrid($this, $name);

        /** Localization */
        $grid->setTranslator($this->presenter->translator);

        $grid->setPrimaryKey(ModuleManager::COLUMN_ID);
        // Dotaz na vsechny moduly v DB
        $grid->setDataSource($this->moduleManager->getAllRows(ModuleManager::TABLE_NAME));
        // Jméno modulu
        $grid->addColumnText(ModuleManager::COLUMN_NAME, 'core.settingsCMS.modules.name')
                ->setReplacement(['' => '-----'])
                ->setSortable();
        $grid->addFilterText(ModuleManager::COLUMN_NAME, 'core.settingsCMS.modules.name');
        // Slug
        $grid->addColumnText(ModuleManager::COLUMN_SLUG, 'core.settingsCMS.modules.slug')
                ->setReplacement(['' => '-----'])
                ->setSortable();
        $grid->addFilterText(ModuleManager::COLUMN_SLUG, 'core.settingsCMS.modules.slug');
        // Aktivnost
        $grid->addColumnStatus(ModuleManager::COLUMN_ACTIVE, 'core.settingsCMS.modules.activity')
                        ->addOption(1, 'core.settingsCMS.modules.activityOn')
                        ->setClass('btn-success')
                        ->endOption()
                        ->addOption(0, 'core.settingsCMS.modules.activityDown')
                        ->setClass('btn-danger')
                        ->endOption()
                ->onChange[] = [$this, 'activityChange'];
        // Přidání filteru
        $grid->getColumn(ModuleManager::COLUMN_ACTIVE)
                ->setFilterSelect([
                    1 => $this->presenter->translator->translate('core.settingsCMS.modules.activityOn'),
                    0 => $this->presenter->translator->translate('core.settingsCMS.modules.activityDown')
        ]);

        /** Editace položky */
        $grid->addInlineEdit()
                ->onControlAdd[] = function($container) {
            $container->addText(ModuleManager::COLUMN_NAME, '');
            $container->addText(ModuleManager::COLUMN_SLUG, '');
        };

        /** Přidávání nové položky */
        $grid->addInlineAdd()
                ->onControlAdd[] = function($container) {
            $container->addText(ModuleManager::COLUMN_NAME, '');
            $container->addText(ModuleManager::COLUMN_SLUG, '');
        };

        /** Callback pro uložení položky */
        $grid->getInlineAdd()->onSubmit[] = function($values) {
            $this->saveValues($values);
        };

        /** Set defaults pro editaci položky */
        $grid->getInlineEdit()->onSetDefaults[] = function($container, $item) {
            $container->setDefaults([
                ModuleManager::COLUMN_NAME => $item[ModuleManager::COLUMN_NAME],
                ModuleManager::COLUMN_SLUG => $item[ModuleManager::COLUMN_SLUG],
            ]);
        };

        /** Callback pro editaci položky */
        $grid->getInlineEdit()->onSubmit[] = function($id, $values) {
            $this->saveValues($values, $id);
        };

        return $grid;
    }
    
     public function activityChange($id, $activity) {
        // Presenter
        $presenter = $this->presenter;
        //Hodnoty pro ulozeni
        $values[ModuleManager::COLUMN_ACTIVE] = $activity;
        $values[ModuleManager::COLUMN_ID] = $id;

        // Uložení hodnot
        if ($row = $this->moduleManager->saveValues(ModuleManager::TABLE_NAME, ModuleManager::COLUMN_ID, $values)) {
            // Najdu si novou roli pro flash message
            $moduleName = $this->moduleManager->getById(ModuleManager::TABLE_NAME, ModuleManager::COLUMN_ID, $id)[ModuleManager::COLUMN_NAME];
            // Zaznamení akce do DB
            $presenter->logAction('Změna aktivnosti - ' . $moduleName . ' na ' . $activity);
            // Zpráva pro editaci hodnoty
            $presenter->flashMessage($presenter->translator->translate('core.settingsCMS.modules.flashes.editModule', ['module_name' => $moduleName]), MessageType::SUCCESS);
            /*
            if ($presenter->isAjax()) {
                // Překreslím notifikace
                $presenter->redrawControl('ajaxNotification');
                // Překreslení gridu
                $this['dataGrid']->reload();
            } else {
                $presenter->redirect('this');
            }
             */
            $presenter->redirect('this');
        }
    }

    /**
     * Akce, která se volá při editaci, nebo při vkládání nové položky přímo z datagridu
     * @param ArrayHash $values hodnoty pro uložení, editaci
     * @param int|null $id id položky v případě editace
     */
    private function saveValues($values, $id = null) {
        // Získám si presenter
        $presenter = $this->presenter;

        // Pokud nejsou vyplněny hodnoty, předám zprávu, obnovím notifikace a funkci skončím
        if ($values[ModuleManager::COLUMN_NAME] == '' || $values[ModuleManager::COLUMN_SLUG] == '') {
            $presenter->flashMessage($presenter->translator->translate('core.userRole.role.flashes.saveError'), MessageType::ERROR);
            $presenter->redrawControl('ajaxNotification');
            $this['dataGrid']->reload();
            return;
        }

        if (isset($id)) {
            // Pokud edituji, nasetuji do values i ID položky, metoda saveValues si s tím už poradí
            $values[ModuleManager::COLUMN_ID] = $id;
        }

        // Uložení hodnot
        if ($row = $this->moduleManager->saveValues(ModuleManager::TABLE_NAME, ModuleManager::COLUMN_ID, $values)) {
            if (isset($id)) {
                // Zpráva pro editaci hodnoty
                $presenter->flashMessage($presenter->translator->translate('core.settingsCMS.modules.flashes.editModule', ['module_name' => $values[ModuleManager::COLUMN_NAME]]), MessageType::SUCCESS);
                // Zaznamení akce do DB
                $presenter->logAction('Vytvoření nového modulu - ' . $values[ModuleManager::COLUMN_NAME]);
            } else {
                // Zpráva pro uložení nové hodnoty
                $presenter->flashMessage($presenter->translator->translate('core.settingsCMS.modules.flashes.newModule', ['module_name' => $values[ModuleManager::COLUMN_NAME]]), MessageType::SUCCESS);
                // Zaznamení akce do DB
                $presenter->logAction('Editace modulu - ' . $values[ModuleManager::COLUMN_NAME]);
            }

            if ($presenter->isAjax()) {
                // Překreslím notifikace
                $presenter->redrawControl('ajaxNotification');
                // Překreslení gridu
                $this['dataGrid']->reload();
            } else {
                $presenter->redirect('this');
            }
        }
    }

}

/**
 * Interface IModuleDataGridFactory
 * @package App\CoreModule\Components\DataGrids
 */
interface IModuleDataGridFactory {

    /**
     * @return ModuleDatagridControl
     */
    public function create();
}
