<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\CoreModule\Components\DataGrids;

use App\Model\Enum\MessageType;
use Ublaboo\DataGrid\DataGrid;
use Nette\Application\UI\Control;
use App\CoreModule\Model\SocialManager;
use App\Services\WebDir;

/**
 * Komponenta datagridu pro správu stránek
 * @package App\CoreModule\Components\DataGrids
 *
 * @author Petr Vojáček <petr at ideatech.cz>
 */
class SocialDatagridControl extends Control {

    /** Cesta k souboru šablony pro tuto komponentu. */
    const TEMPLATE = '/../../../../templates/components/DataGrids/dataGrid.latte';
    const TEMPLATE_IMAGE_COL = __DIR__ . '/../../templates/components/DataGrids/SocialDatagrid/image.latte';

    /** @var SocialManager Instance třídy modelu pro práci se stránkami. */
    private $socialManager;

    /** @var WebDir */
    private $webDir;

    /**
     * Konstruktor s injectovanými třídami modelu pro práci se stránkami
     * @param SocialManager $socialManager automaticky injektovaná třída modelu pro práci se stránkami
     */
    public function __construct(SocialManager $socialManager, WebDir $webDir) {
        parent::__construct();
        $this->socialManager = $socialManager;
        $this->webDir = $webDir;
    }

    /** Vykreslí komponentu formuláře pro správu osob. */
    public function render() {
        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE);

        // Vykreslí komponentu.
        $this->template->render();
    }

    public function createComponentDataGrid($name) {
        $grid = new DataGrid($this, $name);
        $presenter = $this->presenter;

        $grid->setDataSource($this->socialManager->getTable());
        $grid->setSortable();

        $grid->setSortableHandler('socialDatagrid:sort!');
        $grid->addColumnNumber('position', 'core.social.datagrid.column-position', 'position')->setDefaultHide(true);

        $grid->setDefaultSort(['position' => 'DESC']);
        $grid->addColumnText('title', 'core.social.datagrid.column-name')
                ->setSortable();

        $grid->addColumnText('image', 'core.social.datagrid.column-image')
                ->setTemplate(self::TEMPLATE_IMAGE_COL, []);

        /**
         * Odkaz na editaci
         */
        $grid->addAction('edit', '', ':Core:Social:edit')
                ->setIcon('pencil');

        /**
         * Mazani
         */
        $grid->addAction('delete', '', 'deleteSocial!')
                ->setIcon('trash')
                ->setTitle('core.social.datagrid.action-delete')
                ->setClass('btn btn-xs btn-danger ajax')
                ->setConfirm('core.social.datagrid.action-delete-confirm', 'title');


        $grid->setTranslator($presenter->translator);

        return $grid;
    }

    public function positionEdited($id, $value) {
        $this->socialManager->saveValues(SocialManager::TABLE_NAME, SocialManager::COLUMN_ID,[
            'id' => $id,
            'position' => $value
        ]);
    }

    public function handleDeleteSocial($id) {
        $presenter = $this->presenter;
        $social = $this->socialManager->getById(SocialManager::TABLE_NAME, SocialManager::COLUMN_ID, $id);
        if (isset($social->image) && file_exists($this->webDir->getPath($social->image))) {
            unlink($this->webDir->getPath($social->image));
        }

        $this->socialManager->remove(SocialManager::TABLE_NAME, SocialManager::COLUMN_ID, $id);
        // Zaznamení akce do DB
        $presenter->logAction('Smazání sociální sítě - ' . $id);
        $presenter->flashMessage($presenter->translator->translate('core.social.flashes.deleted-success', ['id' => $id]), MessageType::SUCCESS);
        if ($presenter->isAjax()) {
            $presenter->redrawControl('ajaxNotification');
            $this['dataGrid']->reload();
        } else {
            $presenter->redirect('this');
        }
    }

    public function handleSort($item_id, $prev_id, $next_id) {
        $presenter = $this->presenter;
        $presenter->flashMessage(
                $presenter->translator->translate('core.social.flashes.order-success'), MessageType::SUCCESS
        );

        $item = $this->socialManager->getById(SocialManager::TABLE_NAME, SocialManager::COLUMN_ID, $item_id);
        $prev = $this->socialManager->getById(SocialManager::TABLE_NAME, SocialManager::COLUMN_ID, $prev_id);
        $next = $this->socialManager->getById(SocialManager::TABLE_NAME, SocialManager::COLUMN_ID, $next_id);
        $position = $item->position;

        if ($next && $prev) {
            $space = $prev->position - $next->position;
            if ($space > 1) {
                $position = $next->position + 1;
            } elseif ($space == 1) {
                $prevPosition = $prev->position + 1;
                $position = $prev->position;
            } elseif ($space == 0) {
                $prevPosition = $prev->position + 2;
                $position = $prev->position + 1;
            }
        } elseif ($next && $position == 0) {
            if ($next->position < 0) {
                $nextPosition = 0;
                $position = 1;
            } else {
                $position = $next->position + 1;
            }
        } elseif ($prev && $position == 0) {
            $prevPosition = $position + 1;
        } elseif ($prev) {
            $position = 0;
        } elseif ($next) {
            $position = $next->position + 1;
        }

        $this->socialManager->saveValues(SocialManager::TABLE_NAME, SocialManager::COLUMN_ID,[
            SocialManager::COLUMN_ID => $item->{SocialManager::COLUMN_ID},
            SocialManager::COLUMN_POSITION => $position
        ]);

        if (isset($prevPosition)) {
            $this->socialManager->saveValues(SocialManager::TABLE_NAME, SocialManager::COLUMN_ID,[
                SocialManager::COLUMN_ID => $prev->{SocialManager::COLUMN_ID},
                SocialManager::COLUMN_POSITION => $prevPosition
            ]);
        }

        if (isset($nextPosition)) {
            $this->socialManager->saveValues(SocialManager::TABLE_NAME, SocialManager::COLUMN_ID,[
                SocialManager::COLUMN_ID => $next->{SocialManager::COLUMN_ID},
                SocialManager::COLUMN_POSITION => $nextPosition
            ]);
        }

        if ($presenter->isAjax()) {
            $presenter->redrawControl('ajaxNotification');
            $this['dataGrid']->reload();
        } else {
            $presenter->redirect('this');
        }
    }

}

/**
 * Interface ISocialDatagridFactory
 * @package App\CoreModule\Components\DataGrids
 */
interface ISocialDataGridFactory {

    /**
     * @return SocialDatagridControl
     */
    public function create();
}
