<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\CoreModule\Components\DataGrids;

use App\Model\Enum\MessageType;
use App\Model\PersonManager;
use App\Model\UserManager;
use App\CoreModule\Model\UserRoleManager;
use Ublaboo\DataGrid\DataGrid;
use Nette\Application\UI\Control;

/**
 * Komponenta datagridu pro správu uživatelů systému
 * @package App\CoreModule\Components\DataGrids
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class UsersDatagridControl extends Control {

    /** Cesta k souboru šablony pro tuto komponentu. */
    const TEMPLATE = '/../../../../templates/components/DataGrids/dataGrid.latte';

    /** @var PersonManager Instance třídy modelu pro práci s osobami. */
    private $personManager;

    /** @var UserManager Instance třídy modelu pro práci s uživateli. */
    private $userManager;

    /** @var UserRoleManager Instance třídy modelu pro práci s rolemi. */
    private $userRoleManager;

    /**
     * Konstruktor s injectovanými třídami modelu pro práci s moduly
     * @param PersonManager $personManager  třída modelu pro práci s osobami
     * @param UserManager   $userManager    třída modelu pro práci s uživateli
     * @param UserRoleManager $userRoleManager třída modelu pro práci s rolemi
     */
    public function __construct(PersonManager $personManager, UserManager $userManager, UserRoleManager $userRoleManager) {
        parent::__construct();

        $this->userManager = $userManager;
        $this->personManager = $personManager;
        $this->userRoleManager = $userRoleManager;
    }

    /** Vykreslí komponentu formuláře pro správu osob. */
    public function render() {
        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE);

        // Vykreslí komponentu.
        $this->template->render();
    }

    public function createComponentDataGrid($name) {

        $grid = new DataGrid($this, $name);

        /** Localization */
        $grid->setTranslator($this->presenter->translator);

        $grid->setPrimaryKey(UserManager::COLUMN_ID);
        // Dotaz na vsechny osoby, které  jsou registrovány. Musim spojit tabulky s person detail a user
        $grid->setDataSource($this->getDataSource());
        // Jméno osoby
        $grid->addColumnText(PersonManager::COLUMN_FIRST_NAME, 'eshop.person.labels.name')
                ->setReplacement(['' => '-----'])
                ->setSortable();
        $grid->addFilterText(PersonManager::COLUMN_FIRST_NAME, 'eshop.person.labels.name');
        // Příjmení osoby
        $grid->addColumnText(PersonManager::COLUMN_LAST_NAME, 'eshop.person.labels.lastName')
                ->setReplacement(['' => '-----'])
                ->setSortable();
        $grid->addFilterText(PersonManager::COLUMN_LAST_NAME, 'eshop.person.labels.lastName');
        // Název firmy
        $grid->addColumnText(PersonManager::COLUMN_COMPANY_NAME, 'eshop.person.labels.companyName')
                ->setReplacement(['' => '-----'])
                ->setSortable();
        $grid->addFilterText(PersonManager::COLUMN_COMPANY_NAME, 'eshop.person.labels.companyName');
        // Login do systému
        $grid->addColumnText(UserManager::COLUMN_NAME, 'eshop.person.labels.login')
                ->setSortable();
        $grid->addFilterText(UserManager::COLUMN_NAME, 'eshop.person.labels.login');

        // Uživatelská role
        $grid->addColumnStatus(UserManager::COLUMN_ROLE, 'core.settingsCMS.userRole')
                ->onChange[] = [$this, 'statusChange'];

        // Přidání filteru
        $grid->getColumn(UserManager::COLUMN_ROLE)
                ->setFilterSelect($this->getFilterRoles());

        // Přidám položky do selectu - všechny role ve tvaru role_slug => role_name
        $grid->addColumnCallback(UserManager::COLUMN_ROLE, function($column) {
            $this->getColumsRoles($column);
        });

        /**
         * Odkaz na editaci
         */
        $grid->addAction('edit', '', ':Core:Users:userDetail')
                ->setIcon('pencil');

        // Tlačítko pro vložení nového uživatele
        $grid->addToolbarButton(':Core:Users:registerAccount', 'core.settingsCMS.newUser')
                ->setClass('btn btn-xs btn-default');

        return $grid;
    }

    /**
     * Pomocná funkce pro získání dat dat do datagridu
     * @return Selection vrací všechny uživatelské akce spojené s údaji pro uživatele a jeho údaje z detailu osoby
     */
    private function getDataSource() {
        $users = $this->personManager->getTable()
                ->select('*')
                ->where(PersonManager::TABLE_DETAIL_NAME . '.' . PersonManager::COLUMN_PERSON_DETAIL_ID)
                ->where(UserManager::TABLE_NAME . '.' . UserManager::COLUMN_ID);

        // Pokud není super admin, nezobrazím mu ani tyto uživatele
        if (!$this->presenter->getUser()->isInRole(UserRoleManager::ROLE_TYPE_SUPERADMIN)) {
            $users->where('NOT ' . UserManager::COLUMN_ROLE, UserRoleManager::ROLE_TYPE_SUPERADMIN);
        }

        return $users->fetchAll();
    }

    /**
     * Pomocná funkce pro přidávání položek do statusu - změny uživatelských rolí
     * @param array $column Položky selectu
     * @return array Vrací seznam položek do selectu
     */
    private function getColumsRoles($column) {
        $roles = $this->userRoleManager->getRolesForForm();
        // Pokud neni super admin, nemuze tuto roli nikomu nastavovat
        if (!$this->presenter->getUser()->isInRole(UserRoleManager::ROLE_TYPE_SUPERADMIN)) {
            unset($roles[UserRoleManager::ROLE_TYPE_SUPERADMIN]);
        }

        foreach ($roles as $key => $value) {
            if ($key == UserRoleManager::ROLE_TYPE_ADMIN || $key == UserRoleManager::ROLE_TYPE_SUPERADMIN) {
                $column->addOption($key, $value)
                        ->setClass('btn-success')
                        ->endOption();
            } else {
                $column->addOption($key, $value)
                        ->setClass('btn-info')
                        ->endOption();
            }
        }

        return $column;
    }
    
    /**
     * Pomocná funkce pro přidání názvů rolí pro filtrování podle rolí
     * @return array Vrací role pro filter
     */
    private function getFilterRoles(){
        $roles[''] = 'Všechny role';
        
        // Sjednotím nadefinované pole s nově získanýma rolema
        $roles = array_merge($roles, $this->userRoleManager->getRolesForForm());
        
        // Pokud neni super admin, nemuze tuto roli nikomu nastavovat
        if (!$this->presenter->getUser()->isInRole(UserRoleManager::ROLE_TYPE_SUPERADMIN)) {
            unset($roles[UserRoleManager::ROLE_TYPE_SUPERADMIN]);
        }
        
        return $roles;
    }

    public function statusChange($id, $new_role) {
        // Presenter
        $presenter = $this->presenter;
        //Hodnoty pro ulozeni
        $values[UserManager::COLUMN_ROLE] = $new_role;
        $values[UserManager::COLUMN_ID] = $id;

        // Uložení hodnot
        if ($row = $this->userRoleManager->saveValues(UserManager::TABLE_NAME, UserManager::COLUMN_ID, $values)) {
            // Najdu si novou roli pro flash message
            $role_name = $this->userRoleManager->getById(UserRoleManager::TABLE_ROLE_NAME, UserRoleManager::COLUMN_ROLE_SLUG, $new_role)[UserRoleManager::COLUMN_ROLE_NAME];
            $user_login = $this->userRoleManager->getById(UserManager::TABLE_NAME, UserManager::COLUMN_ID, $id)[UserManager::COLUMN_NAME];

            // Zaznamení akce do DB
            $presenter->logAction('Změna role uživatele - ' . $user_login . ' na ' . $role_name);

            // Zpráva pro editaci hodnoty
            $presenter->flashMessage($presenter->translator->translate('core.userRole.role.flashes.setNewRoleSuccess', ['user_name' => $user_login, 'role_name' => $role_name]), MessageType::SUCCESS);
            $presenter->redirect('this');

            /*
              if ($presenter->isAjax()) {
              // Překreslím notifikace
              $presenter->redrawControl('ajaxNotification');
              // Překreslení gridu
              $this['dataGrid']->reload();
              } else {
              $presenter->redirect('this');
              }
             */
        }
    }

}

/**
 * Interface IUserDataGridFactory
 * @package App\CoreModule\Components\DataGrids
 */
interface IUserDataGridFactory {

    /**
     * @return UsersDatagridControl
     */
    public function create();
}
