<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\CoreModule\Components\DataGrids;

use App\CoreModule\Model\UserRoleManager;
use App\Model\Enum\MessageType;
use Ublaboo\DataGrid\DataGrid;
use Nette\Application\UI\Control;
use Nette\Database\ForeignKeyConstraintViolationException;

/**
 * Komponenta datagridu pro správu uživatelských rolí
 * @package App\CoreModule\Components\DataGrids
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class UserRoleDatagridControl extends Control {

    /** Cesta k souboru šablony pro tuto komponentu. */
    const TEMPLATE = '/../../../../templates/components/DataGrids/dataGrid.latte';

    /** @var UserRoleManager Instance třídy modelu pro práci s uživatelskými rolemi. */
    private $userRoleManager;

    /**
     * Konstruktor s injectovanými třídami modelu pro práci s uživatelskými rolemi
     * @param UserRoleManager   $userRoleManager    třída modelu pro práci s uživatelskými rolemi
     */
    public function __construct(UserRoleManager $userRoleManager) {
        parent::__construct();

        $this->userRoleManager = $userRoleManager;
    }

    /** Vykreslí komponentu formuláře pro správu osob. */
    public function render() {
        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE);

        // Vykreslí komponentu.
        $this->template->render();
    }

    public function createComponentDataGrid($name) {
        $grid = new DataGrid($this, $name);

        /** Localization */
        $grid->setTranslator($this->presenter->translator);
        // Primary key
        $grid->setPrimaryKey(UserRoleManager::COLUMN_ROLE_ID);
        // Všechny uživatelské role
        $grid->setDataSource($this->userRoleManager->getAllRows(UserRoleManager::TABLE_ROLE_NAME));
        // Jméno role
        $grid->addColumnText(UserRoleManager::COLUMN_ROLE_NAME, 'core.userRole.role.name')
                ->setSortable();
        $grid->addFilterText(UserRoleManager::COLUMN_ROLE_NAME, 'core.userRole.role.name');
        // Slug role
        $grid->addColumnText(UserRoleManager::COLUMN_ROLE_SLUG, 'core.userRole.role.slug')
                ->setSortable();
        $grid->addFilterText(UserRoleManager::COLUMN_ROLE_SLUG, 'core.userRole.role.slug');
        // Popisek role
        $grid->addColumnText(UserRoleManager::COLUMN_ROLE_DESRCIPTION, 'core.userRole.role.description');

        /** Editace položky */
        $grid->addInlineEdit()
                ->onControlAdd[] = function($container) {
            $container->addText(UserRoleManager::COLUMN_ROLE_NAME, '');
            $container->addText(UserRoleManager::COLUMN_ROLE_SLUG, '');
            $container->addText(UserRoleManager::COLUMN_ROLE_DESRCIPTION, '');
        };

        /** Přidávání nové položky */
        $grid->addInlineAdd()
                ->onControlAdd[] = function($container) {
            $container->addText(UserRoleManager::COLUMN_ROLE_NAME, '');
            $container->addText(UserRoleManager::COLUMN_ROLE_SLUG, '');
            $container->addText(UserRoleManager::COLUMN_ROLE_DESRCIPTION, '');
        };

        /** Callback pro uložení položky */
        $grid->getInlineAdd()->onSubmit[] = function($values) {
            $this->saveValues($values);
        };

        /** Set defaults pro editaci položky */
        $grid->getInlineEdit()->onSetDefaults[] = function($container, $item) {
            $container->setDefaults([
                UserRoleManager::COLUMN_ROLE_NAME => $item->role_name,
                UserRoleManager::COLUMN_ROLE_SLUG => $item->role_slug,
                UserRoleManager::COLUMN_ROLE_DESRCIPTION => $item->role_description,
            ]);
        };

        /** Callback pro editaci položky */
        $grid->getInlineEdit()->onSubmit[] = function($id, $values) {
            $this->saveValues($values, $id);
        };

        // Mazání
        $grid->addAction('delete', '', 'deleteRole!')
                ->setIcon('trash')
                ->setTitle('core.userRole.role.action-delete')
                ->setClass(function($item) {
                    if ($item[UserRoleManager::COLUMN_ROLE_SLUG] == UserRoleManager::ROLE_TYPE_SUPERADMIN) {
                        return 'hidden';
                    } else {
                        return 'btn btn-xs btn-danger ajax';
                    }
                })
                ->setConfirm('core.userRole.role.delete-confirm', UserRoleManager::COLUMN_ROLE_NAME);

        // Akce pro editaci konrétního oprávnění
        $grid->addAction(':Core:UserRole:manage', 'Upravit oprávnění', '')
                ->setClass(function($item) {
                    if ($item[UserRoleManager::COLUMN_ROLE_SLUG] == UserRoleManager::ROLE_TYPE_SUPERADMIN) {
                        return 'hidden';
                    } else {
                        return 'btn btn-xs btn-primary';
                    }
                });

        return $grid;
    }

    /**
     * Smazání konkrétní uživatelské role
     * @param int $role_id id role, která se má smazat
     */
    public function handleDeleteRole($role_id) {
        $presenter = $this->presenter;

        try {
            $this->userRoleManager->remove(UserRoleManager::TABLE_ROLE_NAME, UserRoleManager::COLUMN_ROLE_ID, $role_id);
            // Zaznamení akce do DB
            $presenter->logAction('Smazání uživatelské role - ' . $role_id);

            $presenter->flashMessage($this->presenter->translator->translate('core.userRole.role.flashes.deleteSuccess', ['role_id' => $role_id]), MessageType::SUCCESS);
        } catch (ForeignKeyConstraintViolationException $ex) {
            $presenter->flashMessage($this->presenter->translator->translate('core.userRole.role.flashes.deleteError'), MessageType::ERROR);
        }

        if ($presenter->isAjax()) {
            $presenter->redrawControl('ajaxNotification');
            // Překreslení gridu
            $this['dataGrid']->reload();
        } else {
            $presenter->redirect('this');
        }
    }

    /**
     * Akce, která se volá při editaci, nebo při vkládání nové položky přímo z datagridu
     * @param ArrayHash $values hodnoty pro uložení, editaci
     * @param int|null $id id položky v případě editace
     */
    private function saveValues($values, $id = null) {
        // Získám si presenter
        $presenter = $this->presenter;

        // Pokud nejsou vyplněny hodnoty, předám zprávu, obnovím notifikace a funkci skončím
        if ($values[UserRoleManager::COLUMN_ROLE_NAME] == '' || $values[UserRoleManager::COLUMN_ROLE_SLUG] == '' || $values[UserRoleManager::COLUMN_ROLE_DESRCIPTION] == '') {
            $presenter->flashMessage($presenter->translator->translate('core.userRole.role.flashes.saveError'), MessageType::ERROR);
            $presenter->redrawControl('ajaxNotification');
            $this['dataGrid']->reload();
            return;
        }

        if (isset($id)) {
            // Pokud edituji, nasetuji do values i ID položky, metoda saveValues si s tím už poradí
            $values[UserRoleManager::COLUMN_ROLE_ID] = $id;
        }

        // Uložení hodnot
        if ($row = $this->userRoleManager->saveValues(UserRoleManager::TABLE_ROLE_NAME, UserRoleManager::COLUMN_ROLE_ID, $values)) {
            if (isset($id)) {
                // Zaznamení akce do DB
                $presenter->logAction('Editace uživatelské role - ' . $values[UserRoleManager::COLUMN_ROLE_NAME]);
                // Zpráva pro editaci hodnoty
                $presenter->flashMessage($presenter->translator->translate('core.userRole.role.flashes.editRole', ['role_name' => $values[UserRoleManager::COLUMN_ROLE_NAME]]), MessageType::SUCCESS);
            } else {
                // Zaznamení akce do DB
                $presenter->logAction('Vytvoření nové uživatelské role - ' . $values[UserRoleManager::COLUMN_ROLE_NAME]);
                // Zpráva pro uložení nové hodnoty
                $presenter->flashMessage($presenter->translator->translate('core.userRole.role.flashes.newRole', ['role_name' => $values[UserRoleManager::COLUMN_ROLE_NAME]]), MessageType::SUCCESS);
            }

            if ($presenter->isAjax()) {
                // Překreslím notifikace
                $presenter->redrawControl('ajaxNotification');
                // Překreslení gridu
                $this['dataGrid']->reload();
            } else {
                $presenter->redirect('this');
            }
        }
    }

}

/**
 * Interface IUserRoleDataGridFactory
 * @package App\CoreModule\Components\DataGrids
 */
interface IUserRoleDataGridFactory {

    /**
     * @return UserRoleDatagridControl
     */
    public function create();
}
