<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\CoreModule\Components\Forms;

use App\Model\Enum\MessageType;
use App\Forms\BaseFormFactory;
use Nette\Application\UI\Control;
use Nette\Application\UI\Form;
use Nette\Utils\ArrayHash;
use Nette\InvalidStateException;
use App\CoreModule\Model\UserRoleManager;
use App\CoreModule\Model\ModuleManager;

/**
 * Class PermissionFormControl - formulář pro nastavení oprvánění pro konkrétní roli
 * @package App\CoreModule\Components\Forms
 */
class PermissionFormControl extends Control {

    /** Cesta k souborům šablon pro tuto komponentu. */
    const TEMPLATE_PATH = '/../../templates/components/Forms/PermissionSettingsForm/';

    /**
     * @var UserRoleManager
     */
    private $userRolesManager;
    
    /** 
     * @var ModuleManager Instance třídy modelu pro práci s moduly. 
     */
    private $moduleManager;

    /**
     * @var BaseFormFactory
     */
    private $formFactory;

    /**
     * PermissionFormControl constructor.
     * @param UserRoleManager $userRolesManager
     * @param ModuleManager $moduleManager
     * @param BaseFormFactory $baseFormFactory
     */
    public function __construct(UserRoleManager $userRolesManager, ModuleManager $moduleManager, BaseFormFactory $baseFormFactory) {
        parent::__construct();
        $this->userRolesManager = $userRolesManager;
        $this->moduleManager = $moduleManager;
        $this->formFactory = $baseFormFactory;
    }

    /**
     * Výchozí metoda pro vykreslení komponenty
     */
    public function render($formPath = null) {
        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE_PATH . 'form.latte');
        $this->template->formPath = $formPath;

        // Zdroje
        $this->template->resources = $this->userRolesManager->getAllRows(UserRoleManager::TABLE_NAME_RESOURCE);
        // Akce
        $this->template->actions = $this->userRolesManager->getAllRows(UserRoleManager::TABLE_ACTION_NAME);
        // Moduly
        $this->template->modules = $this->moduleManager->getAllRows(ModuleManager::TABLE_NAME);
        
        // Vykreslí komponentu.
        $this->template->render();
    }

    public function createComponentPermissionForm() {
        $form = $this->formFactory->create();

        // Skryté pole pro id role
        $form->addHidden('role_id');

        // Všechny akce
        $actions = $this->userRolesManager->getAllRows(UserRoleManager::TABLE_ACTION_NAME);
        // Všechny zdroje
        $resources = $this->userRolesManager->getAllRows(UserRoleManager::TABLE_NAME_RESOURCE);

        foreach ($actions as $action) {
            $form->addCheckbox('action' . $action[UserRoleManager::COLUMN_ACTION_ID], $action[UserRoleManager::COLUMN_ACTION_NAME]);
        }

        foreach ($resources as $resource) {
            $form->addCheckbox('resource' . $resource[UserRoleManager::COLUMN_RESOURCE_ID], $resource[UserRoleManager::COLUMN_RESOURCE_NAME]);
        }

        $form->addSubmit('submit', 'core.email.form.submit')
                ->getControlPrototype();

        $form->onSuccess[] = function (Form $form, ArrayHash $values) {
            $this->succeeded($form, $values);
        };

        return $form;
    }

    public function succeeded(Form $form, ArrayHash $values) {
        $presenter = $form->getPresenter();

        // Id role
        $roleId = $values[UserRoleManager::COLUMN_ROLE_ID];
        $roleName = $this->userRolesManager->getById(UserRoleManager::TABLE_ROLE_NAME, UserRoleManager::COLUMN_ROLE_ID, $roleId)[UserRoleManager::COLUMN_ROLE_NAME];
        $values->offsetUnset(UserRoleManager::COLUMN_ROLE_ID);

        // Pole pro akce a zdroje
        $actions = array();
        $resources = array();

        foreach ($values as $name => $value) {
            if (strpos($name, 'action') !== false) {
                // Otestuji, jestli, jeslti je zaškrnutá a přidám ji do pole akcí
                if ($value) {
                    array_push($actions, substr($name, (strlen('action'))));
                }
            } else {
                // Otestuji, jestli, jeslti je zaškrnutá a přidám ji do pole zdrojů
                if ($value) {
                    array_push($resources, substr($name, (strlen('resource'))));
                }
            }
        }
        
        try{
            $this->userRolesManager->updateRolePermission($roleId, $resources, $actions);
            // Zaznamení akce do DB
            $presenter->logAction('Změna oprávnění pro roli - ' . $roleName);
            $this->presenter->flashMessage($presenter->translator->translate('core.userRole.role.flashes.editRole', ['role_name' => $roleName]), MessageType::SUCCESS);
        } catch (InvalidStateException $ex) {
            $this->presenter->flashMessage($ex->getMessage(), MessageType::ERROR);
        }
        

        if ($presenter->isAjax()) {
            $presenter->redrawControl('ajaxNotification');
            $presenter->redrawControl("permissionSnippet");
        } else {
            $presenter->redirect("this");
        }
    }

    /**
     * Pomocná funkce pro nasetování defaultních hodnot - funkci se volá z venku
     * @param array $values Hodnoty, které se nasetují do formuáře
     */
    public function setDefaults(array $values) {
        $this['permissionForm']->setDefaults($values);
    }

}

/**
 * Interface IPermissionFormFactory
 * @package App\CoreModule\Components\Forms
 */
interface IPermissionFormFactory {

    /**
     * @return PermissionFormControl
     */
    public function create();
}
