<?php

/**
 * @resourceDescription Zpracovává vykreslování administrační sekce.
 * Zdroje, které jsou potřeba pro config, ale nemají vykreslovací metodu
 * @package App\CoreModule\Presenters
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */

namespace App\CoreModule\Presenters;

use App\EshopModule\Model\OrderManager;
use App\CmsModule\Model\PageManager;
use App\Model\UserManager;
use App\CoreModule\Model\ContactMessageManager;
use App\Model\UserLogManager;
use App\Model\Enum\MessageType;
use App\Services\FormatHelperService;
use App\CoreModule\Components\Charts\ISalesDashboardChartFactory;
use Nette\Database\IRow;
use Latte\Runtime\Template;

class AdministrationPresenter extends BaseCorePresenter {

    /**
     * @var array Společné instrukce pro přihlašovací a registrační formuláře.
     */
    private $instructions;

    /** @var OrderManager */
    private $orderManager;

    /** @var PageManager */
    private $pageManager;

    /** @var UserManager */
    private $userManager;
    
    /** @var UserLogManager Instance třídy modelu pro ukládání přihlašovacích logů do db. */
    private $userLogManager;
    
    /** @var ContactMessageManager Instance třídy modelu pro ukládání zpráv do databáze. */
    private $contactMessageManager;

    /** @var FormatHelperService Pomocná služba pro formátování data, jména adresy atd. */
    private $formatHelperService;

    /** @var ISalesDashboardChartFactory */
    private $salesChartFactory;

    public function __construct(OrderManager $orderManager, PageManager $pageManager, UserManager $userManager, ContactMessageManager $contactMessageManager, UserLogManager $userLogManager, FormatHelperService $formatHelperService, ISalesDashboardChartFactory $salesChartFactory) {
        parent::__construct();
        $this->orderManager = $orderManager;
        $this->pageManager = $pageManager;
        $this->userManager = $userManager;
        $this->contactMessageManager = $contactMessageManager;
        $this->userLogManager = $userLogManager;
        $this->formatHelperService = $formatHelperService;
        $this->salesChartFactory = $salesChartFactory;
    }

    /** Volá se před každou akcí presenteru a inicializuje společné proměnné. */
    public function startup() {
        parent::startup();
        $this->instructions = array(
            'message' => null,
            'redirection' => ':Core:Administration:'
        );
    }

    /**
     * @actionName logout
     * @actionDescription Odhlášení uživatele. 
     */
    public function actionLogout() {
        $this->getUser()->logout();
        $this->flashMessage($this->translator->translate('core.common.logout'), MessageType::SUCCESS);
        $this->redirect($this->loginPresenter);
    }

    /**
     * @actionName default
     * @actionDescription Vykreslí výchozí administrační stránku. 
     */
    public function renderDefault() {
        $this->setLayout('layoutBackend');

        // Formátuje jméno kupce dané objednávky.
        $this->template->addFilter('name', [$this->formatHelperService, 'formatName']);
        // Formátuje datum.
        $this->template->addFilter('dateTime', [$this->formatHelperService, 'prettyDateTime']);
        // Získává celkovou cenu dané objednávky s uživatelovými slevami.
        $this->template->addFilter('priceWithSales', function (IRow $order) {
            return $this->orderManager->getOrderSummary($order[OrderManager::COLUMN_ID], FALSE)[OrderManager::ATTR_PRICE];
        });

        // Dosazení do šablony
        $this->template->orders = $this->orderManager->getOrdersFromMonth(date('m'), 10);
        $this->template->orderStates = $this->orderManager->getPrettyOrderStates();
        $this->template->users = sizeof($this->userManager->getAllRows(UserManager::TABLE_NAME));
        $this->template->pages = sizeof($this->pageManager->getAllRows(PageManager::TABLE_NAME));
        $this->template->newMessages = sizeof($this->contactMessageManager->getAllRowsById(ContactMessageManager::TABLE_NAME, 'MONTH(' . ContactMessageManager::COLUMN_CREATED .')', date('m')));
        $this->template->oldMessages = sizeof($this->contactMessageManager->getAllRowsById(ContactMessageManager::TABLE_NAME, 'MONTH(' . ContactMessageManager::COLUMN_CREATED .')', date('m')-1));
        $this->template->userLogs = $this->userLogManager->getAllRows(UserLogManager::TABLE_NAME, 10)->order(UserLogManager::COLUMN_DATE . ' DESC');
        // Pomocná metoda pro naplnění šablony
        $this->fillDashboardTemplate($this->template);
    }

    /**
     * Pomocná metoda pro naplnění šablony daty pro dashboard
     * @param Template $template šablona, která se má naplnit daty
     */
    private function fillDashboardTemplate($template) {
        // Získání objednávek
        $orderThisMonth = $this->orderManager->getOrdersFromMonth(date('m'));
        $orderLastMonth = $this->orderManager->getOrdersFromMonth(date('m') - 1);
        $lastCsutomers = $this->orderManager->getUniqueCustomers($orderLastMonth);
        $newCustomers = $this->orderManager->getUniqueCustomers($orderThisMonth, $lastCsutomers);

        $template->ordersCount = sizeof($orderThisMonth);
        $template->lastOrderCount = sizeof($orderLastMonth);
        $template->totalSales = $this->orderManager->getMounthSales($orderThisMonth);
        $template->lastTotalSales = $this->orderManager->getMounthSales($orderLastMonth);
        $template->newCustomers = sizeof($newCustomers);
        $template->oldCustomers = sizeof($lastCsutomers);
    }

    /**
     * @return \App\CoreModule\Components\SalesDashboardChartControl
     */
    protected function createComponentSalesChart() {
        return $this->salesChartFactory->create();
    }

}
