<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\CoreModule\Presenters;

use App\Mailing\NewUserEmail;
use App\Model\Enum\MessageType;
use App\Services\MailFactory;
use Nette\Application\UI\Form;
use Nette\Utils\ArrayHash;
use App\Components\IPersonFormFactory;
use App\Components\PersonFormControl;
use App\Model\PersonManager;
use App\Model\UserManager;
use App\CoreModule\Components\DataGrids\IUserDataGridFactory;
use App\CoreModule\Components\DataGrids\UsersDatagridControl;

/**
 * @resourceDescription Prsenter pro práci s uživateli systému
 * @actionName management
 * @actionDescription Přehled vytvořených uživatelů a jejich rolí - uživatelé i z webu a jejich zákaznické role
 * Dost podobný presenteru z :Eshop:Person
 * @package App\CoreModule\Presenters
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class UsersPresenter extends BaseCorePresenter {

    /**
     * @var PersonManager Instance třídy modelu pro práci s osobami.
     */
    private $personManager;

    /**
     * @var IUserDataGridFactory
     */
    private $userDataGridFactory;

    /**
     * @var IPersonFormFactory Instance třídy továrničky na tvorbu formulářů pro správu osobami.
     */
    private $personFormFactory;

    /**
     * @var MailFactory
     */
    private $mailFactory;

    public function __construct(PersonManager $personManager, IUserDataGridFactory $userDataGridFactory, IPersonFormFactory $personFormFactory, MailFactory $mailFactory)
    {
        parent::__construct();
        $this->personManager = $personManager;
        $this->userDataGridFactory = $userDataGridFactory;
        $this->personFormFactory = $personFormFactory;
        $this->mailFactory = $mailFactory;
    }

    /**
     * Většina akcí v tomto presenteru využívá layoutBackend, proto ho zde nastavíme pro všechny akce.
     * Pokud budeme potřebovat jiný layout, dá se změnit v renderXX nebo actionXX metodě
     */
    public function beforeRender() {
        parent::beforeRender();
        $this->setLayout('layoutBackend');
    }

    /**
     * @actionName userDetail
     * @actionDescription Editace dat osoby uživatelem systému - editace cizí osoby
     * @param int $user_id ID osoby
     */
    public function actionUserDetail($user_id) {
        // Jssem v adminu, chci formulář renderovat pomocí defaultního renderu
        $this['personForm']->formPath = self::FORM_PATH;

        // Nasetoval jsem do formu ID uživatele a ID jeho osoby
        $this['personForm']->setDefaults(
                [
                    UserManager::COLUMN_ID => $user_id,
                    PersonManager::COLUMN_ID => $this->personManager->getPersonId($user_id)
                ]
        );

        // Doplníme vlastní zpracování k úspěšnému odeslání formuláře pro správu osob.
        $this['personForm']->onSuccess[] = function (Form $form, ArrayHash $values) {
            if (isset($values->person_id)) {
                // Zaznamení akce do DB
                $this->logAction('Editace údajů uživatele - ' . $values->person_detail[PersonManager::COLUMN_EMAIL]);
                $this->flashMessage($this->translator->translate('eshop.person.flashes.saveSuccess'), MessageType::SUCCESS);
            } else {
                $this->flashMessage($this->translator->translate('eshop.person.flashes.saveError'), MessageType::ERROR);
            }

            $this->redirect('this');
        };
    }

    /**
     * Editace dat osoby uživatelem 
     * @param int $user_id ID osoby
     */
    public function renderUserDetail($user_id) {
        $this->template->userId = $user_id;
    }

    /**
     * @actionName registerAccount
     * @actionDescription Vytvoření nového uživatelského účtu spolu s osobou 
     */
    public function actionRegisterAccount() {
        // Jssem v adminu, chci formulář renderovat pomocí defaultního renderu
        $this['personForm']->formPath = self::FORM_PATH;

        // Nasetoval jsem do formu ID uživatele, který mění údaje - jen potřebuji, aby tam něco bylo
        $this['personForm']->setDefaults([UserManager::COLUMN_ID => $this->getUser()->getId()]);

        // Doplní se vlastní zpracování k úspěšnému odeslání formuláře pro správu osob.
        $this['personForm']->onSuccess[] = function (Form $form, ArrayHash $values) {
            if (isset($values->person_id) && isset($values->password)) {
                
                // Zaznamení akce do DB
                $this->logAction('Vytvoření nového uživatele - ' . $values->person_detail[PersonManager::COLUMN_EMAIL]);

                /** @var NewUserEmail $email */
                $email = $this->mailFactory->createByType(NewUserEmail::class, ['password' => $values->password, 'recipient' => $values->person_detail->email]);
                $email->send();

                $this->flashMessage($this->translator->translate('core.settingsCMS.flashes.newUserSuccess'), MessageType::SUCCESS);
                $this->redirect(':Core:Users:management');
            } else {
                $this->flashMessage($this->translator->translate('core.settingsCMS.flashes.newUserError'), MessageType::ERROR);
                $this->redirect('this');
            }
        };
    }

    /**
     * @return UsersDatagridControl
     */
    protected function createComponentUserDatagrid() {
        return $this->userDataGridFactory->create();
    }

    /**
     * Vytváří a vrací komponentu formuláře pro správu osoby.
     * @return PersonFormControl formulář pro správu osoby
     */
    protected function createComponentPersonForm() {
        $personForm = $this->personFormFactory->create();
        return $personForm;
    }

}
