<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Components\DataGrids;

use mPDF;
use Nette\Application\UI\Control;
use Nette\Application\BadRequestException;
use Nette\Bridges\ApplicationLatte\Template;
use Nette\Mail\SendException;
use Ublaboo\DataGrid\DataGrid;
use App\Model\Enum\MessageType;
use App\Services\OrderService;
use App\Services\OrderEmailService;
use App\Model\PersonManager;
use App\EshopModule\Model\OrderManager;
use App\EshopModule\Model\SettingsManager;
use App\Components\IPriceControlFactory;

/**
 * Komponenta datagridu pro správu produktů
 * @package App\EshopModule\Components
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class OrderDataGridControl extends Control {

    /** Cesta k souboru šablony pro tuto komponentu. */
    const TEMPLATE = '/../../../../templates/components/DataGrids/dataGrid.latte';

    /** Cesta k šabloně PDF faktury objednávky. */
    const PDF_TEMPLATE = '/../../templates/Order/pdf.latte';

    /** @var OrderManager Instance třídy modelu pro práci s objednávakami. */
    private $orderManager;

    /** @var PersonManager Instance třídy modelu pro práci s objednávakami. */
    private $personManager;

    /** @var SettingsManager Instance třídy modelu pro práci s účetním nastavením v e-shopu. */
    private $settingsManager;

    /** @var OrderService Služba pro OrderPresentery, která poskytne společné metody pro naplnění šablon */
    private $orderService;

    /** @var OrderEmailService Služba pro OrderPresentery, která slouží pro odesílání e-mailů s informacemi o objednávce */
    private $orderEmailService;

    /** @var IPriceControlFactory Instance třídy továrničky na tvorbu komponent pro vykreslování ceny. */
    private $priceControlFactory;

    public function __construct(OrderManager $orderManager, PersonManager $personManager, SettingsManager $settingsManager, OrderService $orderService, OrderEmailService $orderEmailService, IPriceControlFactory $priceControlFactory) {
        parent::__construct();
        $this->orderManager = $orderManager;
        $this->personManager = $personManager;
        $this->settingsManager = $settingsManager;
        $this->orderService = $orderService;
        $this->orderEmailService = $orderEmailService;
        $this->priceControlFactory = $priceControlFactory;
    }

    /**
     * Vytváří a vrací komponentu pro vykreslení ceny.
     * @return PriceControl komponenta pro vykreslení ceny
     */
    protected function createComponentPrice() {
        return $this->priceControlFactory->create();
    }

    /**
     * Vykreslí komponentu formuláře pro správu objednávek.
     */
    public function render() {
        // Nastaví šablonu komponenty.
        $this->template->setFile(__DIR__ . self::TEMPLATE);

        // Vykreslí komponentu.
        $this->template->render();
    }

    public function createComponentDataGrid($name) {
        $grid = new DataGrid($this, $name);

        /** Localization */
        $grid->setTranslator($this->presenter->translator);
        // Primární klíč
        $grid->setPrimaryKey(OrderManager::COLUMN_ID);
        // Všechny vytvořené obendávky spojené s údaji na osoby - přes pomocnou funkci
        $grid->setDataSource($this->getDataSource());

        // Datum
        $grid->addColumnDateTime(OrderManager::COLUMN_CREATED, 'eshop.order.management.date')
                ->setFormat('d.m.Y H:i')
                ->setSortable()
                ->setFilterDate();
        // Jméno osoby
        $grid->addColumnText(PersonManager::COLUMN_FIRST_NAME, 'eshop.person.labels.name')
                ->setReplacement(['' => '-----'])
                ->setSortable();
        $grid->addFilterText(PersonManager::COLUMN_FIRST_NAME, 'eshop.person.labels.name');
        // Příjmení osoby
        $grid->addColumnText(PersonManager::COLUMN_LAST_NAME, 'eshop.person.labels.lastName')
                ->setReplacement(['' => '-----'])
                ->setSortable();
        $grid->addFilterText(PersonManager::COLUMN_LAST_NAME, 'eshop.person.labels.lastName');
        // Název firmy
        $grid->addColumnText(PersonManager::COLUMN_COMPANY_NAME, 'eshop.person.labels.companyName')
                ->setReplacement(['' => '-----'])
                ->setSortable();
        $grid->addFilterText(PersonManager::COLUMN_COMPANY_NAME, 'eshop.person.labels.companyName');
        // Cena k zaplaceni se slevami a DPH
        $grid->addColumnText('totalToPay', 'eshop.order.manage.prices.priceTotal')
                ->setRenderer(function ($order) {
                    $orderSum = $this->orderManager->getOrderSummary($order[OrderManager::COLUMN_ID], FALSE)[OrderManager::ATTR_PRICE];
                    $priceComponent = $this->presenter->getComponent('price');
                    $priceWithSales = $priceComponent->render($orderSum, $this->settingsManager->getById(SettingsManager::TABLE_NAME, SettingsManager::COLUMN_ID, $order[OrderManager::COLUMN_SETTINGS]));
                    return $priceWithSales;
                });
        // Odkaz na vygenerování PDF faktury
        $grid->addColumnLink(OrderManager::COLUMN_NUMBER, 'eshop.order.form.addProduct.label.orderNumber', 'pdf', null, [OrderManager::COLUMN_ID])
                ->setSortable();
        // Změna stavu faktury
        $grid->addColumnStatus(OrderManager::COLUMN_STATE, 'eshop.order.management.state')
                ->onChange[] = [$this, 'setState'];
        // Přidání filteru
        $grid->getColumn(OrderManager::COLUMN_STATE)
                ->setFilterSelect($this->orderManager->getPrettyOrderStates());
        // Přidám položky do selectu - všechny stavy ve tvaru key => název role
        $grid->addColumnCallback(OrderManager::COLUMN_STATE, function($column) {
            $this->setStateColums($column);
        });

        /**
         * Odkaz na editaci
         */
        $grid->addAction('edit', '', ':Eshop:Order:manage')
                ->setIcon('pencil');

        return $grid;
    }

    /**
     * Pomocná funkce pro získání dat dat do datagridu
     * @return Selection vrací všechny uživatelsky dokončené ojednávky spojené s údaji pro osoby
     */
    private function getDataSource() {
        return $this->orderManager->getTable()
                        ->select('*')
                        ->whereOr([
                            OrderManager::COLUMN_BUYER => PersonManager::TABLE_NAME . '.' . PersonManager::COLUMN_ID,
                            PersonManager::TABLE_DETAIL_NAME . '.' . PersonManager::COLUMN_PERSON_DETAIL_ID,
                        ])
                        ->where(OrderManager::COLUMN_STATE . '!=?', OrderManager::STATE_CREATED)
                        ->order(OrderManager::COLUMN_CREATED . ' DESC')
                        ->fetchAll();
    }

    /**
     * Pomocná funkce pro nasetování hodnot stavů objednávky do selectu
     * @param DataGrid $column buňka datagridu
     */
    private function setStateColums($column) {
        $states = $this->orderManager->getPrettyOrderStates();
        foreach ($states as $key => $value) {
            if ($key == OrderManager::STATE_SENT) {
                $column->addOption($key, $value)
                        ->setClass('btn-success')
                        ->endOption();
            } else if ($key == OrderManager::STATE_SUSPENDED || $key == OrderManager::STATE_CANCELED) {
                $column->addOption($key, $value)
                        ->setClass('btn-danger')
                        ->endOption();
            } else if ($key == OrderManager::STATE_ACCEPTED) {
                $column->addOption($key, $value)
                        ->setClass('btn-primary')
                        ->endOption();
            } else {
                $column->addOption($key, $value)
                        ->setClass('btn-warning')
                        ->endOption();
            }
        }
    }

    /**
     * Změní stav objednávky, pokusí se odeslat mail a zpracovat nový stav (nastavit ho).
     * Po každé akci odesílám přáslušný response - bud výsledek, nebo chybovou zprávu pro ajaxové zpracování
     * @param int    $id    ID objednávky
     * @param string $state stav objednávky
     */
    public function setState($id, $state) {
        // Presenter
        $presenter = $this->presenter;
        // Nastavení stavu objednávky.
        $this->orderManager->setState($id, $state);
        $message = array();

        try {
            $stateTitle = array_search($state, $this->orderManager->getOrderStates());

            if ($state == OrderManager::STATE_ACCEPTED) {
                $message = 'eshop.order.changeStateAccepted';
            } elseif ($state == OrderManager::STATE_COMPLETED) {
                $message = 'eshop.order.changeStateAccepted';
            } elseif ($state == OrderManager::STATE_SENT) {
                $message = 'eshop.order.changeStateSent';
            } else {
                $message[0] = $stateTitle;
                $message[1] = 'eshop.order.changeState';
            }

            // Odeslání emailu.
            $order = $this->orderEmailService->sendOrderEmail($message, $presenter->createTemplate(), $presenter, $id);

            // Zaznamení akce do DB
            $presenter->logAction('Změna stavu objednávky ' . $order[OrderManager::COLUMN_NUMBER] . ' na ' . $stateTitle);

            // Zpráva pro úspěšnou změnu stavu a odeslání emailu
            $presenter->flashMessage($presenter->translator->translate('eshop.order.management.flashes.newStateSuccess'), MessageType::SUCCESS);
        } catch (SendException $e) {
            // Zpráva pro neúspěšnou změnu stavu a odeslání emailu
            $presenter->flashMessage($presenter->translator->translate('eshop.order.management.flashes.newStateError'), MessageType::ERROR);
        }
        /*
          if ($presenter->isAjax()) {
          // Překreslím notifikace
          $presenter->redrawControl('ajaxNotification');
          // Překreslení gridu
          $this['dataGrid']->reload();
          } else {
          $presenter->redirect('this');
          }
         */
        $presenter->redirect('this');
    }

    /**
     * Vyrenderuje PDF faktury.
     * @param int $e_order_id ID objednávky
     * @throws BadRequestException Jestliže se danou objednávku nepodařilo najít.
     */
    public function handlePdf($e_order_id) {
        // Vytváření šablony pro PDF.
        /** @var Template $template */
        $template = $this->createTemplate();
        $template->setFile(dirname(__FILE__) . self::PDF_TEMPLATE);

        $order = $this->orderService->fillTemplateWithOrderDetail($template, $e_order_id);

        // Render PDF.
        $mPdf = new mPDF();
        // parametr 1 (předání CSS)
        $mPdf->WriteHTML(file_get_contents(dirname(__FILE__) . '/../../../../../www/css/global/invoice.css'), 1);
        // parametr 2 (předání Latte šablony)
        $mPdf->WriteHTML($template, 2);

        // PDF se vygeneruje metodou Output(), kde 2. parametr - 'D' = vynutí stažení faktury na místo zobrazení v prohlížeči.
        $mPdf->Output($this->presenter->translator->translate('eshop.order.invoice') . '_' . $order[OrderManager::COLUMN_NUMBER] . '.pdf', 'D');
        $this->terminate();
    }

}

interface IOrderDataGridFactory {

    /**
     * @return OrderDataGridControl
     */
    public function create();
}
