<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Components\DataGrids;
use App\EshopModule\Model\CategoryManager;
use App\EshopModule\Model\ProductManager;
use App\Model\Enum\Status;
use Nette\Application\UI\Control;
use Ublaboo\DataGrid\DataGrid;
use App\Model\Enum\MessageType;

/**
 * Komponenta datagridu pro správu produktů
 * @package App\EshopModule\Components
 *
 * @author Petr Vojáček <petr at ideatech.cz>
 */
class ProductDataGridControl extends Control
{

    /** Cesta k souboru šablony pro tuto komponentu. */
    const TEMPLATE = '/../../../../templates/components/DataGrids/dataGrid.latte';
    const TEMPLATE_IMAGE_COL = __DIR__ . '/../../templates/components/DataGrids/ProductDataGrid/image.latte';
    const TEMPLATE_TYPE_COL = __DIR__ . '/../../templates/components/DataGrids/ProductDataGrid/type.latte';
    const TEMPLATE_STOCK_COL = __DIR__ . '/../../templates/components/DataGrids/ProductDataGrid/stock.latte';

    /** @var ProductManager Instance třídy modelu pro práci s produkty. */
    private $productManager;

    /** @var CategoryManager Instance třídy modelu pro práci s kategoriemi. */
    private $categoryManager;


    public function __construct(ProductManager $productManager, CategoryManager $categoryManager)
    {
        parent::__construct();
        $this->productManager = $productManager;
        $this->categoryManager = $categoryManager;
    }

    /** Vykreslí komponentu formuláře pro správu osob. */
    public function render()
    {
        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE);

        // Vykreslí komponentu.
        $this->template->render();
    }

    public function createComponentDataGrid($name)
    {
        $grid = new DataGrid($this, $name);
        /** Localization */
        $grid->setTranslator($this->presenter->translator);
        $grid->setPrimaryKey(ProductManager::COLUMN_URL);
        $grid->setDataSource($this->productManager->getTable()
            ->where(ProductManager::COLUMN_STATUS . " != ?", Status::HIDDEN)
            ->where(ProductManager::COLUMN_TYPE, [ProductManager::TYPE_SIMPLE, ProductManager::TYPE_VARIABLE]));

        $grid->addColumnText('images_count', 'Obrázek')
            ->setTemplate(self::TEMPLATE_IMAGE_COL, []);

        $grid->addColumnText(ProductManager::COLUMN_TITLE, 'Název')
            ->setSortable();
        $grid->addFilterText(ProductManager::COLUMN_TITLE, 'Název');

        $grid->addColumnText(ProductManager::COLUMN_TYPE, 'Typ produktu')
            ->setSortable()
            ->setTemplate(self::TEMPLATE_TYPE_COL, []);

        $grid->addColumnText(ProductManager::COLUMN_CODE, 'Kód produktu')
            ->setSortable();
        $grid->addFilterText(ProductManager::COLUMN_CODE, 'Kód produktu');

        $grid->addColumnText(ProductManager::COLUMN_STATUS, 'Status')
            ->setSortable()
            ->setReplacement(Status::$STATUSES);
        $grid->addFilterSelect(ProductManager::COLUMN_STATUS, 'Status:', Status::$STATUSES);

        $grid->addColumnNumber(ProductManager::COLUMN_PRICE, 'Cena')
            ->setSortable()
            ->setFormat(2, ',', '.');

        $grid->addColumnNumber(ProductManager::COLUMN_STOCK, 'Sklad')
            ->setSortable()
            ->setEditableCallback([$this, 'columnStockEdited'])
            ->setTemplate(self::TEMPLATE_STOCK_COL, []);

        $grid->addGroupAction('Smazat produkty')->onSelect[] = [$this, 'deleteProducts'];

        /**
         * Odkaz na editaci
         */
        $grid->addAction('edit', '', ':Eshop:Product:manage')
            ->setIcon('pencil');

        /**
         * Mazani
         */
        $grid->addAction('delete', '', 'deleteProduct!')
            ->setIcon('trash')
            ->setTitle('Smazat')
            ->setClass('btn btn-xs btn-danger ajax')
            ->setConfirm('Opravdu chcete smazat stránku %s?', 'title');

        // Tlačítko pro vložení nového productu
        $grid->addToolbarButton(':Eshop:Product:manage', 'eshop.product.newProduct')
            ->setClass('btn btn-xs btn-default');


        return $grid;
    }

    /**
     * Odstranění produktu.
     * @param string $url URL produktu
     */
    public function handleDeleteProduct($url)
    {
        $this->productManager->removeProduct($url);
        // Zaznamení akce do DB
        $this->presenter->logAction('Smazání produktu s url ' . $url);
        $this->presenter->flashMessage("Produkt s URL:[$url] byl smazán", MessageType::SUCCESS);
        if ($this->presenter->isAjax()) {
            $this->presenter->redrawControl('ajaxNotification');
            $this['dataGrid']->reload();
        } else {
            $this->presenter->redirect('this');
        }
    }

    /**
     * Hromadné odstranění produktů.
     * @param array $urls Pole URL produktu ke smazani
     */
    public function deleteProducts($urls = [])
    {
        foreach ($urls as $url) {
            $this->productManager->removeProduct($url);
        }
        $urlList = implode(',', $urls);
        // Zaznamení akce do DB
        $this->presenter->logAction('Hromadné smazání produktů s url ' . $urlList);
        $this->presenter->flashMessage("Produkty s URL:[$urlList] byly smazány", MessageType::SUCCESS);
        if ($this->presenter->isAjax()) {
            $this->presenter->redrawControl('ajaxNotification');
            $this['dataGrid']->reload();
        } else {
            $this->presenter->redirect('this');
        }
    }

    public function columnStockEdited($url, $value)
    {
        $product = $this->productManager->getProduct($url);
        $stock = $product->stock;
        if ($value - $stock <= 0) {
            $this->productManager->addToStock($product->product_id, $value - $stock);
            // Zaznamení akce do DB
            $this->presenter->logAction('Naskladnění produktu ' . $url . ' na hodnotu ' . $value - $stock);
        } else {
            $this->productManager->removeFromStock($product->product_id, $stock - $value);
            // Zaznamení akce do DB
            $this->presenter->logAction('Odebrání produktů ' . $url . ' ze skladu na hodnotu ' . $value - $stock);
        }
    }

}

interface IProductDataGridFactory
{
    /**
     * @return ProductDataGridControl
     */
    public function create();
}