<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Components\DataGrids;

use App\EshopModule\Model\ProductAttributeManager;
use App\EshopModule\Model\ProductManager;
use App\Model\Enum\Status;
use Nette\Application\UI\Control;
use Ublaboo\DataGrid\DataGrid;
use App\Model\Enum\MessageType;

/**
 * Komponenta datagridu pro správu produktů
 * @package App\EshopModule\Components
 *
 * @author Petr Vojáček <petr at ideatech.cz>
 */
class ProductVariantsDataGridControl extends Control {

    /** Cesta k souboru šablony pro tuto komponentu. */
    const TEMPLATE = '/../../../../templates/components/DataGrids/dataGrid.latte';

    /** @var ProductManager Instance třídy modelu pro práci s produkty. */
    private $productManager;

    /** @var ProductAttributeManager Instance třídy modelu pro práci s atributy produktů. */
    private $productAttributeManager;

    public function __construct(ProductManager $productManager, ProductAttributeManager $productAttributeManager) {
        parent::__construct();
        $this->productManager = $productManager;
        $this->productAttributeManager = $productAttributeManager;
    }

    /** Vykreslí komponentu formuláře pro správu osob. */
    public function render() {
        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE);

        // Vykreslí komponentu.
        $this->template->render();
    }

    public function createComponentDataGrid($name) {
        $grid = new DataGrid($this, $name);
        $grid->setPrimaryKey(ProductManager::COLUMN_ID);
        $grid->setDataSource($this->productManager->getTable()
                        ->where(ProductManager::COLUMN_STATUS . " != ?", Status::HIDDEN)
                        ->where(ProductManager::COLUMN_PARENT, $this->presenter->productId));

        $grid->addColumnText(ProductManager::COLUMN_TITLE, 'Název')
                ->setSortable()
                ->setRenderer(function ($item) {
                    $attributes = $this->productAttributeManager->getProductAttributes($item->product_id);
                    $str = '';
                    foreach ($attributes as $attribute) {
                        $str .= $attribute->attribute->name . ': ' . $attribute->attribute_values->value;
                    }

                    return $item->title . ' - ' . $str;
                });


        $grid->addColumnNumber(ProductManager::COLUMN_PRICE, 'Cena')
                ->setSortable()
                ->setFormat(2, ',', '.');

        $grid->addColumnNumber(ProductManager::COLUMN_STOCK, 'Sklad')
                ->setSortable()
                ->setEditableCallback([$this, 'columnStockEdited']);

        $grid->addGroupAction('Smazat produkty')->onSelect[] = [$this, 'deleteProducts'];

        /**
         * Odkaz na editaci
         */
        $grid->addAction('edit', '', ':Eshop:ProductVariant:edit')
                ->setIcon('pencil');

        /**
         * Mazani
         */
        $grid->addAction('delete', '', 'deleteProduct!')
                ->setIcon('trash')
                ->setTitle('Smazat')
                ->setClass('btn btn-xs btn-danger ajax')
                ->setConfirm('Opravdu chcete smazat stránku %s?', 'title');

        /** Localization */
        $grid->setTranslator($this->presenter->translator);

        return $grid;
    }

    /**
     * Odstranění produktu.
     * @param string $product_id URL produktu
     */
    public function handleDeleteProduct($product_id) {
        $this->productManager->removeVariation($product_id);
        // Zaznamení akce do DB
        $this->presenter->logAction('Odstranění varianty produktu ' . $product_id);
        $this->presenter->flashMessage("Varianta s ID:[$product_id] byla smazána", MessageType::SUCCESS);
        if ($this->presenter->isAjax()) {
            $this->presenter->redrawControl('flashes');
            $this['DataGrid']->reload();
        } else {
            $this->presenter->redirect('this');
        }
    }

    /**
     * Hromadné odstranění produktů.
     * @param array $ids Pole URL produktu ke smazani
     */
    public function deleteProducts($ids = []) {
        foreach ($ids as $id) {
            $this->productManager->removeVariation($id);
        }
        $idList = implode(',', $ids);
        // Zaznamení akce do DB
        $this->presenter->logAction('Odstranění variant produktu ' . $idList);
        $this->presenter->flashMessage("Varianty s ID:[$idList] byly smazány", MessageType::SUCCESS);
        if ($this->presenter->isAjax()) {
            $this->presenter->redrawControl('flashes');
            $this['DataGrid']->reload();
        } else {
            $this->presenter->redirect('this');
        }
    }

    public function columnStockEdited($id, $value) {
        $product = $this->productManager->getProductFromId($id);
        $stock = $product->stock;
        if ($value - $stock <= 0) {
            $this->productManager->addToStock($product->product_id, $value - $stock);
            // Zaznamení akce do DB
             $this->presenter->logAction('Naskladnění produktu ' . $id . ' na hodnotu ' . $value - $stock);
        } else {
            $this->productManager->removeFromStock($product->product_id, $stock - $value);
            // Zaznamení akce do DB
            $this->presenter->logAction('Odebrání produktů ' . $id . ' ze skladu na hodnotu ' . $value - $stock);
        }
    }

}

interface IProductVariantsDataGridFactory {

    /**
     * @return ProductVariantsDataGridControl
     */
    public function create();
}
