<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Components\Forms;

use App\Model\Enum\MessageType;
use Nette\Application\UI\Control;
use App\Forms\BaseFormFactory;
use Nette\Application\UI\Form;
use Nette\Utils\ArrayHash;
use App\EshopModule\Model\OrderManager;
use App\EshopModule\Model\ProductManager;
use Nette\Application\Responses\TextResponse;
use Nette\Http\IResponse;
use Nette\InvalidArgumentException;

/**
 * Class AddProductToOrderFormControl - formulář pro editaci objednávky - přidání produktu do objednávky
 * @package App\EshopModule\Forms
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class AddProductToOrderFormControl extends Control
{

    const TEMPLATE = '/../../templates/components/Forms/form.latte';

    /**
     * @var BaseFormFactory
     */
    private $formFactory;

    /**
     * @var OrderManager
     */
    private $orderManager;

    /**
     * @var ProductManager
     */
    private $productManager;

    /**
     * @param BaseFormFactory $baseFormFactory
     * @param OrderManager $orderManager
     * @param ProductManager $productManager
     */
    public function __construct(BaseFormFactory $baseFormFactory, OrderManager $orderManager, ProductManager $productManager)
    {
        parent::__construct();
        $this->formFactory = $baseFormFactory;
        $this->orderManager = $orderManager;
        $this->productManager = $productManager;
    }

    public function setDefaults($values)
    {
        $this['form']->setDefaults($values);
    }

    public function render($formPath = null)
    {
        $this->template->setFile(__DIR__ . self::TEMPLATE);
        $this->template->formPath = $formPath;
        // Vykreslí komponentu.
        $this->template->render();
    }

    /**
     * Vrátí formulář pro přidání produktu v editaci objednávky.
     * @return Form formulář k přidání produktu
     */
    protected function createComponentForm()
    {
        $form = $this->formFactory->create();
        $form->getElementPrototype()->addAttributes(['class' => 'ajax']);
        $form->addGroup($this->presenter->translator->translate('eshop.order.form.addProduct.group.title'));
        $form->addHidden('e_order_id');
        $form->addSelect('product_id', 'eshop.order.form.addProduct.label.product', $this->productManager->getProductList())
            ->setRequired();
        $form->addText('quantity', 'eshop.order.form.addProduct.label.quantity')->setType('number')
            ->setRequired()
            ->addRule(Form::INTEGER)
            ->setValue(1);
        $form->setCurrentGroup();

        $form->addSubmit('add', 'eshop.order.form.addProduct.label.submit');

        $form->onSuccess[] = function (Form $form, ArrayHash $values) {
            $this->succeeded($form, $values);
        };

        return $form;
    }

    /**
     * Funkce se vykonaná při úspěšném odeslání formuláře k přidání produktu.
     * @param Form $form formulář k přidání produktu
     * @param ArrayHash $values odeslané hodnoty formuláře
     */
    private function succeeded(Form $form, ArrayHash $values)
    {
        $presenter = $this->presenter;
        $product = $this->productManager->getProductFromId($values->product_id);
        if ($product->{ProductManager::COLUMN_STOCK} <= 0) {

        }

        try {
            $success = $this->orderManager->addProducts($values->product_id, $values->quantity, $values->e_order_id);

            if (!$success) {
                if ($presenter->isAjax()) {
                    $presenter->getHttpResponse()->setCode(IResponse::S500_INTERNAL_SERVER_ERROR);
                    $presenter->sendResponse(new TextResponse('eshop.order.flashes.productSaveError'));
                } else {
                    $presenter->flashMessage($this->presenter->translator->translate('eshop.order.flashes.productSaveError'), MessageType::ERROR);
                }
            }
            // Zaznamení akce do DB
            $presenter->logAction('Přidání ' . $values[OrderManager::COLUMN_QUANTITY] . ' produktů ' . $values[ProductManager::COLUMN_ID] . ' do objednávky' . $values[OrderManager::COLUMN_ID]);
        } catch (InvalidArgumentException $e) {
            if ($presenter->isAjax()) {
                $presenter->getHttpResponse()->setCode(IResponse::S400_BAD_REQUEST);
                $presenter->sendResponse(new TextResponse($e->getMessage()));
            } else {
                $presenter->flashMessage($e->getMessage(), MessageType::ERROR);
            }
        }

        if ($presenter->isAjax()) {
            $presenter->redrawControl('products');
            $presenter->redrawControl('summary');
        } else {
            $presenter->flashMessage($this->presenter->translator->translate('eshop.order.flashes.productSaveSuccess'), MessageType::SUCCESS);
            $presenter->redirect('this');
        }
    }

}

interface IAddProductToOrderFormFactory
{
    /**
     * @return AddProductToOrderFormControl
     */
    public function create();
}

