<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Components\Forms;
use App\EshopModule\Model\CategoryManager;
use App\Forms\BaseFormFactory;
use App\Model\Enum\MessageType;
use Nette\Application\Responses\JsonResponse;
use Nette\Application\UI\Control;
use Nette\Application\UI\Form;
use Nette\Database\Table\ActiveRow;
use Nette\Database\UniqueConstraintViolationException;
use Nette\Utils\ArrayHash;
use Nette\Utils\Json;

/**
 * Komponenta pro vytváření kategorií pro články
 * @package App\EshopModule\Components
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class CategoryEditorFormControl extends Control
{

    /** Cesta k souborům šablon pro tuto komponentu. */
    const TEMPLATE_PATH = '/../../templates/components/Forms/CategoryEditorForm/';

    /**
     * @var CategoryManager
     */
    private $categoryManager;

    /**
     * @var BaseFormFactory
     */
    private $baseFormFactory;

    /**
     * @param CategoryManager $categoryManager
     * @param BaseFormFactory $baseFormFactory
     */
    public function __construct(CategoryManager $categoryManager, BaseFormFactory $baseFormFactory)
    {
        parent::__construct();
        $this->categoryManager = $categoryManager;
        $this->baseFormFactory = $baseFormFactory;
    }

    /**
     * Výchozí metoda pro vykreslení komponenty
     */
    public function render($formPath = null)
    {
        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE_PATH . 'form.latte');
        $this->template->formPath = $formPath;
        // Vykreslí komponentu.
        $this->template->render();
    }

    /**
     * Vytváří a vrací komponentu formuláře pro editaci kategorií produktů.
     * @return Form formulář pro editaci kategorií produktů
     */
    public function createComponentCategoryEditorForm()
    {
        $form = $this->baseFormFactory->create();
        // Do textového inputu se doplní přes json data - kategori
        $form->addText('output');
        $form->addSubmit('submit');

        $form->onSuccess[] = function (Form $form, ArrayHash $values) {
            $this->succeeded($form, $values);
        };

        return $form;
    }

    /**
     * Funkce se vykonává při úspěšném odeslání formuláře pro editaci kategorií produktů.
     * V odeslaném formuláři příjde rovnou JSON, který se dekóduje a kategorie, které obsahuje se uloží
     * @param Form      $form   formulář pro editaci kategorií produktů
     * @param ArrayHash $values odeslané hodnoty formuláře
     */
    public function succeeded(Form $form, ArrayHash $values) {
        try {
            if (($categories = Json::decode($values->output, Json::FORCE_ARRAY)) && $this->categoryManager->saveCategories($categories)) {
                $this->presenter->flashMessage($this->presenter->translator->translate('eshop.category.flashes.success'), MessageType::SUCCESS);
            } else {
                $this->presenter->flashMessage($this->presenter->translator->translate('eshop.category.flashes.errorSave'), MessageType::ERROR);
            }
        } catch (UniqueConstraintViolationException $ex) {
            $this->presenter->flashMessage($this->presenter->translator->translate('eshop.category.flashes.errorUrl'), MessageType::ERROR);
        }
        $this->redirect('this');
    }

    public function setDefaults(array $values)
    {
        $this['categoryEditorForm']->setDefaults($values);
    }

    /** Vrací JSON s kategoriemi. */
    public function handleGetJson()
    {
        $categoriesArray = [];

        $categories = $this->categoryManager->getAllCategories();

        array_walk($categories, function (ActiveRow $category) use (&$categoriesArray) {
            $categoriesArray[] = $category->toArray();
        });

        $this->presenter->sendResponse(new JsonResponse($categoriesArray));
    }

}

/**
 * Interface ICategoryEditorFormFactory
 * @package App\EshopModule\Components
 */
interface ICategoryEditorFormFactory {

    /**
     * @return CategoryEditorFormControl
     */
    public function create();
}
