<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Components\Forms;

use App\EshopModule\Model\OrderManager;
use App\Model\Enum\MessageType;
use Nette\Application\UI\Control;
use Nette\Application\UI\Form;
use Nette\Utils\ArrayHash;
use App\Forms\BaseFormFactory;
use App\Services\FormatHelperService;

/**
 * Class OrderFormControl - formulář pro editaci objednávky - fakturační údaje
 * @package App\EshopModule\Forms
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class OrderFormControl extends Control {

    const TEMPLATE = '/../../templates/components/Forms/form.latte';

    /** Konstanty pro práci s daty v místním formuláři. */
    const
            FORM_PLACEHOLDER = 'dd.mm.yyyy',
            FORM_PATTERN = '[0-3]?[0-9]\.[0-1]?[0-9]\.[0-9]{4}';

    /**
     * @var BaseFormFactory
     */
    private $formFactory;

    /**
     * @var OrderManager
     */
    private $orderManager;

    /**
     * @var FormatHelperService Pomocná služba pro formátování data, jména adresy atd.
     */
    private $formatHelperService;

    /**
     * @param BaseFormFactory  $baseFormFactory
     * @param OrderManager $orderManager
     * @param FormatHelperService $formatHelperService
     */
    public function __construct(BaseFormFactory $baseFormFactory, OrderManager $orderManager, FormatHelperService $formatHelperService) {
        parent::__construct();
        $this->formFactory = $baseFormFactory;
        $this->orderManager = $orderManager;
        $this->formatHelperService = $formatHelperService;
    }

    public function setDefaults($values)
    {
        $this['form']->setDefaults($values);
    }

    public function render($formPath = null)
    {
        $this->template->setFile(__DIR__ . self::TEMPLATE);
        $this->template->formPath = $formPath;
        // Vykreslí komponentu.
        $this->template->render();
    }


    /**
     * Vrátí formulář k editaci objednávky - fakturační údaje.
     * @return Form formulář k editaci objednávky
     */
    protected function createComponentForm() {
        $form = $this->formFactory->create();

        $form->addGroup($this->presenter->translator->translate('eshop.order.form.orderForm.group.title'));
        $form->addHidden('e_order_id');
        $form->addText('issued', 'eshop.order.form.orderForm.label.issueDate')
                ->setRequired()
                ->setAttribute('placeholder', self::FORM_PLACEHOLDER)
                ->addRule(Form::PATTERN, 'eshop.common.patterns.date', self::FORM_PATTERN);
        $form->addText('due_date', 'eshop.order.form.orderForm.label.dueDate')
                ->setRequired()
                ->setAttribute('placeholder', self::FORM_PLACEHOLDER)
                ->addRule(Form::PATTERN, 'eshop.common.patterns.date', self::FORM_PATTERN);
        $form->setCurrentGroup();

        $form->addSubmit('change', 'eshop.order.form.orderForm.label.submit');

        $form->onSuccess[] = function (Form $form, ArrayHash $values) {
            $this->succeeded($form, $values);
        };

        return $form;
    }

    /**
     * Funkce se vykonaná při úspěšném odeslání formuláře k editaci objednávky.
     * @param Form      $form   formulář k editaci objednávky
     * @param ArrayHash $values odeslané hodnoty formuláře
     */
    private function succeeded(Form $form, ArrayHash $values) {
        $presenter = $form->getPresenter();
        $values['issued'] = $this->formatHelperService->formatDate($values['issued']);
        $values['due_date'] = $this->formatHelperService->formatDate($values['due_date']);

        $this->orderManager->updateOrder($values);
        
        // Zaznamení akce do DB
        $presenter->logAction('Editace fakturačních údajů objednávky ' . $values[OrderManager::COLUMN_ID]);
        // Info o změně
        $presenter->flashMessage($this->presenter->translator->translate('eshop.order.flashes.orderSaveSuccess'), MessageType::SUCCESS);

        if ($presenter->isAjax()) {
            $presenter->redrawControl("orderForm");
            $presenter->redrawControl("ajaxNotification");
        } else {
            $presenter->redirect('this');
        }
    }

}

interface IOrderFormFactory
{
    /**
     * @return OrderFormControl
     */
    public function create();
}
