<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Components\Forms;

use App\EshopModule\Model\PaymentMethodsManager;
use App\MediaModule\Model\FileRelationsManager;
use App\MediaModule\Model\FileManager;
use App\Model\Enum\MessageType;
use Nette\Application\UI\Control;
use Nette\Application\UI\Form;
use Nette\Utils\ArrayHash;
use App\Forms\BaseFormFactory;
use Nette\Database\UniqueConstraintViolationException;

/**
 * Class PaymentMethodsFormControl - formulář pro práci s metodami platby
 * @package App\EshopModule\Forms
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class PaymentMethodsFormControl extends Control {

    const TEMPLATE = '/../../templates/components/Forms/form.latte';

    /**
     * @var PaymentMethodsManager Instance třídy modelu pro práci s produkty.
     */
    private $paymentMethodsManager;

    /**
     * @var FileManager
     */
    private $fileManager;

    /**
     * @var FileRelationsManager
     */
    private $fileRelationsManager;

    /**
     * @var BaseFormFactory Továrnička na formuláře. 
     */
    private $formFactory;

    /**
     * @param PaymentMethodsManager  $paymentMethodsManager
     * @param BaseFormFactory $baseFormFactory
     * @param FileRelationsManager $fileRelationsManager
     * @param FileManager $fileManager
     */
    public function __construct(PaymentMethodsManager $paymentMethodsManager, BaseFormFactory $baseFormFactory, FileRelationsManager $fileRelationsManager, FileManager $fileManager) {
        parent::__construct();
        $this->paymentMethodsManager = $paymentMethodsManager;
        $this->formFactory = $baseFormFactory;
        $this->fileRelationsManager = $fileRelationsManager;
        $this->fileManager = $fileManager;
    }

    public function setDefaults($values) {
        $this['form']->setDefaults($values);
    }

    public function render($formPath = null) {
        $this->template->setFile(__DIR__ . self::TEMPLATE);
        $this->template->formPath = $formPath;
        // Vykreslí komponentu.
        $this->template->render();
    }

    /**
     * Vytváří a vrací komponentu formuláře pro přidání a editování metody platby
     * @return Form formulář pro správu metod dopravy
     */
    protected function createComponentForm() {
        $form = $this->formFactory->create();

        $form->addHidden('payment_method_id');

        $form->addText('payment_name', 'eshop.orderMethods.form.name')
                ->setRequired()
                ->getControlPrototype()
                ->addClass('form-control');

        $form->addText('payment_price', 'eshop.orderMethods.form.price')
                ->setType('number')
                ->setRequired()
                ->setDefaultValue(0)->addRule(Form::FLOAT)
                ->getControlPrototype()
                ->addClass('form-control');

        $form->addUpload('image', 'eshop.orderMethods.form.image')
                ->setAttribute('accept', 'image/*')
                ->addRule(Form::IMAGE, 'eshop.common.patterns.images')
                ->addRule(Form::MAX_FILE_SIZE, 'eshop.common.patterns.imageSize', 1024 * 1024 /* v bytech */)
                ->setRequired(FALSE);

        $form->addSubmit('submit', 'eshop.common.btns.send')
                ->getControlPrototype();


        $form->onSuccess[] = function (Form $form, ArrayHash $values) {
            $this->succeeded($form, $values);
        };

        return $form;
    }

    /**
     * Funkce se vykonaná při úspěšném odeslání formuláře pro správu dorpavních metod.
     * @param Form $form formulář pro správu produktu
     * @param ArrayHash $values odeslané hodnoty formuláře
     */
    private function succeeded(Form $form, ArrayHash $values) {
        $presenter = $form->getPresenter();

        $values->payment_method_id = $values->payment_method_id ? : NULL;

        $image = $values->image;
        unset($values->image);

        try {
            $payment_method = $this->paymentMethodsManager->saveValues(PaymentMethodsManager::TABLE_NAME, PaymentMethodsManager::COLUMN_ID, $values);
            $id = $payment_method[PaymentMethodsManager::COLUMN_ID];

            // Zaznamení akce do DB
            $presenter->logAction('Vytvoření/Editace způsobu platby ' . $values[PaymentMethodsManager::COLUMN_NAME]);

            $presenter->flashMessage($presenter->translator->translate('eshop.orderMethods.flashes.saveSuccess', ['methodName' => $values->payment_name]), MessageType::SUCCESS);
        } catch (UniqueConstraintViolationException $ex) {
            $presenter->flashMessage($presenter->translator->translate('eshop.orderMethods.flashes.errorUrl'), MessageType::ERROR);
        }

        if (isset($image) && !is_null($image->getContentType())) {
            $this->fileRelationsManager->removeEntityImages(PaymentMethodsManager::COLUMN_ID, $id);
            $ids = $this->fileManager->uploadFiles([$image]);
            foreach ($ids as $fileId) {
                $this->fileRelationsManager->assignImage($fileId, PaymentMethodsManager::COLUMN_ID, $id);
                $this->fileRelationsManager->setMain(PaymentMethodsManager::COLUMN_ID, $id, $fileId);
            }
        }

        // Nepoužívat ajax, protože kdž edituju produkt, tak ho skrývám a vytvářím nový kvůli fkaturám - potřebuji reload stránky
        if ($presenter->isAjax()) {
            $presenter->redrawControl('ajaxNotification');
            $presenter->redrawControl("methodsSnippet");
        } else {
            $presenter->redirect('PaymentMethods:management');
        }
    }

}

interface IPaymentMethodsFormFactory {

    /**
     * @return PaymentMethodsFormControl
     */
    public function create();
}
