<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Components\Forms;

use App\EshopModule\Model\CategoryManager;
use App\EshopModule\Model\ProductManager;
use App\Forms\BaseFormFactory;
use App\MediaModule\Model\FileManager;
use App\Model\Enum\MessageType;
use Nette\Application\UI\Control;
use Nette\Application\UI\Form;
use Nette\Database\UniqueConstraintViolationException;
use Nette\Utils\ArrayHash;
use Nette\Utils\DateTime;

/**
 * Class ProductFormControl
 * @package App\EshopModule\Forms
 *
 * @author Petr Vojáček <petr at ideatech.cz>
 */
class ProductFormControl extends Control {

    /** Cesta k souboru šablony pro tuto komponentu. */
    const TEMPLATE = '/../../templates/components/Forms/form.latte';

    /** @var BaseFormFactory Továrnička na formuláře. */
    private $formFactory;

    /**
     * @var ProductManager
     */
    private $productManager;

    /**
     * @var CategoryManager
     */
    private $categoryManager;

    /**
     * @var \App\BlogModule\Model\CategoryManager
     */
    private $blogCategoryManager;

    /**
     * @var FileManager
     */
    private $fileManager;

    /**
     * ProductFormControl constructor.
     * @param ProductManager $productManager
     * @param CategoryManager $categoryManager
     * @param FileManager $fileManager
     * @param BaseFormFactory $baseFormFactory
     * @param \App\BlogModule\Model\CategoryManager $blogCategoryManager
     */
    public function __construct(ProductManager $productManager, CategoryManager $categoryManager, FileManager $fileManager, BaseFormFactory $baseFormFactory, \App\BlogModule\Model\CategoryManager $blogCategoryManager) {
        parent::__construct();
        $this->productManager = $productManager;
        $this->categoryManager = $categoryManager;
        $this->fileManager = $fileManager;
        $this->formFactory = $baseFormFactory;
        $this->blogCategoryManager = $blogCategoryManager;
    }

    /**
     * @param null $formPath
     */
    public function render($formPath = null) {
        // Nastaví šablonu komponenty.
        $this->template->setFile(__DIR__ . self::TEMPLATE);
        $this->template->formPath = $formPath;
        // Vykreslí komponentu.
        $this->template->render();
    }

    /**
     * Vytváří a vrací komponentu formuláře pro hodnocení produktu.
     * @return Form formulář pro hodnocení produktu
     */
    protected function createComponentForm() {
        $form = $this->formFactory->create();

        $form->addText('title', 'Název produktu')
                ->setRequired();

        // Form::URL funguje pouze pro absolutní adresy.
        $form->addText('url', 'URL adresa')
                ->setRequired()
                ->addRule(Form::PATTERN, null, '[A-Za-z0-9\\-]+');


        $form->addHidden('product_id');
        $form->addText('code', 'Kód produktu');

        $form->addSelect('type', 'Typ produktu', [
            'simple' => 'Jednoduchý produkt',
            'variable' => 'Produkt s variantami'
        ]);


        $form->addText('short_description', 'Krátký popis produktu')
                ->setRequired();

        $form->addTextArea('description', '')
                ->getControlPrototype()
                ->addClass('form-control');

        $form->addMultiSelect('categories', 'Kategorie', $this->categoryManager->getCategoryLeafs())
                ->setRequired()->getControlPrototype()->addClass('form-control');

        $form->addText('cart_label', 'Popisek v košíku/množství');

        $form->addText('price', 'Cena')
                ->setRequired()
                ->setDefaultValue(0)->addRule(Form::FLOAT);

        $form->addSelect('blog_category', 'Kategorie receptů', $this->blogCategoryManager->getCategoryLeafs());

        $form->addCheckbox('is_action', 'Akce')->getControlPrototype()->addClass('form-control');
        $form->addCheckbox('is_new', 'Novinka')->getControlPrototype()->addClass('form-control');
        $form->addCheckbox('is_orderable', 'Dá se objednat')->getControlPrototype()->addClass('form-control');

        $form->addText('old_price', 'Cena před slevou')
                ->setDefaultValue(0)
                ->addRule(Form::FLOAT)
                ->setRequired(FALSE);

        $form->addUpload('sizes_table', 'Defaultní obrázek')
                ->setAttribute('accept', 'image/*')
                ->addRule(Form::IMAGE, 'eshop.common.patterns.images')
                ->addRule(Form::MAX_FILE_SIZE, 'eshop.common.patterns.imageSize', 1024 * 1024 /* v bytech */)
                ->setRequired(FALSE);

        $form->addSubmit('submit', 'Odeslat')
                ->getControlPrototype()
                ->setAttribute('id', 'productFormSubmit');

        $form->onSuccess[] = function (Form $form, ArrayHash $values) {
            $this->succeeded($form, $values);
        };

        return $form;
    }

    private function succeeded(Form $form, ArrayHash $values) {
        $presenter = $form->getPresenter();
        $categories = $values->categories;
        $values->offsetUnset('categories');

        $values->product_id = $values->product_id ? : NULL;

        if (isset($values->sizes_table) && $values->sizes_table->hasFile()) {
            $date = new DateTime();
            $filename = 'uploads/' . $date->format('Y') . '/' . $date->format('m') . '/' . $this->fileManager->uploadFile($values->sizes_table);
        } else {
            $values->offsetUnset('sizes_table');
        }

        try {
            if (isset($filename)) {
                $values->sizes_table = $filename;
            }

            $productId = $this->productManager->saveProduct($values);

            // Zaznamení akce do DB
            $presenter->logAction('Vytvoření/Editace produktu ' . $values[ProductManager::COLUMN_URL]);

            $this->categoryManager->updateProductCategories($productId, $categories);

            $presenter->flashMessage('Produkt byl úspěšně uložen.', MessageType::SUCCESS);
            $presenter->redirect(':Eshop:Product:manage', $values->url);
        } catch (UniqueConstraintViolationException $ex) {
            $presenter->flashMessage('Produkt s touto URL adresou již existuje.', MessageType::ERROR);
        }
    }

    public function setDefaults(array $defaults) {
        $this['form']->setDefaults($defaults);
    }

}

interface IProductFormFactory {

    /**
     * @return ProductFormControl
     */
    public function create();
}
