<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Components\Forms;

use App\Model\Enum\MessageType;
use Nette\Application\UI\Control;
use Nette\Application\UI\Form;
use Nette\Utils\ArrayHash;
use App\Forms\BaseFormFactory;
use App\Model\PersonManager;
use App\EshopModule\Model\SettingsManager;
use App\Services\FormatHelperService;

/**
 * Class SettingsFormControl - formulář pro editaci objednávky - fakturační údaje
 * @package App\EshopModule\Forms
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class SettingsFormControl extends Control {

    const TEMPLATE = '/../../templates/components/Forms/form.latte';

    /** Konstanty pro práci s místním formulářem. */
    const
            FORM_PLACEHOLDER = 'dd.mm.yyyy',
            FORM_PATTERN = '[0-3]?[0-9]\.[0-1]?[0-9]\.[0-9]{4}';

    /** @var BaseFormFactory Továrnička na formuláře. */
    private $formFactory;

    /**
     * @var SettingsManager Instance třídy modelu pro práci s účetním nastavením v e-shopu.
     */
    private $settingsManager;

    /**
     * @var PersonManager Instance třídy modelu pro práci s osobami v e-shopu. 
     */
    private $personManager;

    /**
     * @var FormatHelperService Pomocná služba pro formátování data, jména adresy atd.
     */
    private $formatHelperService;

    /**
     * @param BaseFormFactory $baseFormFactory
     * @param SettingsManager $settingsManager
     * @param PersonManager $personManager
     * @param FormatHelperService $formatHelperService
     */
    public function __construct(BaseFormFactory $baseFormFactory, SettingsManager $settingsManager, PersonManager $personManager, FormatHelperService $formatHelperService) {
        parent::__construct();
        $this->formFactory = $baseFormFactory;
        $this->settingsManager = $settingsManager;
        $this->personManager = $personManager;
        $this->formatHelperService = $formatHelperService;
    }

    public function setDefaults($values) {
        $this['form']->setDefaults($values);
    }

    public function render($formPath = null) {
        $this->template->setFile(__DIR__ . self::TEMPLATE);
        $this->template->formPath = $formPath;
        // Vykreslí komponentu.
        $this->template->render();
    }

    /**
     * Vytváří a vrací komponentu formuláře pro přidání účetního nastavení.
     * @return Form formulář pro přidání účetního nastavení
     */
    protected function createComponentForm() {
        $form = $this->formFactory->create();

        $form->addText('valid_from', 'eshop.settings.form.label.validFrom')
                ->setRequired()
                ->setAttribute('placeholder', self::FORM_PLACEHOLDER)
                ->addRule(Form::PATTERN, 'eshop.settings.form.patterns.date', self::FORM_PATTERN);

        $form->addText('valid_to', 'eshop.settings.form.label.validTo')
                ->setRequired()
                ->setAttribute('placeholder', self::FORM_PLACEHOLDER)
                ->addRule(Form::PATTERN, 'eshop.common.patterns.date', self::FORM_PATTERN);

        $form->addText('vat', 'eshop.settings.form.label.vat')
                ->setType('number')
                ->setRequired()
                ->setValue(21);

        //jen uzivatele, kteri jsou amdinistratori
        $form->addSelect('accountant_detail_id', 'eshop.settings.form.label.accountat', $this->personManager->getAccountants())
                ->setRequired();

        //jen administratori
        $form->addSelect('seller_id', 'eshop.settings.form.label.seller', $this->personManager->getSellers())
                ->setRequired();

        $form->addUpload('signature', 'eshop.settings.form.label.signature')
                ->setAttribute('accept', 'image/*')
                ->addRule(Form::IMAGE, 'eshop.common.patterns.images')
                ->addRule(Form::MAX_FILE_SIZE, 'eshop.common.patterns.imageSize', 1024 * 1024 /* v bytech */)
                ->setRequired(FALSE);
        $form->setCurrentGroup();

        $form->addSubmit('add', 'eshop.settings.form.label.submit');

        $form->onSuccess[] = function (Form $form, ArrayHash $values) {
            $this->succeeded($form, $values);
        };

        return $form;
    }

    /**
     * Funkce se vykonaná při úspěšném odeslání formuláře a přidá účetní nastavení.
     * @param Form $form formulář pro přidání účetního nastavení
     * @param ArrayHash $values odeslané hodnoty formuláře
     */
    private function succeeded($form, $values) {
        $presenter = $form->getPresenter();
        
        
        if (file_exists($values[SettingsManager::COLUM_SIGNATURE])) {
            $signature = $values->signature->toImage();
        } else {
            $signature = null;
        }
        
        unset($values['signature']);

        $values['valid_from'] = $this->formatHelperService->formatDate($values['valid_from']);
        $values['valid_to'] = $this->formatHelperService->formatDate($values['valid_to']);

        $this->settingsManager->addSettings($values, $signature);
        $presenter->flashMessage($presenter->translator->translate('eshop.settings.flashes.saveSuccess'), MessageType::SUCCESS);

        // Zaznamení akce do DB
        $presenter->logAction('Vytvoření účetního nastavení platného od ' . $values[SettingsManager::COLUMN_VALID_FROM] . ' do ' . $values[SettingsManager::COLUMN_VALID_TO] . ' s DPH ' . $values[SettingsManager::COLUMN_VAT]);

        if ($presenter->isAjax()) {
            $presenter->redrawControl('ajaxNotification');
            $presenter->redrawControl("settingsSnippet");
            $presenter->redrawControl("settingsForm");
        } else {
            $presenter->redirect('this');
        }
    }

}

interface ISettingsFormFactory {

    /**
     * @return SettingsFormControl
     */
    public function create();
}
