<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Model;

use App\Model\BaseManager;
use App\Model\Enum\Status;
use Nette\Database\Context;
use Nette\Database\Table\Selection;
use Nette\Utils\ArrayHash;
use Nette\Http\Request;
use Nette\Http\Response;
use Exception;

/**
 * Spravuje kurz a měny pro E-shop.
 * Nastavení kurzů se projeví v cenách produktů a objednávky.
 * Defaultně se nastavují ceny v české měně, takže kurz pro korunu by měl být 1.
 * 
 * @package App\EshopModule\Model
 * 
 * @author Jan Vodák <vodak at ideatech.cz>
 */
class CurrencyManager extends BaseManager {

    /** Konstanty pro manipulaci s modelem. */
    const
            TABLE_NAME = 'currency',
            COLUMN_ID = 'currency_id',
            COLUMN_NAME = 'currency_name',
            COLUMN_CODE = 'currency_code',
            COLUMN_EXCHANGE_RATE = 'exchange_rate',
            COLUMN_CURRENCY_SYMBOL = 'currency_symbol',
            COLUMN_STATUS = 'currency_status',
            COLUMN_DEFAULT_CURRENCY = 'default_currency';

    /** Konstanty pro manipulaci s cookies. */
    const
            COOKIE_NAME = 'currency',
            COOKIE_ID = 'currencyId';

    /** Doba platnosti nastavení měny ve dnech. */
    const CURRENCY_VALIDITY = 365;

    /** @var Response Odpověď na aktuální HTTP požadavek. */
    private $response;

    /** @var Request Aktuální HTTP požadavek. */
    private $request;

    /**
     * Konstruktor s injektovanou třídou pro práci s databází, aktuálním HTTP požadavkem a odpovědí na aktuální HTTP
     * požadavek.
     * @param Context        $database       automaticky injektovaná třída pro práci s databází
     * @param Request        $request        automaticky injektovaná třída aktuálního HTTP požadavku
     * @param Response       $response       automaticky injektovaná třída odpovědi na aktuální HTTP požadavek
     */
    public function __construct(Context $database, Request $request, Response $response) {
        parent::__construct($database);
        $this->request = $request;
        $this->response = $response;
    }

    /**
     * Vrátí měnu se všemi informace podle zadaného - hlavně nastavený kurz
     * @param int $currencyId id konkrétního kurzu
     * @return ArrayHash kornétní měnový kurz se všmi informacemi
     */
    public function getCurrency($currencyId) {
        return $this->database->table(self::TABLE_NAME)
                        ->where(self::COLUMN_ID, $currencyId)
                        ->fetch();
    }

    /**
     * Vrátí měnu se všemi informace podle zadaného - hlavně nastavený kurz
     * @param string $code kod pro konkrétní kurz
     * @return ArrayHash kornétní měnový kurz se všmi informacemi
     */
    public function getCurrencyByCode($code) {
        return $this->database->table(self::TABLE_NAME)
                        ->where(self::COLUMN_CODE, $code)
                        ->fetch();
    }

    /**
     * Vrátí všechny měnové kurzy, které jsou viditelné (nejsou skryté ani smazané)
     * @return Selection všechny měny, které se na v systému budou vyskytovat a jsou viditelné
     */
    public function getAllCurrencies() {
        return $this->database->table(self::TABLE_NAME)
                        ->where(self::COLUMN_STATUS, Status::PUBLISHED);
    }

    /**
     * Vrátí aktuální měnu, kterou získá z cookies.
     * Pokud žádnou nenajde, vrátí defaultní měnu
     * @return ArrayHash  kornétní měnový kurz se všmi informacemi
     */
    public function getActualCurrency() {
        $currencyId = null;

        // Kontroluji existenci cookie (uživatel ji posílá s každým požadavkem na server).
        $cookie = $this->request->getCookie(self::COOKIE_NAME);

        // Pokud existuje, rozbalí její hodnoty pomocí PHP funkce parse_str() - naplní definovanou proměnnou ($currencyId)
        if (isset($cookie)) {
            parse_str($cookie);
        }

        $currency = $this->database->table(self::TABLE_NAME)
                ->where(self::COLUMN_ID, $currencyId)
                ->where(self::COLUMN_STATUS, Status::PUBLISHED)
                ->fetch();

        // Případně vrátím defaultní, akutálně používanou měnu
        if (!$currency) {
            $currency = $this->getDefaultCurrency();
        }

        return $currency;
    }

    /**
     * Vrátí seznam všech měn pro formulář.
     * @return array seznam všech měn pro formulář
     */
    public function getCurrencyList() {
        return $this->database->table(self::TABLE_NAME)
                        ->select(self::COLUMN_ID . ',' . self::COLUMN_NAME)
                        ->where(self::COLUMN_STATUS, Status::PUBLISHED)
                        ->fetchPairs(self::COLUMN_ID, self::COLUMN_NAME);
    }
    
    /**
     * Vrátí defaultní měnu.
     * @return array seznam všech měn pro formulář
     */
    public function getDefaultCurrency(){
        // Najdu původní defaultí měnu a nastavím ji false
        return $this->database->table(self::TABLE_NAME)
                ->where(self::COLUMN_DEFAULT_CURRENCY)
                ->fetch();
    }

    /**
     * Přidá novou hlavní měnu
     * Původní defaultni měnu zruší a nastaví novou
     * @param array|ArrayHash $currency  údaje pro měnový kurz
     */
    public function saveDefaultCurrency($currency) {
        // Najdu původní defaultí měnu a nastavím ji false
        $this->getDefaultCurrency()
                ->update(array(
                    self::COLUMN_DEFAULT_CURRENCY => 0
        ));
        
        // Pro novou měnu nastavím true
        $this->database->table(self::TABLE_NAME)
                ->where(self::COLUMN_ID, $currency[self::COLUMN_DEFAULT_CURRENCY])
                ->update(array(
                    self::COLUMN_DEFAULT_CURRENCY => 1
        ));
        
        return $this->getCurrency($currency[self::COLUMN_DEFAULT_CURRENCY])[self::COLUMN_NAME];
    }

    /**
     * Přidá měnový kurz.
     * Pokud se edituje nějaký kurz, tak starý kurz nastavim na hidden, ale nesmažu ho kvůli fakturám -> metoda cleanCurrency
     * Ve fakturách potřebuji starý kurz, aby se neměli už staré ceny
     * @param array|ArrayHash $currency  údaje pro měnový kurz
     */
    public function saveCurrency($currency) {
        // Z nejakeho duvodu nefuunguje validace a posila mi to sem misto ID prazdny string
        if($currency[self::COLUMN_ID] == ''){
            $currency->offsetUnset(self::COLUMN_ID);
        }
        
        // Editace
        if (isset($currency[self::COLUMN_ID]) && ($currencyId = $currency[self::COLUMN_ID])) {
            // Odstranění staré měny, pokud se nikde jine v DB k němu nic nevztahuje - napr. faktura
            $this->cleanCurrency($currencyId);
            // Odstraním příchozí položku ID z formuláře
            unset($currency[self::COLUMN_ID]);
        }else{
            $currencyId = null;
        }

        // Vložím vždy novou měnu, protože původní verze může být na fakturách.
        $row = $this->database->table(self::TABLE_NAME)->insert($currency);
        
        // Pokud edituji defaultní měnu, skryté nebo smazané staré měně odeberu default a dám to nové verzi
        if($currencyId == $this->getDefaultCurrency()[self::COLUMN_ID]){
            $currency->offsetSet(self::COLUMN_DEFAULT_CURRENCY, $row[self::COLUMN_ID]);
            $this->saveDefaultCurrency($currency);
        }
    }

    /**
     * Odstraní měny korektním způsobem.
     * Provolám metodu cleanCurrency
     * Nesmí se odstranit defaultní metoda
     * @param int $currencyID id měny
     * @return string název měny, který se dosadi do flash messages
     */
    public function removeCurrency($currencyID) {
        // Najdu si měnu
        $currency = $this->getCurrency($currencyID);

        if (!$currency[self::COLUMN_DEFAULT_CURRENCY]) {
            $currencyName = $this->cleanCurrency($currencyID);
            return $currencyName;
        } else {
            throw new Exception('Defaultní měna nesmí být smazána!');
        }
    }

    /**
     * Pokusí se odstranit měnu, pokud není vázaná na objednávku - v tom případě nelze smazat.
     * Využívám princip statusů jako jsou u produků. 
     * Položka se nejdřív vloží do koše a z koše se odstraní úplně ze systému, nebo se jí nastaví status na hidden - nelze smazat.
     * @param int $currencytId     ID měny k odstranění
     * @return string název měny, který se dosadi do flash messages
     */
    private function cleanCurrency($currencytId) {
        // Najdu si měnu
        $currency = $this->getCurrency($currencytId);

        try {
            //Zjistí, jestli se už někdo měnu pokusil smazat - dal ji do koše
            if ($currency[self::COLUMN_STATUS] == Status::DELETED) {
                // Pokusí se starý kurz odstranit, pokud není navázaný na fakturu a někdo ji dal už do koše.
                $this->database->table(self::TABLE_NAME)->where(self::COLUMN_ID, array($currencytId))->delete();
            } else {
                // Měnu se nikdo nepokusil smazat, takže jí dám status na smazaná - tím je v koši a potom se může někdo pokusit ji smazat
                $this->database->table(self::TABLE_NAME)
                        ->where(self::COLUMN_ID, array($currencytId))
                        ->update(array(self::COLUMN_STATUS => Status::DELETED));
            }
        } catch (ForeignKeyConstraintViolationException $ex) {
            // Jinak ji ponechám jako skrytou - není smazaná, ale jen skrytá a skrytá měny by neměla být mazány vůbec.
            $this->database->table(self::TABLE_NAME)
                    ->where(self::COLUMN_ID, array($currencytId))
                    ->update(array(self::COLUMN_TYPE => Status::HIDDEN));
        }

        return $currency[self::COLUMN_NAME];
    }

    /**
     * Uložím do cookies aktuální měnu - id aktuální měny
     */
    public function createCurrencyCookie($currencyId) {
        // Nastavení cookies - jméno, serializované hodnoty ID a token, délka platnosti.
        $this->response->setCookie(
                self::COOKIE_NAME, http_build_query(
                        array(
                            self::COOKIE_ID => $currencyId
                        )
                ), time() + (3600 * 24 * self::CURRENCY_VALIDITY)
        );
    }

}
