<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Model;

use App\Model\BaseManager;
use App\Model\Enum\Status;
use Nette\Database\Table\Selection;
use Nette\Utils\ArrayHash;
use App\Model\PersonManager;
use Nette\Database\Context;
use Nette\InvalidStateException;
use Nette\Utils\DateTime;

/**
 * Spravuje účetní nastavení e-shopu.
 * Vypočítává cenu pro PriceControl podle DPH nastavení a selovových hladin
 * Slevy se budou počítat jen pro uživatele registrované do systému - neregistrovaní mají smůlu.
 * Registrovanému uživateli budu sčítat jeho útratu v jeho detailu
 * @package App\EshopModule\Model
 * 
 * @author Jan Vodák <jan at ideatech.cz>
 */
class DiscountLevelsManager extends BaseManager {

    /** Konstanty pro manipulaci s modelem. */
    const
            TABLE_NAME = 'discount_levels',
            COLUMN_ID = 'discount_id',
            COLUMN_PRICE = 'price',
            COLUMN_QUANTITY = 'quantity',
            COLUMN_DISCOUNT = 'discount',
            COLUMN_TYPE = 'type',
            COLUMN_COUPON_CODE = 'coupon_code',
            COLUMN_LIMIT_DATE = 'limit_date',
            COLUMN_LIMIT_USAGE = 'limit_usages',
            COLUMN_USAGES_NUMBER = 'usages_number',
            COLUMN_STATUS = 'status';

    /** Konstanty pro práci s typy */
    const
            TYPE_USER_DISCOUNT = 'user_discount',
            TYPE_DISCOUNT_COUPON = 'discount_coupon',
            TYPE_QUANTITY_DISCOUNT = 'quantity_discount';

    /** Konstanty pro manipulaci s modelem - atributy. */
    const ATTR_ORDER_DESC = 'DESC';

    /** Konstanty pro manipulaci s agregačním modelem - discount_levels_e_order. */
    const
            COLUMN_USER_DISCOUNT_ID = 'user_discount_id',
            COLUMN_COUPON_DISCOUNT_ID = 'coupon_discount_id';

    /** @var PersonManager Instance třídy modelu pro práci s uživatelem. */
    private $personManager;

    /**
     * Konstruktor s injektovanou třídou pro práci s databází
     * @param PersonManager  $personManager  automaticky injektovaná třída modelu pro práci s osobami
     */
    public function __construct(Context $database, PersonManager $personManager) {
        parent::__construct($database);
        $this->personManager = $personManager;
    }

    /**
     * Vrátí všechny typy slev - tři manuálně vypsané typy, které se budou používat
     * @return array typy slev
     */
    public function getDiscountTypes() {
        return array(
            self::TYPE_USER_DISCOUNT => 'Uživatelské slevy',
            self::TYPE_DISCOUNT_COUPON => 'Slevové kupóny',
            self::TYPE_QUANTITY_DISCOUNT => 'Množstevní slevy'
        );
    }

    /**
     * Vrátí všechny slevy, které nejsou smazané a patří do konkrétího typi
     * @param string $type typ konkkrétní slevy - uživatelské, množstevní, kupon
     * @return Selection nastavení pro všechna období
     */
    public function getAllDiscountsByType($type) {
        return $this->database->table(self::TABLE_NAME)
                        ->where(self::COLUMN_STATUS, Status::PUBLISHED)
                        ->where(self::COLUMN_TYPE, $type)
                        ->order(self::COLUMN_DISCOUNT)
                        ->fetchAll();
    }

    /**
     * Vrátí slevu se všemi informace podle zadaného id
     * @param int $discountId id konkrétní slevy
     * @return ArrayHash kornétní měnový kurz se všmi informacemi
     */
    public function getDiscount($discountId) {
        return $this->database->table(self::TABLE_NAME)
                        ->where(self::COLUMN_ID, $discountId)
                        ->fetch();
    }

    /**
     * Vrátí kupón podle kódu
     * @param string $couponeCode kód konkrétního kupónu
     * @return ArrayHash kornétní kupón
     */
    public function getCoupon($couponeCode) {
        return $this->database->table(self::TABLE_NAME)
                        ->where(self::COLUMN_COUPON_CODE, $couponeCode)
                        ->where(self::COLUMN_STATUS, Status::PUBLISHED)
                        ->where(self::COLUMN_TYPE, self::TYPE_DISCOUNT_COUPON)
                        ->fetch();
    }

    /**
     * Vrátí slevovou hladinu pro zadanou cenu.
     * Bere pouze uzivatelske slevy se statusem published
     * @param int $price  cena, pro kterou se bude hledat slevov8 hladina
     * @param int $discount  přepínač - chci vratit jen slevu - defultní hodnota TRUE, nebo cely zaznam z DB - false
     * @return int | null konkretni sleva nebo null
     */
    public function getActualUserDiscountLevel($price, $discount = TRUE) {
        $discountLevel = false;
        if ($price) {
            $discountLevel = $this->database->table(self::TABLE_NAME)
                    ->where(self::COLUMN_PRICE . ' <= ?', $price)
                    ->where(self::COLUMN_TYPE, self::TYPE_USER_DISCOUNT)
                    ->where(self::COLUMN_STATUS, Status::PUBLISHED)
                    ->order(self::COLUMN_PRICE . ' ' . self::ATTR_ORDER_DESC)
                    ->limit(1)
                    ->fetch();
        }

        return $discountLevel ? ($discount ? $discountLevel[self::COLUMN_DISCOUNT] : $discountLevel) : null;
    }

    /**
     * Přidá slevovu.
     * @param array|ArrayHash $discount  data z formuláře, co se vloží do db jako sleva
     */
    public function addDiscount($discount) {
        $this->database->table(self::TABLE_NAME)->insert($discount);
    }

    /**
     * Použije slevový kupón
     * Použije v tom smyslu, že zkontroluje, jestli je kupón ještě k dispozici - datum a množství a není smazaný
     * Navíc se bude hlídat, jeslti uživatel už kupón nepoužil - navrhnout řešení
     * Pokud je vše v pořádku, vrátím id kupónu, které se spojí s objednávkou v tabulce
     * @param string $couponCode kód kupónu
     * @return int ID slevového kupónu
     */
    public function useCoupon($couponCode) {
        if ($coupon = $this->checkCouponAvailability($couponCode)) {
            return $coupon[self::COLUMN_ID];
        }
    }

    /**
     * Započítá reálné použití kupónu
     * Zase testuju, jestli kupón existuje a je použitelný
     * Při dokončení objednávky přičtu do počtu použití jedničku
     * @param int $couponId id kupónu
     */
    public function includeCouponUse($couponId) {
        if ($coupon = $this->checkCouponAvailability($this->getDiscount($couponId)[self::COLUMN_COUPON_CODE])) {
            $this->database->table(self::TABLE_NAME)
                    ->where(self::COLUMN_ID, $coupon[self::COLUMN_ID])
                    ->update([self::COLUMN_USAGES_NUMBER => Context::literal(self::COLUMN_USAGES_NUMBER . '+1')]);
        }
    }

    /**
     * Pocná funkce pro Zjištění, jestli je kupón dostupný a může být použit
     * Kontroluji limit datumu a množství. Pokud je limit použití 0, tak to znamená, že kupón není omezen.
     * @param ArrayHash konkrétní kupón
     * @return ArrayHash konkrétní kupón
     * @throws InvalidStateException Nebo vyhodí vyjímku, pokud kupón nesplňuje podmínky
     */
    private function checkCouponAvailability($couponCode) {
        if ($coupon = $this->getCoupon($couponCode)) {
            $date = new DateTime();
            if ((!$coupon[self::COLUMN_LIMIT_DATE] || $coupon[self::COLUMN_LIMIT_DATE] >= $date) && ($coupon[self::COLUMN_USAGES_NUMBER] < $coupon[self::COLUMN_LIMIT_USAGE] || $coupon[self::COLUMN_LIMIT_USAGE] == 0 || !$coupon[self::COLUMN_LIMIT_USAGE])) {
                // Kupón existuje a je k dispozi - konotrla, jestli ho už uživatel nepoužil
                /* if (true) {
                  // Metoda pro zapsání do spojovací tabulky pro kupony a osoby
                  } else {
                  throw new Exception('Je nám líto, ale tento kupón jste již použil.');
                  }
                 */
                // Vrátím kupón
                return $coupon;
            } else {
                throw new InvalidStateException('Je nám líto, ale platnost kupónu již vypršela.');
            }
        } else {
            throw new InvalidStateException('Zadali jste neplatný kupón.');
        }
    }

    /**
     * Odstraní měny korektním způsobem.
     * Provolám metodu cleanCurrency
     * @param int $discountId ID slevové hladiny
     */
    public function removeDiscount($discountId) {
        $this->cleanDiscount($discountId);
    }

    /**
     * Odstraní danou slevu.
     * Je to to stejný mechanismus jak v měnách. Remove, clean, obycejny get - nadefinovat v baseManageru
     * Clean bude vždy stejné asi pro product, page, slevu, měnu atd... Specifické věci se budou nastavovat v delete každeho modelu
     * Vždy se bude používat stejná metoda pro zavedení koše - vše co se bude dát mazat se bude první nastavovat status a pak kdyžtak delete
     * @param int $discountId ID slevové hladiny
     */
    public function cleanDiscount($discountId) {
        $discount = $this->getDiscount($discountId);

        //Zjistí, jestli se už někdo měnu pokusil smazat - dal ji do koše
        if ($discount[self::COLUMN_STATUS] == Status::DELETED) {
            // Odstraní úplně z databáze slevu, pokud ji někdo už dal do koše.
            $this->database->table(self::TABLE_NAME)
                    ->where(self::COLUMN_ID, $discountId)
                    ->delete();
        } else {
            // Měnu se nikdo nepokusil smazat, takže jí dám status na smazaná - tím je v koši a potom se může někdo pokusit ji smazat
            $this->database->table(self::TABLE_NAME)
                    ->where(self::COLUMN_ID, array($discountId))
                    ->update(array(self::COLUMN_STATUS => Status::DELETED));
        }
    }

    /**
     * Vypočítá výslednou cenu pro Price komponentu podle účetního nastavení a podle slevových hladin konkrétního uživatele
     * @param int               $price  holá cena produktu bez DPH a slevových hladin
     * @param ArrayHash         $currency měna a kurz měny z nastavení v adminu
     * @param null|ArrayHash    $settings účetní nastavení nebo null, pokud se má pouze zformátovat cena bez DPH
     * @param int               $userID  id uživtele, pro kterého se bude hledat slevová hladina
     * @param null|int          $aditionalPrice Dodatečná částka, pro kterou se nepočítají slevy
     *                          Vypočítám cenu + DPH + uživatelské slevy a nakonec k tomu přičtu částku, které se slevy netýkají
     * @return int výsledná cena
     */
    public function calculatePrice($price, $currency, $settings, $userID, $aditionalPrice) {
        // Obdoba jako v settings presenteru, kde si ten atribut setuji do settings - objednávka má své settings bez toho atributu
        $vat_payer = isset($settings) && isset($this->personManager->getById(PersonManager::TABLE_DETAIL_NAME, PersonManager::COLUMN_PERSON_DETAIL_ID, $settings[SettingsManager::COLUMN_ACCOUNTANT])[PersonManager::COLUMN_TAX_NUMBER]);
        
        // K ceně připočtu DPH, které připočítám i k dodatečné částce, pokud je zadaná
        if ($settings && $vat_payer) {
            $price = $price * (1 + ($settings[SettingsManager::COLUMN_VAT] / 100));
            if ($aditionalPrice) {
                $aditionalPrice = $aditionalPrice * (1 + ($settings[SettingsManager::COLUMN_VAT] / 100));
            }
        }

        // Uživatelské slevy
        if ($userID) {
            // Získám si celkou uživatelovu útratu
            $userAmount = $this->personManager->getUserTotalAmount($userID);
            // Získám si kolik % z aktuálních cen má slevu
            $discountLevel = $this->getActualUserDiscountLevel($userAmount);
            // Snížím cenu o % slevových hladin pokud byla slevová hladina nalezena
            $price = $discountLevel ? $price * (1 - ($discountLevel / 100)) : $price;
        }

        // Po slevách přičítám dodatečnou částku
        if ($aditionalPrice) {
            $price += $aditionalPrice;
        }

        return $price / $currency[CurrencyManager::COLUMN_EXCHANGE_RATE];
    }

}
