<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Model;

use App\Model\BaseManager;
use App\Model\Enum\Status;
use Nette\Database\Context;
use Nette\Database\Table\IRow;
use Nette\Http\Request;
use Nette\Http\Response;
use Nette\InvalidArgumentException;
use Nette\InvalidStateException;
use Nette\Utils\ArrayHash;
use Nette\Utils\DateTime;
use Nette\Utils\Random;
use Nette\Utils\Strings;
use App\Model\PersonManager;

/**
 * Správce objednávek.
 * Pokud uživatel ještě neudělal žádnou objednávku - nemá košík a přidá zboží do košíku - vytvoří se v DB objednávka 
 * + ID objednávky s náhodným tokenem se uluží do cookiesm kddyž potom přidá cokoli do košíku,
 * Opakované přidání do košíku: v PHP přijde cookie s ID objednávky uživatele,
 * které se ověří a použije pro vložení dalšího produktu. Každý uživatel má svůj košík, 
 * který není nijak časově omezený a uchovávají se na serveru v DB i ty košíky, které se nedostaly až k dokončení objednávky. 
 * Zároveň není možné přistoupit ke košíku jiného uživatele.
 * Po odstranění cookie uživatel tak ztratí přístup ke svému košíku, nicméně v databázi stále zůstane. 
 * 
 * Z objednávky se stává faktura. Cizí klíče referují na části osob a adresy - po změně osob se to neprojeví do faktur. 
 * Hodnoty NULL, protože osoby se přiřazují k objednávce až po dokončení nákupu pomocí cookies.
 * Více položek v jednom košíku (objednávce) - agregační tabulka product_e_order
 * 
 * Způsob dopravy je realizován jako specialni produkty ve skyte kategorie produktu a přidává se jako skrytý produkt do objednávky (košíku)
 * Navíc je ulož v DB jako delivery_produc­t_id
 * 
 * Ve funkci completeOrder se pracuje s DPH a ukládáním celkové částky za objednávku do DB - jediní místo kde to neřeší komponenta
 * 
 * @package App\EshopModule\Model
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class OrderManager extends BaseManager {

    /** Formát data v databázi. */
    const DATE_FORMAT = 'Y-m-d H:i:s';

    /** Konstanty pro manipulaci s modelem. */
    const
            TABLE_NAME = 'e_order',
            COLUMN_ID = 'e_order_id',
            COLUMN_TOKEN = 'token',
            COLUMN_CREATED = 'created',
            COLUMN_SETTINGS = 'settings_id',
            COLUMN_STATE = 'state',
            COLUMN_QUANTITY = 'quantity',
            COLUMN_BUYER = 'buyer_id',
            COLUMN_BUYER_PERSON_DETAIL = 'buyer_person_detail_id',
            COLUMN_BUYER_ADDRESS = 'buyer_address_id',
            COLUMN_BUYER_DELIVERY_ADDRESS = 'buyer_delivery_address_id',
            COLUMN_DELIVERY_PRODUCT = 'delivery_product_id',
            COLUMN_PAYMENT_PRODUCT = 'payment_product_id',
            COLUMN_SELLER = 'seller_id',
            COLUMN_SELLER_PERSON_DETAIL = 'seller_person_detail_id',
            COLUMN_SELLER_ADDRESS = 'seller_address_id',
            COLUMN_SELLER_BANK_ACCOUNT = 'seller_bank_account_id',
            COLUMN_ACCOUNTANT_DETAIL = 'accountant_detail_id',
            COLUMN_NUMBER = 'number',
            COLUMN_ISSUED = 'issued',
            COLUMN_DUE_DATE = 'due_date';

    /** Konstanty pro pomocné atributy. */
    const
            ATTR_PRODUCTS = 'products',
            ATTR_PRICE = 'price';

    /** Konstanty pro manipulaci s cookies. */
    const
            COOKIE_NAME = 'order',
            COOKIE_ID = 'orderId',
            COOKIE_TOKEN = 'token';

    /** Konstanty pro stav objednávky. */
    const
            STATE_CREATED = 'created', // Objednávka byla vytvořena.
            STATE_COMPLETED = 'completed', // Objednávka byla dokončena.
            STATE_ACCEPTED = 'accepted', // Objednávka byla přijata ke zpracování.
            STATE_SENT = 'sent', // Objednávka byla odeslána.
            STATE_SUSPENDED = 'suspended', // Objednávka byla pozastavena.
            STATE_CANCELED = 'canceled'; //  Objednávka byla zamítnuta.

    /** Doba platnosti objednávky ve dnech. */
    const ORDER_VALIDITY = 365;

    /** @var null|int ID košíku zákazníka nebo null pokud nemá ještě vytvořený košík. */
    private $orderId = null;

    /** @var Response Odpověď na aktuální HTTP požadavek. */
    private $response;

    /** @var Request Aktuální HTTP požadavek. */
    private $request;

    /** @var ProductManager Instance třídy modelu pro práci s produkty. */
    private $productManager;

    /** @var PersonManager Instance třídy modelu pro práci s osobami. */
    private $personManager;

    /** @var SettingsManager Instance třídy modelu pro práci s účetním nastavením. */
    private $settingsManager;

    /** @var DiscountLevelsManager Instance třídy modelu pro práci se slevými hladinami */
    private $discountLevelManager;

    /** @var CurrencyManager Instance třídy modelu pro práci s měnami a kurzy */
    private $currencyManager;

    /** @var PaymentMethodsManager Instance třídy modelu pro práci s metodami platby. */
    private $paymentMethodsManager;

    /** @var ShippingMethodsManager Instance třídy modelu pro práci s metodami dopravy. */
    private $shippingMethodsManager;

    /**
     * Konstruktor s injektovanou třídou pro práci s databází, aktuálním HTTP požadavkem a odpovědí na aktuální HTTP
     * požadavek.
     * @param Context                   $database               automaticky injektovaná třída pro práci s databází
     * @param Request                   $request                automaticky injektovaná třída aktuálního HTTP požadavku
     * @param Response                  $response               automaticky injektovaná třída odpovědi na aktuální HTTP požadavek
     * @param ProductManager            $productManager         automaticky injektovaná třída modelu pro práci s produkty
     * @param PersonManager             $personManager          automaticky injektovaná třída modelu pro práci s osobami
     * @param SettingsManager           $settingsManager        automaticky injektovaná třída modelu pro práci s účetní nastavením
     * @param DiscountLevelsManager     $discountLevelManager   automaticky injektovaná třída modelu pro práci se slevami
     * @param CurrencyManager           $currencyManager        automaticky injektovaná třída modelu pro práci s měnami a kurzy
     * @param PaymentMethodsManager     $paymentMethodsManager  automaticky injektovaná třída modelu pro práci s metodami platby
     * @param ShippingMethodsManager    $shippingMethodsManager automaticky injektovaná třída modelu pro práci s metodami dopravy
     */
    public function __construct(Context $database, Request $request, Response $response, ProductManager $productManager, PersonManager $personManager, SettingsManager $settingsManager, DiscountLevelsManager $discountLevelManager, CurrencyManager $currencyManager, PaymentMethodsManager $paymentMethodsManager, ShippingMethodsManager $shippingMethodsManager) {
        parent::__construct($database);
        $this->request = $request;
        $this->response = $response;
        $this->productManager = $productManager;
        $this->personManager = $personManager;
        $this->settingsManager = $settingsManager;
        $this->discountLevelManager = $discountLevelManager;
        $this->currencyManager = $currencyManager;
        $this->paymentMethodsManager = $paymentMethodsManager;
        $this->shippingMethodsManager = $shippingMethodsManager;
    }

    /**
     * Vrátí tabulku jako zdroj pro DataGrid
     * @return Table\Selection
     */
    public function getTable() {
        return $this->database->table(self::TABLE_NAME);
    }

    /**
     * Vrátí objednávku podle zadaého ID, pokud není zadané, vrátí objednávku aktuálního uživatele.
     * @param null|int $orderId ID objednávky
     * @return bool|mixed|IRow objednávka
     */
    public function getOrder($orderId = null) {
        if (!$orderId) {
            $orderId = $this->getOrderId(true);
        }

        return $this->database->table(self::TABLE_NAME)->where(self::COLUMN_ID, $orderId)->fetch();
    }

    /**
     * Vrátí objednávky z určitého měsíce
     * @param int $month číslo měsíce
     * @return Selection Objednávky pro požadaovaný měsíc
     */
    public function getOrdersFromMonth($month, $limit = null) {
        return $this->database->table(self::TABLE_NAME)
                        ->where(self::COLUMN_STATE . '!=?', self::STATE_CREATED)
                        ->where('MONTH(' . OrderManager::COLUMN_CREATED . ')', $month)
                        ->limit($limit)
                        ->order(OrderManager::COLUMN_CREATED . ' DESC')
                        ->fetchAll();
    }

    /**
     * Funkce pro výpočet měsíčních tržrb
     * @param Selection $mounthOrders všechny objednávky pro daný měsíc
     * @return float celkové tržby za dané měsíc včetně DPH, které může být pro každou objednávku jiné 
     */
    public function getMounthSales($mounthOrders) {
        $totalSales = 0;
        // Tržby pro předchozí měsíc
        foreach ($mounthOrders as $order) {
            $orderSum = $this->getOrderSummary($order[OrderManager::COLUMN_ID], FALSE)[OrderManager::ATTR_PRICE];
            $settings = $this->settingsManager->getById(SettingsManager::TABLE_NAME, SettingsManager::COLUMN_ID, $order[self::COLUMN_SETTINGS]);
            $totalSales += $orderSum * (1 + ($settings[SettingsManager::COLUMN_VAT] / 100));
        }

        return $totalSales;
    }

    /**
     * Funkce pro nalezení unikátních zákazníků v objednávkách, může hledat unikátní i o proti předchozímu měsíci
     * @param Selection $mounthOrders všechny objednávky pro daný měsíc
     * @param array $lastCustomers pole záknzíků z minulého měsíce, pokud je zadáno, hledá unikátní i oproti minulému měsíci
     * @return int celkové počet unikátních zákazníků v objednávkách
     */
    public function getUniqueCustomers($mounthOrders, $lastCustomers = null) {
        $newCustomers = array();

        foreach ($mounthOrders as $order) {
            if ($lastCustomers) {
                // Pokud jsou zadáni zákaznici z milého měsíce, kontroluji i zákazníky oproti předchozímu měsici
                if (!in_array($order[OrderManager::COLUMN_BUYER], $lastCustomers)) {
                    $newCustomers[$order[OrderManager::COLUMN_BUYER]] = $order[OrderManager::COLUMN_BUYER];
                }
            } else {
                // Jinak vracím unikátní zákazníky jen pro tento měsíc
                $newCustomers[$order[OrderManager::COLUMN_BUYER]] = $order[OrderManager::COLUMN_BUYER];
            }
        }
        
        return $newCustomers;
    }

    /**
     * Vrátí ID košíku zákazníka.
     * @param bool $create zda se má košík v případě neexistence vytvořit
     * @return false|int ID košíku zákazníka nebo false, pokud ID neexistuje a nechceme ho vytvořit
     */
    public function getOrderId($create = true) {
        // Kontroluje, zda ID náhodou již není uložené ve vlastnosti $orderId. Pokud ano, vrátíme ho a nebudeme dělat nic navíc.
        if ($this->orderId) {
            return $this->orderId;
        }

        // Pokud ne, kontroluje existenci cookie (uživatel ji posílá s každým požadavkem na server).
        $orderId = null;
        $token = null;
        $cookie = $this->request->getCookie(self::COOKIE_NAME);

        // Pokud existuje, rozbalí její hodnoty pomocí PHP funkce parse_str() - naplní 2 definované proměnné ($order_id a $token)
        if (isset($cookie)) {
            parse_str($cookie);
        }

        // Pokud si nepřeji vyvářet objednávku a uživatel neposlal cookie - vracím false
        if (!$create && is_null($orderId)) {
            return false;
        }

        // Zpracování databáze - kontrola zda existuje objednávka s tímto ID a tokenem. Pokud ano, vrátíme její ID.
        $databaseOrderId = $this->database->table(self::TABLE_NAME)
                ->select(self::COLUMN_ID)
                ->where(self::COLUMN_ID, $orderId)
                ->where(self::COLUMN_TOKEN, $token)
                ->where(self::COLUMN_STATE, self::STATE_CREATED)
                ->fetch();

        // Případně vytváří novou objednávku spolu s cookies a vratí jeji ID.
        if (!$databaseOrderId) {
            return $this->createOrder();
        }

        // Našlo to objednávku v DB, takže ji vrátím
        return $databaseOrderId[self::COLUMN_ID];
    }

    /**
     * Vrátí poslední objednávky, které mají jiný stav než jen pouze vytvořené.
     * @return array objednávky
     */
    public function getOrders() {
        return $this->database->table(self::TABLE_NAME)
                        ->where(self::COLUMN_STATE . '!=?', self::STATE_CREATED)
                        ->order(self::COLUMN_ID . ' DESC')->limit(30)
                        ->fetchAll();
    }

    /**
     * Vrátí objednávky konkrétní osoby, které mají jiný stav než jen pouze vytvořené.
     * @param int $personId ID konkrétní osoby
     * @return array objednávky
     */
    public function getPersonOrders($personId) {
        return $this->database->table(self::TABLE_NAME)
                        ->where(self::COLUMN_STATE . '!=?', self::STATE_CREATED)
                        ->where(self::COLUMN_BUYER, $personId)
                        ->order(self::COLUMN_ID . ' DESC')
                        ->fetchAll();
    }

    /**
     * Vrátí stavy objednávky pro formulář.
     * @return array stavy objednávky pro formulář
     */
    public function getOrderStates() {
        return [
            'Nová' => self::STATE_COMPLETED,
            'Přijatá' => self::STATE_ACCEPTED,
            'Expedovaná' => self::STATE_SENT,
            'Pozastavená' => self::STATE_SUSPENDED,
            'Zrušená' => self::STATE_CANCELED,
        ];
    }

    /**
     * Vrátí stavy key => value pro výpis hezkého stavu objednávky
     * @return array stavy objednávky pro výpis v šabloně (key je stav z DB)
     */
    public function getPrettyOrderStates() {
        return [
            self::STATE_COMPLETED => 'Nová',
            self::STATE_ACCEPTED => 'Přijatá',
            self::STATE_SENT => 'Expedovaná',
            self::STATE_SUSPENDED => 'Pozastavená',
            self::STATE_CANCELED => 'Zrušená',
        ];
    }

    /**
     * Vrátí datum splatnosti faktury na základě data vystavení - přičte k datu dobu splatnosti.
     * @param string|int|DateTime $date datum vystavení faktury
     * @return DateTime datum splatnosti jako datum o 14 dní později než datum vystavení
     */
    private function getDueDate($date) {
        return DateTime::from($date)->modify("+ 14 DAYS");
    }

    /**
     * Vrátí id kupónu, přiřazenému k objednávce
     * @param int $orderId id objednávky
     * @return int|null buď vrací id kupónu, nebo null v případě, že objednávky žádný kupón nemá
     */
    public function getOrderCoupon($orderId) {
        return $this->database->table(DiscountLevelsManager::TABLE_NAME . '_' . self::TABLE_NAME)
                        ->where(self::COLUMN_ID, $orderId)
                        ->where(DiscountLevelsManager::COLUMN_COUPON_DISCOUNT_ID)
                        ->fetch()[DiscountLevelsManager::COLUMN_COUPON_DISCOUNT_ID];
    }

    /**
     * Vrátí id uživatelské slevy, přiřazené k objednávce
     * @param int $orderId id objednávky
     * @return int|null buď vrací id slevy, nebo null v případě, že objednávka žádný nemá
     */
    public function getOrderUserDiscount($orderId) {
        return $this->database->table(DiscountLevelsManager::TABLE_NAME . '_' . self::TABLE_NAME)
                        ->where(self::COLUMN_ID, $orderId)
                        ->where(DiscountLevelsManager::COLUMN_USER_DISCOUNT_ID)
                        ->fetch()[DiscountLevelsManager::COLUMN_USER_DISCOUNT_ID];
    }

    /**
     * Vrátí přehled košíku - součet množství vybraných produkutů a součet jejich ceny.
     * Můžu zahrnout i metody paltby a dopravy - někde nejsou součástí objednávky (při vytváření) a někde už jsou (po dokončení). 
     * Těmto metodám neodčítám slevu - proto dva způsoby.
     * @param null|int $orderId ID objednávky; pokud není zadané, pracuje s aktuální objednávkou
     * @param bool $methods - jestli chci v součtu započítat i skryté produkty pro dopravu a platbu do slev. 
     *                      Pokud je započítám tak při pozdějším výpočtu slevy z celkové částky se bude počítat sleva i z těchto produktů.
     *                      True se vyjžije defaultně a to jen tehdy, dokud nejsou součástí objednávky tyto metody platby a dopravy
     * @param bool $discountLevels - jestli chci počítat celkouvou částku i s uživatelskou slevou. 
     *                              Hodí se když to ve faktuře rozepisuju i bez slev
     * @param bool $coupon  Jestli chci automaticky pčítat slevu z kupónu
     * @return ArrayHash přehled košíku
     */
    public function getOrderSummary($orderId = null, $methods = true, $discountLevels = TRUE, $coupon = TRUE) {
        // Získám si id košíku / objednávky
        if (!$orderId) {
            $orderId = $this->getOrderId(false);
        }

        // Pokud objednávka neexistuje, vrátí nulové hodnoty a nebude ji zbytečně vytvářet.
        if (!$orderId) {
            return ArrayHash::from([self::ATTR_PRODUCTS => 0, self::ATTR_PRICE => 0]);
        }

        // Získám si objednávku
        $order = $this->getOrder($orderId);

        // Do pole si si uložím součet produktů a celkovou sumu objednávky
        $orderSummary = ArrayHash::from([
                    self::ATTR_PRODUCTS => $this->database->table(ProductManager::TABLE_NAME . '_' . self::TABLE_NAME)
                            ->where(self::COLUMN_ID, $orderId)
                            ->sum(self::COLUMN_QUANTITY),
                    self::ATTR_PRICE => $this->database->table(ProductManager::TABLE_NAME . '_' . self::TABLE_NAME)
                            ->where(self::COLUMN_ID, $orderId)
                            ->sum(self::COLUMN_QUANTITY . ' * ' . ProductManager::TABLE_NAME . '.' . ProductManager::COLUMN_PRICE)
        ]);

        // Pokud chci uživatelské slevy - vratí mi orderSummary po aplikaci uživatelských slev
        if ($discountLevels) {
            $orderSummary = $this->calculateDiscountsFormOrderSum($orderId, $orderSummary, $coupon);
        }

        // Pokud chci počítat s uživatelskýma slevama - najdu k objednávce způsob dopravy a metodu platby a přičtu jejic ceny k celkovému součtu
        if (!$methods) {
            $paymentMethod = $this->paymentMethodsManager->getById(PaymentMethodsManager::TABLE_NAME, PaymentMethodsManager::COLUMN_ID, $order[self::COLUMN_PAYMENT_PRODUCT])[PaymentMethodsManager::COLUMN_PAYMENT_PRICE];
            $shippingMethod = $this->shippingMethodsManager->getById(ShippingMethodsManager::TABLE_NAME, ShippingMethodsManager::COLUMN_ID, $order[self::COLUMN_DELIVERY_PRODUCT])[ShippingMethodsManager::COLUMN_SHIPPING_PRICE];

            $orderSummary[self::ATTR_PRICE] = $orderSummary[self::ATTR_PRICE] + $paymentMethod + $shippingMethod;
        }

        return $orderSummary;
    }

    /**
     * Pomocná metoda pro vypočítání slev v kalkukaci objednávky
     * @param int $orderId              Id objednávky
     * @param ArrayHash $orderSummary   Přehled košíku
     * @param bool $coupon              Jestli chci automaticky pčítat slevu z kupónu
     * @return ArrayHash přehled košíku
     */
    public function calculateDiscountsFormOrderSum($orderId, $orderSummary, $coupon) {
        if (($discounId = $this->getOrderUserDiscount($orderId))) {
            $userDiscount = $this->discountLevelManager->getDiscount($discounId)[DiscountLevelsManager::COLUMN_DISCOUNT];
            $orderSummary[self::ATTR_PRICE] = $userDiscount ? $orderSummary[self::ATTR_PRICE] * (1 - ($userDiscount / 100)) : $orderSummary[self::ATTR_PRICE];
        }

        // Snižím cenu objednývky o hodnotu slevy z kupónu
        if (($couponId = $this->getOrderCoupon($orderId)) && $coupon) {
            $couponDiscount = $this->discountLevelManager->getDiscount($couponId)[DiscountLevelsManager::COLUMN_DISCOUNT];
            $orderSummary[self::ATTR_PRICE] = $orderSummary[self::ATTR_PRICE] * (1 - ($couponDiscount / 100));
        }

        return $orderSummary;
    }

    /**
     * Získá emailovou adresu, na kterou se má doručit email s rekapitulací objednávky.
     * Z tabulky person_detail získám email přes provázání s tabulkou e_roder a id osoby kupujícího a id objednávky
     * @param int $orderId ID objednávky
     * @return null|string email nebo null, pokud se email nepodařilo načíst
     */
    public function getOrderEmail($orderId) {
        $row = $this->database->table(PersonManager::TABLE_DETAIL_NAME)
                        ->where(':' . self::TABLE_NAME . '(' . self::COLUMN_BUYER_PERSON_DETAIL . ')' .
                                '.' . self::COLUMN_ID, $orderId)->fetch();
        return $row ? $row[PersonManager::COLUMN_EMAIL] : null;
    }

    /**
     * Vrátí všechny produkty v košíku (v objednávce).
     * @param null|int $orderId ID objednávky; pokud není zadané, pracuje s aktuální objednávkou
     * @return array produkty v košíku
     */
    public function getProducts($orderId = null) {
        if (!$orderId) {
            $orderId = $this->getOrderId(false);
        }

        if (!$orderId) {
            return array();
        }

        return $this->database->table(ProductManager::TABLE_NAME . '_' . self::TABLE_NAME)
                        ->where(self::COLUMN_ID, $orderId)
                        ->order(ProductManager::TABLE_NAME . '_' . self::COLUMN_ID)
                        ->fetchAll();
    }

    /**
     * Vrátí číslo pro další fakturu.
     * Formát čísla faktury: ROK-MESIC-DEN-PORADI, kde pořadí je číslo faktury v určitém dni, měsíci a roce. 
     * Např. 5. vystavená faktura v 1. červnu 2015 by měla číslo: 2015060105.
     * Vybere nejvyšší číslo faktury od aktuálního dodavatele podle nastavení v aktuálním období. 
     * Pokud je faktura z dnešního dne, připočte k číslu pouze jedničku. 
     * Pokud ne, sestavíme číslo znovu podle dnešního data a na konec přidáme "01".
     * @return null|int číslo pro další fakturu nebo null, pokud neexistuje účetní nastavení pro aktuální datum
     */
    private function getNextInvoiceNumber() {
        if (($accountSettings = $this->settingsManager->getSettings())) {
            $lastNumber = $this->database->table(self::TABLE_NAME)
                    ->where(self::COLUMN_SETTINGS, $accountSettings[SettingsManager::COLUMN_ID])
                    ->max(self::COLUMN_NUMBER);
        } else {
            return null;
        }

        // Poslední faktura byla vydána v dnešní den, zvýšíme číslo.
        $datePrefix = date('Ymd');
        return (int) Strings::startsWith($lastNumber, $datePrefix) ? ++$lastNumber : $datePrefix . '01';
    }

    /**
     * Vytvoří uživateli nový košík - nová objednávka v DB.
     * $token - náhodný řetězec dlouhý 32 znaků (Nette\Utils\Random) spolu s datem vytvoření se uloží jako nová objednávka.
     * Aby mohl uživatel objednávku používat, potřebuje si ve svém prohlížeči pamatovat její ID a token. 
     * Řešení: cookies => sessionsnení vhodný kvůli krátké době expirace. 
     * Zabránění náhodné změny jiné objednávky přes změnnu coockies = kombinace ID a TOKEN
     * 
     * COOKIES
     * Vytváří a mažou se v rámci třídy Nette\Http\Response, z jejíž instance se pak tvoří HTTP odpověď klientovi, 
     * Čtení cookies v Nette = Nette\Http\Request. Instance tvoří HTTP dotaz od klienta.
     * V ní se posílají hodnoty cookies, které má daný klient nastavené.
     * Ošetření krádeže cookies javasciptem díky NETTE přes "httponly"
     * Cookies v sobě ukládá pouze jednu hodnotu, serializují se hodnoty do podoby "query stringu" (např. order_id=1&token=fe5fd6sfd4ferp2). 
     * Tento řetězec se získa z asociativního pole díky PHP funkce http_build_query().
     * Cookies pojmenováváme "order" a životnost podle výše definované konstanty 1 rok.
     * 
     * @return int ID košíku
     */
    private function createOrder() {
        $token = Random::generate(32);

        // Vložení nového košíku do databáze.
        $row = $this->database->table(self::TABLE_NAME)->insert(array(
            self::COLUMN_TOKEN => $token,
            self::COLUMN_CREATED => Context::literal('NOW()'),
        ));

        $orderId = $row[self::COLUMN_ID];

        // Nastavení cookies - jméno, serializované hodnoty ID a token, délka platnosti.
        $this->response->setCookie(
                self::COOKIE_NAME, http_build_query(
                        array(
                            self::COOKIE_ID => $orderId,
                            self::COOKIE_TOKEN => $token,
                        )
                ), time() + (3600 * 24 * self::ORDER_VALIDITY)
        );

        return $orderId;
    }

    /**
     * Po ptvrzení obědvnávky nastaví k objednávce osobu.
     * @param int      $personId      ID osoby
     * @param null|int $orderId       ID objednávky; pokud není zadané, pracuje s aktuální objednávkou
     * @param null|int $orderPersonId umožňuje nastavit jiné ID zákazníka, než z jaké osoby pocházejí data;
     *                                pokud není zadané, předpokládá se stejné jako ID osoby
     * @throws InvalidArgumentException Jestliže osoba se zadaným ID neexistuje.
     */
    public function setPerson($personId, $orderId = null, $orderPersonId = null) {
        if (!$orderId) {
            $orderId = $this->getOrderId(false);
        }

        if (!$orderPersonId) {
            $orderPersonId = $personId;
        }

        if (!($person = $this->personManager->getPerson($personId))) {
            throw new InvalidArgumentException('Osoba se zadaným ID neexistuje.');
        }

        // Přemapujeme data z osoby do objednávky.
        $data = [
            // Doručovací adresa, pokud nezadána tak adresa bydliště
            self::COLUMN_BUYER_DELIVERY_ADDRESS => isset($person[PersonManager::COLUMN_DELIVERY_ADDRESS_ID]) ?
                    $person[PersonManager::COLUMN_DELIVERY_ADDRESS_ID] : $person[PersonManager::COLUMN_ADDRESS_ID],
            self::COLUMN_BUYER_ADDRESS => $person[PersonManager::COLUMN_ADDRESS_ID],
            self::COLUMN_BUYER_PERSON_DETAIL => $person[PersonManager::COLUMN_PERSON_DETAIL_ID],
            self::COLUMN_BUYER => $orderPersonId
        ];

        $this->database->table(self::TABLE_NAME)->where(self::COLUMN_ID, $orderId)->update($data);
    }

    /**
     * Nastaví objednávce stav v DB a odešle emailové upozornění o změně stavu.
     * Pokud se objednávka nastaví jako odeslaná, vygeneruje se číslo faktury a sníží se počet položek z objednávky na skladu.
     * @param int    $orderId ID objednávky
     * @param string $state   stav objednávky
     * @throws InvalidStateException Jestliže nejde nastavit požadovaný stav.
     */
    public function setState($orderId, $state) {
        // Získam si původní stav objednávky než ho přepíšu
        $oldOrderState = $this->getOrder($orderId)[self::COLUMN_STATE];
        // Vytvářím si nové pole order - změní se stav a se doplním údaje do db
        $order = [self::COLUMN_STATE => $state];

        // Jakmile expedujeme objednávku, nastaví se údaje pro fakturu.
        if ($state == self::STATE_SENT) {
            // Datum vystavení faktury, takže dnešní
            $order[self::COLUMN_ISSUED] = date(self::DATE_FORMAT);
            // Datum splatnosti, takže defaultně to bylo dnešní datum + 14 dní
            $order[self::COLUMN_DUE_DATE] = $this->getDueDate($order[self::COLUMN_ISSUED])->format(self::DATE_FORMAT);

            // Snížení počtu položek na skladě u všech produktů objednávky.
            $this->database->beginTransaction();
            $products = $this->database->table(ProductManager::TABLE_NAME . '_' . self::TABLE_NAME)
                    ->where(self::COLUMN_ID, $orderId)
                    ->fetchAll();

            // sklady
            foreach ($products as $product) {
                // Odkazuji se z produktu na detail produktu přes cizí klíč
                $this->productManager->removeFromStock($product[ProductManager::COLUMN_ID], $product[self::COLUMN_QUANTITY]);
            }

            $this->database->commit();
        }

        $this->database->table(self::TABLE_NAME)->where(self::COLUMN_ID, $orderId)->update($order);

        // Část pro věrnostní program - nastavení útraty zákazníka -> pamatuji si kolik u nás celkem utratil
        $this->setUserAmount($orderId, $oldOrderState, $state);
    }

    /**
     * Při potvrzení objednávky si ukládám k jednotlivým zákazníkům jejich útraty - přičtu částku za objednávku
     * Budu rozhodovat podle starého stavu objednávky a nově nastaveného stavu, jestli se útrata přičte, odečte, nebo se nic neprovede
     * Pro změnu hodnoty využiju funcku updateTotalAmount
     * @param int $orderId ID objednávky
     * @param string $oldOrderState původní stav objednávky získný z DB
     * @param string $state nový stav objednávky získaný z formuláře pro úpravu
     */
    private function setUserAmount($orderId, $oldOrderState, $state) {

        if ($oldOrderState == self::STATE_COMPLETED && $state != self::STATE_CANCELED && $state != self::STATE_COMPLETED) {
            // Pokud byl starý stav completed (dokončená objednávka) a nový stav není zrušený nebo pozastavený -> přičtu zákazníkovi útratu
            $this->updateTotalAmount($orderId, '+');
        } else if ($oldOrderState == self::STATE_CANCELED && $state != self::STATE_CANCELED && $state != self::STATE_COMPLETED) {
            // Pokud byl starý stav zrušeno a nový stav je cokoliv kromě zrušeno nebo vytvořeno - přičtu zákazníkovi útratu za objednávku
            $this->updateTotalAmount($orderId, '+');
        } else if ($oldOrderState != self::STATE_CANCELED && $oldOrderState != self::STATE_COMPLETED && $state == self::STATE_CANCELED) {
            // Pokud nebyl starý stav zrušeno a nový stav je zrušeno - odečtu zákazníkovi útratu za objednávku
            $this->updateTotalAmount($orderId, '-');
        } else if ($oldOrderState != self::STATE_CANCELED && $oldOrderState != self::STATE_COMPLETED && $oldOrderState != self::STATE_SUSPENDED && $state == self::STATE_COMPLETED) {
            // Pokud starý stav nebyl zrušeno, nový stav completed (vracím zakázku do původního přijatého stavu) -> odečtu zákazníkovi útratu
            $this->updateTotalAmount($orderId, '-');
        } else if ($oldOrderState == self::STATE_SUSPENDED && $state == self::STATE_COMPLETED) {
            // Z pozastavené do nové - odčítám částku
            $this->updateTotalAmount($orderId, '-');
        }

        // Prvni podminka -  && $state != self::STATE_SUSPENDED
        // Druhá podminka - && $state != self::STATE_SUSPENDED
    }

    /**
     * Nastaví do db k objednávce id způsobu dopravy.
     * ID objednávky (košíku) získým pomocí metody getOrderId, bez toho aniž bych ju vytvořil
     * @param int $deliveryProductId ID metody, který reprezentuje způsob dopravy
     */
    public function setDeliveryProduct($deliveryProductId) {
        $this->database->table(self::TABLE_NAME)
                ->where(self::COLUMN_ID, $this->getOrderId(false))
                ->update([self::COLUMN_DELIVERY_PRODUCT => $deliveryProductId]);
    }

    /**
     * Nastaví do db k objednávce id způsobu platby.
     * ID objednávky (košíku) získým pomocí metody getOrderId, bez toho aniž bych ju vytvořil
     * @param int $paymentProductId ID produktu, který reprezentuje způsob platby
     */
    public function setPaymentProduct($paymentProductId) {
        $this->database->table(self::TABLE_NAME)
                ->where(self::COLUMN_ID, $this->getOrderId(false))
                ->update([self::COLUMN_PAYMENT_PRODUCT => $paymentProductId]);
    }

    /**
     * Nastaví do db k objednávce id kupónu pro slevu
     * ID objednávky (košíku) získým pomocí metody getOrderId, bez toho aniž bych ju vytvořil
     * Testuju, jeslti objednávka už neobsahuje id kupónu.
     * @param int $couponId ID kupónu
     * @throws InvalidStateException Pokud objednávka již obsahuje id stejného kupónu, upozorním na to uživatele přes vyjímku
     */
    public function setCouponDiscount($couponId) {
        $orderId = $this->getOrderId(false);

        if (!$orderId) {
            throw new InvalidStateException('K této objednávce nelze přiřadit kupón.');
        }

        // Zkusím najít, jestli k objednávce již není teto kupón přiřazen
        $assignedCoupon = $this->getOrderCoupon($couponId);

        // Pokud tento kupón není k objednávce přiřazen, přiradím ho
        if ($assignedCoupon != $couponId) {
            // Vkládáme novou položku do agregační tabulky.
            $this->database->table(DiscountLevelsManager::TABLE_NAME . '_' . self::TABLE_NAME)
                    ->insert(array(
                        self::COLUMN_ID => $orderId,
                        DiscountLevelsManager::COLUMN_COUPON_DISCOUNT_ID => $couponId
            ));
        } else { // Jinak vyhazuji vyjímku
            throw new InvalidStateException('Tento kupón jste ke stejné objednávce již použili.');
        }
    }

    /**
     * Odebere objednávce id kupónu pro slevu - nastavím null hodnotu
     * ID objednávky (košíku) získým pomocí metody getOrderId, bez toho aniž bych ju vytvořil
     */
    public function unsetCouponDiscount($couponId) {
        $orderId = $this->getOrderId(false);

        // Odstranění kupónu od objednávky z agregační tabulky.
        $this->database->table(DiscountLevelsManager::TABLE_NAME . '_' . self::TABLE_NAME)
                ->where(self::COLUMN_ID, $orderId)
                ->where(DiscountLevelsManager::COLUMN_COUPON_DISCOUNT_ID, $couponId)
                ->delete();
    }

    /**
     * Nastaví do db k objednávce id uživatelské slevy
     * Nastavuji to do agregační tabulky discount_levels_e_order
     * @param int $orderId ID objednávky
     * @param int $buyerId ID kupujícího - personId
     */
    private function setOrderUserDiscount($orderId) {
        $order = $this->getOrder($orderId);
        $buyerUser = $this->personManager->getUserIdByPersonId($order[OrderManager::COLUMN_BUYER]);
        $totalAmount = $this->personManager->getUserTotalAmount($buyerUser);
        $discountLevel = $totalAmount ? $this->discountLevelManager->getActualUserDiscountLevel($totalAmount, FALSE) : null;

        // Vkládáme novou položku do agregační tabulky - pouze v případě, že uživatel má nějakou slevu, jinak nic.
        if ($discountLevel) {
            $this->database->table(DiscountLevelsManager::TABLE_NAME . '_' . self::TABLE_NAME)
                    ->insert(array(
                        self::COLUMN_ID => $orderId,
                        DiscountLevelsManager::COLUMN_USER_DISCOUNT_ID => $discountLevel[DiscountLevelsManager::COLUMN_ID]
            ));
        }
    }

    /**
     * Přidá produkt do košíku.
     * Pokud ID objednávky není zadáné, předpokládá se, že se myslí aktuální objednávka zákazníka.
     * 
     * @param int      $productId            ID produktu
     * @param int      $quantity             počet kusů
     * @param int|null $orderId              ID objednávky; null pokud není uvedeno. Přidáváme do aktuální objednávky, nebo se vytvoří nová.
     * @param bool     $ignoreHiddenProducts zda chceme povolit přidávání skrytých produktů - skrytý produkt je způsob dopravy, taky se vkládá k objednávce
     * @return bool vrací true, pokud se podařilo produkt přidat do košíku nebo false, pokud daný produkt není dostupný
     * @throws InvalidArgumentException Pokud je zadán špatný počet kusů produktu.
     */
    public function addProducts($productId, $quantity = 1, $orderId = null, $ignoreHiddenProducts = false) {
        if (!$orderId) {
            $orderId = $this->getOrderId();
        }

        if ($quantity <= 0) {
            throw new InvalidArgumentException('Nelze vložit záporný počet položek.');
        }

        $product = $this->productManager->getProductFromId($productId);

        /*
          if($product[ProductManager::COLUMN_STOCK] <= 0){
          throw new InvalidArgumentException('Nelze vložit produkt, který není na skladě.');
          }
         */

        // Pokud nechci vkládat skryté produkty a produkt není dostupný (je skrytý, nebo je ve skryté kategorii) končím metodu s false)
        if ($product->type != ProductManager::TYPE_VARIABLE && $product->status != Status::PUBLISHED) {
            if (!$ignoreHiddenProducts && !$this->productManager->isProductAvailable($productId)) {
                return false;
            }
        }

        // Pokud je již položka v košíku, pouze zvyšujeme počet u položky v košíku.
        if ($exists = $this->database->table(ProductManager::TABLE_NAME . '_' . self::TABLE_NAME)
                ->where(ProductManager::COLUMN_ID, $productId)
                ->where(self::COLUMN_ID, $orderId)
                ->fetch()
        ) {
            $rows = $this->database->table(ProductManager::TABLE_NAME . '_' . self::TABLE_NAME)
                    ->where(ProductManager::COLUMN_ID, $productId)
                    ->where(self::COLUMN_ID, $orderId)
                    ->update([self::COLUMN_QUANTITY => Context::literal(self::COLUMN_QUANTITY . '+?', $quantity)]);
            return $rows > 0;
        }

        // Vkládáme novou položku do košíku.
        $row = $this->database->table(ProductManager::TABLE_NAME . '_' . self::TABLE_NAME)
                ->insert(array(
            ProductManager::COLUMN_ID => $productId,
            self::COLUMN_ID => $orderId,
            self::COLUMN_QUANTITY => $quantity,
        ));

        return (bool) $row;
    }

    /**
     * Aktualizuje objednávku.
     * @param array|ArrayHash $order objednávka
     */
    public function updateOrder($order) {
        $this->database->table(self::TABLE_NAME)
                ->where(self::COLUMN_ID, $order[self::COLUMN_ID])->update($order);
    }

    /**
     * Upraví počet kusů všech produktů v košíku k aktuální objednávce.
     * @param array $quantities = nové hodnoty pro počty kusů produktů v košíku (key => val - porductID => pocet)
     * @throws InvalidArgumentException Pokud je jeden ze zadaných počtů kusů produktu záporný.
     */
    public function updateCart($quantities) {
        array_walk($quantities, [$this, 'updateProductInOrder'], $this->getOrderId(false));
    }

    /**
     * Upraví produkt v objednávce.
     * @param int $quantity  nový počet kusů; pokud počet upravíme na 0, produkt se odebere z košíku
     * @param int $productId ID produktu
     * @param int $orderId   ID objednávky
     * @throws InvalidArgumentException Pokud je zadán záporný počet kusů produktu.
     */
    public function updateProductInOrder($quantity, $productId, $orderId) {
        if ($quantity < 0) {
            throw new InvalidArgumentException('Počet produktů nesmí být záporný.');
        }

        if ($quantity == 0) {
            // Odstranění produktu z košíku z agregační tabulky.
            $this->database->table(ProductManager::TABLE_NAME . '_' . self::TABLE_NAME)
                    ->where(ProductManager::COLUMN_ID, $productId)
                    ->where(self::COLUMN_ID, $orderId)
                    ->delete();
        } else {
            // Úprava počtu produktů.
            $this->database->table(ProductManager::TABLE_NAME . '_' . self::TABLE_NAME)
                    ->where(ProductManager::COLUMN_ID, $productId)
                    ->where(self::COLUMN_ID, $orderId)
                    ->update(array(self::COLUMN_QUANTITY => $quantity));
        }
    }

    /**
     * Dokončí objednávku.
     */
    public function completeOrder() {
        // Získám si ID objednávky (košíku) z cookies nebo z proměnné bez vytváření
        $orderId = $this->getOrderId(false);

        // Nastavení DPH - id kdo platí DPH
        $accountingSettings = $this->settingsManager->getSettings();

        // id prodejce
        $seller = $this->personManager->getPerson($accountingSettings[SettingsManager::COLUMN_SELLER]);

        // id měny, kterou uživatel momentálně používá
        $actualCurrencyId = $this->currencyManager->getActualCurrency()[CurrencyManager::COLUMN_ID];

        // Pokusím se získat variabilní číslo pro fakturu - musí být vyplněno účetní nastavení, jinak NULL
        $invoiceNumber = $this->getNextInvoiceNumber();

        // Nastaví se všechny potřebné vazby na zákazníka, obchodníka a stav na "completed".
        $this->database->table(self::TABLE_NAME)->where(self::COLUMN_ID, $orderId)->update(array(
            self::COLUMN_CREATED => Context::literal('NOW()'),
            self::COLUMN_SETTINGS => $accountingSettings[SettingsManager::COLUMN_ID],
            self::COLUMN_SELLER_ADDRESS => $seller[PersonManager::COLUMN_ADDRESS_ID],
            self::COLUMN_SELLER_PERSON_DETAIL => $seller[PersonManager::COLUMN_PERSON_DETAIL_ID],
            self::COLUMN_SELLER_BANK_ACCOUNT => $seller[PersonManager::COLUMN_BANK_ACCOUNT_ID],
            self::COLUMN_NUMBER => $invoiceNumber,
            self::COLUMN_STATE => self::STATE_COMPLETED,
            CurrencyManager::COLUMN_ID => $actualCurrencyId
        ));

        // Nastavuji si k objednávce uživatelskou slevu, kterou aktuálně uživatel má
        $this->setOrderUserDiscount($orderId);
    }

    /**
     * Při potvrzení objednávky si ukládám k jednotlivým zákazníkům jejich útraty - přičtu částku za objednávku i s DPH
     * Slouží pro věrnostní program
     * Pro uložení hodnoty využiju připravenou fci setTotalAmount z personManager
     * @param int $orderId ID objednávky
     * @param string $operation operace, která se bude při změně celkové utracené částky provádět
     * @param ArrayHash účetní nastavení pro konkrétní období
     */
    private function updateTotalAmount($orderId, $operation) {
        // Získám si objednávku
        $order = $this->getOrder($orderId);
        // Infromace pro výpočet DPH
        $accountingSettings = $this->settingsManager->getById(SettingsManager::TABLE_NAME, SettingsManager::COLUMN_ID, $order[SettingsManager::COLUMN_ID]);
        // Test, jeslti je účetní plátce DPH
        $vat_payer = isset($accountingSettings) && isset($this->personManager->getById(PersonManager::TABLE_DETAIL_NAME, PersonManager::COLUMN_PERSON_DETAIL_ID, $accountingSettings[SettingsManager::COLUMN_ACCOUNTANT])[PersonManager::COLUMN_TAX_NUMBER]);
        // Získání celkové částky objednávky
        $totalPrice = $this->getOrderSummary($orderId, FALSE)[self::ATTR_PRICE];
        // Pořešení DPH k celkové částce, kterou ukládám do DB
        $total_amount = $vat_payer ? $totalPrice * (1 + ( $accountingSettings[SettingsManager::COLUMN_VAT] / 100)) : $totalPrice;
        // Získam si id osoby (person_id) zákazníka z objednávky
        $buyer = $order[self::COLUMN_BUYER];
        //  Přičete k celkové utracené částce u osoby (person) celkovou sumu objednávky - využiju předpřipravenou metodu
        $this->personManager->setTotalAmount($buyer, $total_amount, $operation);
    }

}
