<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Model;

use App\CmsModule\Model\PageManager;
use App\MediaModule\Model\FileRelationsManager;
use App\Model\BaseManager;
use App\Model\Enum\Status;
use Nette\Database\Context;
use Nette\Database\ForeignKeyConstraintViolationException;
use Nette\Database\Table\IRow;
use Nette\Database\Table\Selection;
use Nette\Database\UniqueConstraintViolationException;
use Nette\Utils\ArrayHash;
use Nette\Utils\DateTime;
use Nette\Utils\Image;

/**
 * Správce produktů.
 * Třída ProductManager je ta hlavní, poskytuje rozhraní pro presentery
 * Třída ProductImageManger je pomocná, má na starost pouze práci se soubory obrázků
 *
 * Produkty se dají vyhledávat podle:
 * fráze (např. "květináč"), podle toho zda jsou skladem,
 * podle minimální a maximální ceny,
 * také podle toho na jaké stránce výsledků vyhledávání se uživatel nachází.
 * Pokud se bude vypisovat např. 6 produktů na stránku a bude na 9. stránce, zajímají nás výsledky v pořadí 46-54.
 * Řadit podle hodnocení, ceny nebo data, sestupně nebo vzestupně, jak uživatel určí.
 * Může se také zadat kategorie, ve které chceme vyhledávat.
 * POZN: Protože je použit princip fulltextové vyhledávání MATCH ... AGAINST,
 * je potřeba na dané sloupce v databázi přidat tzv. fulltextové indexy
 * Nevyhledává se pomocí LIKE, protože takové "vyhledávání" totiž není indexované a na velkých datech je nepoužitelné kvůli nízkému výkonu.
 * * @package App\EshopModule\Model
 *
 * @author Jan Vodák <vodak at ideatech.cz>
 */
class ProductManager extends BaseManager
{

    /** Konstanty pro manipulaci s modelem. */
    const
        TABLE_NAME = 'product',
        COLUMN_ID = 'product_id',
        COLUMN_CODE = 'code',
        COLUMN_URL = 'url',
        COLUMN_RATING_SUM = 'rating_sum',
        COLUMN_RATINGS = 'ratings',
        COLUMN_STOCK = 'stock',
        COLUMN_IMAGE_COUNT = 'images_count',
        COLUMN_PRICE = 'price',
        COLUMN_TITLE = 'title',
        COLUMN_ACTION = 'is_action',
        COLUMN_NEW = 'is_new',
        COLUMN_CREATED = 'created',
        COLUMN_TYPE = 'type',
        COLUMN_PARENT = 'parent_id',
        COLUMN_MODIFIED = 'modified',
        COLUMN_STATUS = 'status';

    /** Konstanty pro různé možnosti seřazení produktů. */
    const
        ORDER_RATING = 'rating',
        ORDER_LOWEST_PRICE = 'lowest_price',
        ORDER_HIGHEST_PRICE = 'highest_price',
        ORDER_NEWEST = 'newest';

    const
        TYPE_SIMPLE = 'simple',
        TYPE_VARIABLE = 'variable',
        TYPE_VARIATION = 'variation';

    /**
     * @var FileRelationsManager
     */
    private $fileRelationsManager;

    public function __construct(Context $database, FileRelationsManager $fileRelationsManager)
    {
        parent::__construct($database);
        $this->fileRelationsManager = $fileRelationsManager;

    }

    /**
     * Vrátí tabulku jako zdroj pro DataGrid
     */
    public function getTable()
    {
        return $this->database->table(self::TABLE_NAME);
    }

    /**
     * Vrátí produkt podle ID.
     * @param int $productId - ID produktu
     * @param null|string $columns sloupce pro SELECT, nebo null pro všechny sloupce
     * @return bool|mixed|IRow produkt
     */
    public function getProductFromId($productId, $columns = null)
    {
        return $this->database->table(self::TABLE_NAME)
            ->select($columns ? $columns : '*')
            ->where(self::COLUMN_ID, $productId)->fetch();
    }

    /**
     * Zkontroluje, zda produkt s danou URL již neexistuje a případně vyhodí příslušnou výjimku.
     * @param string $url URL produktu
     */
    private function checkUrlAvailability($url)
    {
        if ($this->getProduct($url, self::COLUMN_URL)) {
            throw new UniqueConstraintViolationException();
        }
    }

    /**
     * Vrací pole nebo řetězec identifikátorů statických sloupců u produktu tj. těch co se přenášejí na novou kopii produktu mimo formulář.
     * @param bool|false $asArray má vrátit hodnotu jako pole nebo řetězec?
     * @return array|string pole nebo řetězec identifikátorů statických sloupců u produktu
     */
    private function getStaticProductColumns($asArray = false)
    {
        $staticColumns = [
            self::COLUMN_IMAGE_COUNT,
            self::COLUMN_RATING_SUM,
            self::COLUMN_RATINGS,
            self::COLUMN_STOCK,
            self::COLUMN_CREATED,
            self::COLUMN_STATUS,
            self::COLUMN_PARENT
        ];


        return $asArray ? $staticColumns : implode(', ', $staticColumns);
    }

    /**
     * Vrátí produkt podle URL.
     * @param string $url URL produktu
     * @param null|string $columns sloupce pro SELECT, nebo null pro všechny sloupce
     * @return bool|mixed|IRow produkt
     */
    public function getProduct($url, $columns = null)
    {
        return $this->database->table(self::TABLE_NAME)
            ->select($columns ? $columns : '*')
            ->where(self::COLUMN_URL, array($url))
            ->where(self::COLUMN_TYPE, [self::TYPE_SIMPLE, self::TYPE_VARIABLE])
            ->where(self::COLUMN_STATUS, Status::PUBLISHED)->fetch();
    }

    /**
     * Vrátí seznam všech produktů pro formulář kromě variant produků
     * @return array seznam všech produktů pro formulář
     */
    public function getProductList()
    {
        $methodsType = [ProductManager::TYPE_VARIATION];

        return $this->database->table(self::TABLE_NAME)
            ->select(self::COLUMN_ID . ',' . self::COLUMN_TITLE)
            ->where(self::COLUMN_STATUS, Status::PUBLISHED)
            ->where(self::COLUMN_TYPE . ' NOT', $methodsType)
            ->fetchPairs(self::COLUMN_ID, self::COLUMN_TITLE);
    }

    public function getVariants($productId)
    {
        return $this->database->table(self::TABLE_NAME)
            ->where(self::COLUMN_PARENT, $productId)
            ->where(self::COLUMN_TYPE, self::TYPE_VARIATION)
            ->where(self::COLUMN_STATUS, Status::PUBLISHED);
    }

    /**
     * FILTROVÁNÍ PRODUKTŮ
     * Vrátí produkty odpovídající daným parametrům.
     * Zpracování probíhá ověřením existence parametru a případně filtrováním podle něj
     * => metoda umí logicky filtroval jen podle známých parametrů.
     * @param array $parameters parametry pro filtrování produktů: order_by, phrase, category_id, start_price, end_price, stock, created, action_product, not_in_category
     * @return Selection produkty odpovídající daným parametrům
     */
    public function getProducts(array $parameters = null)
    {
        // Nastavení výchozích proměnných pro řazení - key => value. Key je nazev hodnoceni z konstant a value je string do dotazu
        $orderColumns = [
            // Protože hodnocení mohou mít stejné, tak je seřadíme i podle ID.
            self::ORDER_RATING => '(rating_sum/ratings) DESC, product_id DESC',
            self::ORDER_LOWEST_PRICE => 'price',
            self::ORDER_HIGHEST_PRICE => 'price DESC',
            self::ORDER_NEWEST => 'product_id DESC',
        ];

        // Vybere všechny neskryté produkty.
        $products = $this->database->table(self::TABLE_NAME)
            ->where(self::COLUMN_STATUS, Status::PUBLISHED);

        // Vybere jen produkty s typem simple a variable, nechceme vybirat variace ani platebni/dodaci metody
        $products->where(self::COLUMN_TYPE, [self::TYPE_SIMPLE, self::TYPE_VARIABLE]);

        // Způsob řazení.
        if (!empty($parameters['order_by']) && isset($orderColumns[$parameters['order_by']])) {
            // Pokud uživatel zadal koretkní způsob řazení, který je povolený
            $products->order($orderColumns[$parameters['order_by']]);
        } else {
            // Výchozí seřazení je podle hodnocení.
            $products->order($orderColumns[self::ORDER_RATING]);
        }

        /*
         * Přidání fulltextového vyhledávání - yhledávací fráze.
         * Přidají se do SELECT další 2 sloupce a to pomocí vyhledávací klauzule MATCH ... AGAINST. 
         * Nejprve hledáme v titulku a popisku, poté ještě zvlášť v titulku. Díky tomu můžeme dát samotnému titulku vyšší relevanci. 
         * Podle relevance budeme poté také řadit - v SELECT se params musí psát za sebou, pole nefunguje.
         */
        if (!empty($parameters['phrase'])) {
            $phrase = $parameters['phrase'];
            $products->select(
                '*, 
                    MATCH(product.title, short_description) AGAINST (?) AS relevance,
                    MATCH(product.title) AGAINST (?) AS title_relevance', $phrase, $phrase
            );

            $products->where('MATCH(product.title, short_description) AGAINST (?)', $phrase);
            $products->order('title_relevance DESC, relevance DESC');
        }

        // Filtrování podle kategorie - po přeložení konstant: :product_category.category.category_id
        if (!empty($parameters['category_id'])) {
            $products->where(
                ':' . self::TABLE_NAME . '_' . CategoryManager::TABLE_NAME .
                '.' . CategoryManager::TABLE_NAME .
                '.' . CategoryManager::COLUMN_ID, $parameters['category_id']
            );
        }

        // Filtrování podle kategorie - po přeložení konstant: :product_category.category.category_id
        if (!empty($parameters['not_in_category'])) {
            $products->where(
                'NOT :' . self::TABLE_NAME . '_' . CategoryManager::TABLE_NAME .
                '.' . CategoryManager::TABLE_NAME .
                '.' . CategoryManager::COLUMN_ID, $parameters['not_in_category']
            );
        }

        // Omezení cenou.
        if (!empty($parameters['start_price'])) {
            $products->where('price >= ?', $parameters['start_price']);
        }

        if (!empty($parameters['end_price'])) {
            $products->where('price <= ?', $parameters['end_price']);
        }

        // Omezení skladem.
        if (!empty($parameters['in_stock']) && $parameters['in_stock']) {
            $products->where('stock > 0');
        }

        // Omezení datumem vytvoření produktu
        if (!empty($parameters['created'])) {
            $products->where('created > ?', $parameters['created']);
        }

        // Omezení akcí
        if (!empty($parameters['action_product']) && $parameters['action_product']) {
            $products->where('is_action', 1);
        }

        // Omezení akcí
        if (!empty($parameters['new_product']) && $parameters['new_product']) {
            $products->where('is_new', 1);
        }

        return $products;
    }

    /**
     * Zjistí, zda je produkt možné vložit do košíku - není skrytý nebo není ve skryté kategorii.
     * @param int $productId ID produktu
     * @return bool zda je produkt možné vložit do košíku
     */
    public function isProductAvailable($productId)
    {
        return (bool)$this->database->table(self::TABLE_NAME)
            ->select(self::TABLE_NAME . '.' . self::COLUMN_ID)
            ->where(self::TABLE_NAME . '.' . self::COLUMN_ID, $productId)
            ->where(self::TABLE_NAME . '.' . self::COLUMN_STATUS, Status::PUBLISHED)
            ->where('NOT :' . self::TABLE_NAME . '_' . CategoryManager::TABLE_NAME .
                '.' . CategoryManager::TABLE_NAME .
                '.' . CategoryManager::COLUMN_HIDDEN)
            ->fetch();
    }

    /**
     * Pozor na ukládání a editaci prodkutů - Pokud se v produktu provede jakákoliv změnu
     * nesmí se projevit na již existujících fakturách - kde se počítá se starými údaji
     * Proto metoda pro uložení vytvoří vždy nový produkt (a zachová původní verzi).
     * Aby se produkty v databázi zbytečně nehromadily - metod cleanProduct()
     * pokusí se starý produkt odstranit, pokud se neváže na žádnou objednávku.
     * Uloží tedy produkt tak, že vytvoří vždy nový a vrátí jeho ID. Pokud je zadáno product_id, pokusí se odstranit
     * staré produkty, když nejsou použité v objednávkách.
     * @param ArrayHash|array $product produkt
     * @return int ID nového produktu
     * @throws UniqueConstraintViolationException Pokud se pkouší vložit produkt se stejnou url - odchytává presenter.
     */
    public function saveProduct($product)
    {
        // Editace
        if (isset($product[self::COLUMN_ID]) && ($productId = $product[self::COLUMN_ID])) {
            // Uložení hodnot starého produktu pro pozdější update.
            $oldProduct = $this->getProductFromId($productId, $this->getStaticProductColumns());
            $oldProductId = $productId;
            // Odstranění starého produktu, pokud se nikde jine v DB k němu nic nevztahuje - napr. faktura
            $this->cleanProduct($productId);
            unset($product[self::COLUMN_ID]);
        }

        // Podiva se, jeslti jiz neexistuje neskryty produkt se stejnou URL
        $this->checkUrlAvailability($product[self::COLUMN_URL]);

        $product[self::COLUMN_MODIFIED] = new DateTime();
        // Vložíme vždy nový produkt, protože původní verze může být na fakturách.
        $row = $this->database->table(self::TABLE_NAME)->insert($product);

        // Updatuje statická pole nového produktu hodnotami ze starého, pokud se jedná o editaci.
        if (isset($oldProduct)) {
            $this->database->table(self::TABLE_NAME)
                ->where(self::COLUMN_ID, array($row[self::COLUMN_ID]))
                ->update(array_combine($this->getStaticProductColumns(true), $oldProduct->toArray()));
        }

        // Přesunutí recenzí od starého k novému produktu - protože vkládám nový záznam s novým id
        if (isset($oldProductId)) {
            $this->database->table(ProductReviewManager::TABLE_NAME)
                ->where(self::COLUMN_ID, $oldProductId)
                ->update(array(self::COLUMN_ID => $row[self::COLUMN_ID]));
        }

        // Přesunutí obrázků od starého k novému produktu - protože vkládám nový záznam s novým id
        if (isset($oldProductId)) {
            $this->database->table(FileRelationsManager::TABLE_NAME)
                ->where(self::COLUMN_ID, $oldProductId)
                ->update(array(self::COLUMN_ID => $row[self::COLUMN_ID]));
        }

        // Přesunutí atributů od starého k novému produktu - protože vkládám nový záznam s novým id
        if (isset($oldProductId)) {
            $this->database->table(ProductAttributeManager::TABLE_NAME)
                ->where(self::COLUMN_ID, $oldProductId)
                ->update(array(self::COLUMN_ID => $row[self::COLUMN_ID]));
        }

        // Přesunutí variant od starého k novému produktu - protože vkládám nový záznam s novým id
        if (isset($oldProductId)) {
            $this->database->table(ProductManager::TABLE_NAME)
                ->where(self::COLUMN_PARENT, $oldProductId)
                ->update(array(self::COLUMN_PARENT => $row[self::COLUMN_ID]));
        }

        // Přesunutí případných relací se stránkou na nový produkt
        if (isset($oldProductId)) {
            $this->database->table(PageManager::TABLE_NAME)
                ->where(PageManager::COLUMN_PRODUCT, $oldProductId)
                ->update(array(PageManager::COLUMN_PRODUCT => $row[self::COLUMN_ID]));
        }

        return $row[self::COLUMN_ID];
    }

    /**
     * Nahraje obrázky k danému produktu
     * Volá metodu z productImageManager a aktualizuje hodnotu pocet obrázků v tbulce pro produkty.
     * @param int $productId ID produktu, ke kterému nahráváme obrázky
     * @param Image[] $images obrázky
     * @param null|int $oldProductId ID staré verze produktu (každá editace vytvoří nový produkt)
     */
    public function saveProductImages($productId, array $images, $oldProductId = null)
    {
        $imagesCount = $this->productImageManager->saveProductImages($productId, $images, $oldProductId, $oldProductId ? $this->getImagesCount($productId) : 0);
        $this->database->table(self::TABLE_NAME)
            ->where(self::COLUMN_ID, array($productId))
            ->update(array(self::COLUMN_IMAGE_COUNT => $imagesCount));
    }

    /**
     * Aktualizuje celkové hodnocení u produktu - každý produkt u sebe uchovává ifnormaci o svém hodnocení.
     * @param int $productId ID produktu
     * @param int $rating nové hodnocení
     */
    public function updateRatings($productId, $rating)
    {
        $this->database->table(self::TABLE_NAME)
            ->where(self::COLUMN_ID, $productId)
            ->update([
                self::COLUMN_RATING_SUM => Context::literal(self::COLUMN_RATING_SUM . '+?', $rating),
                self::COLUMN_RATINGS => Context::literal(self::COLUMN_RATINGS . '+1')
            ]);
    }

    /**
     * Změní dané množství produktů na skladě podle příslušné operace.
     * @param int $productId ID produktu
     * @param int $quantity počet produktů
     * @param string $operation operace, která se bude při změně počtu produktů provádět
     */
    private function updateStock($productId, $quantity, $operation)
    {
        $this->database->table(self::TABLE_NAME)
            ->where(self::COLUMN_ID, array($productId))
            ->update([self::COLUMN_STOCK => Context::literal(self::COLUMN_STOCK . $operation . '?', $quantity)]);
    }

    /**
     * Naskladní dané množství produktů.
     * @param int $productId ID produktu
     * @param int $quantity počet produktů
     */
    public function addToStock($productId, $quantity)
    {
        $this->updateStock($productId, $quantity, '+');
    }

    /**
     * Odebere dané množství produktů ze skladu.
     * @param int $productId ID produktu
     * @param int $quantity počet produktů
     */
    public function removeFromStock($productId, $quantity)
    {
        $this->updateStock($productId, $quantity, '-');
    }

    /**
     * Odstraní produkt korektním způsobem.
     * @param string $productUrl URL produktu
     */
    public function removeProduct($productUrl)
    {
        $product = $this->getProduct($productUrl);
        $this->cleanProduct($product->{self::COLUMN_ID}, true);
    }

    /**
     * Odstraní variantu korektním způsobem.
     * @param string $productUrl URL produktu
     */
    public function removeVariation($id)
    {
        $product = $this->getProductFromId($id);
        $this->cleanProduct($product->{self::COLUMN_ID}, true);
    }

    /**
     * Odstraní platební metodu a vrátí její jméno pro zobrazení ve flashMessage
     * @param $productId
     * @return mixed
     */
    public function removeMethod($productId)
    {
        $product = $this->getProductFromId($productId);
        $productName = $product->{self::COLUMN_TITLE};
        $this->cleanProduct($product->{self::COLUMN_ID}, false);
        return $productName;
    }

    /**
     * Pokusí se odstranit produkt, pokud není vázaný na objednávku.
     * @param int $productId ID produktu k odstranění
     * @param bool $removeImages dotaz zda odstranit i obrázky
     */
    private function cleanProduct($productId, $removeImages = false)
    {
        if ($removeImages)
            $this->fileRelationsManager->removeEntityImages(self::COLUMN_ID, $productId);

        try {
            // Pokusí se starý produkt odstranit, pokud není navázaný na fakturu.
            $this->database->table(self::TABLE_NAME)->where(self::COLUMN_ID, array($productId))->delete();
        } catch (ForeignKeyConstraintViolationException $ex) {
            // Jinak ho ponechá jako skrytý - buď skrývám celý produkt nebo skrávam metodu, která je už defaultně skrytá.
            $q = $this->database->table(self::TABLE_NAME)
                ->where(self::COLUMN_ID, array($productId))
                ->update(array(self::COLUMN_STATUS => Status::HIDDEN));

        }

        // Odstraní produkt z kategorií.
        $this->database->table(self::TABLE_NAME . '_' . CategoryManager::TABLE_NAME)
            ->where(self::COLUMN_ID, array($productId))->delete();
    }

    /**
     * Odstraní obrázek produktu.
     * @param int $productId ID produktu
     * @param int $imageIndex index obrázku (0 je první)
     */
    public function removeProductImage($productId, $imageIndex)
    {
        $imagesCount = $this->getImagesCount($productId);
        $this->productImageManager->removeProductImage($productId, $imageIndex, $imagesCount);
        $this->database->table(self::TABLE_NAME)
            ->where(self::COLUMN_ID, array($productId))
            ->update(array(self::COLUMN_IMAGE_COUNT => ($imagesCount - 1)));
    }

}
