<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Presenters;

use App\EshopModule\Components\Forms\ICurrencyDefaultFormFactory;
use App\EshopModule\Components\Forms\ICurrencyFormFactory;
use App\EshopModule\Model\CurrencyManager;
use App\EshopModule\Components\Forms\CurrencyFormControl;
use App\EshopModule\Components\Forms\CurrencyDefaultFormControl;
use App\Model\Enum\MessageType;
use App\Model\Enum\Status;
use Exception;

/**
 * @resourceDescription Zpracovává požadavky pro měnové kurzy
 * @package App\EshopModule\Presenters
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class CurrencyPresenter extends BaseEshopPresenter {

    /**
     * @var CurrencyManager
     */
    private $currencyManager;

    /**
     * @var ICurrencyFormFactory
     */
    private $currencyFormFactory;

    /**
     * @var ICurrencyDefaultFormFactory
     */
    private $currencyDefaultFormFactory;

    public function __construct(CurrencyManager $currencyManager, ICurrencyFormFactory $currencyFormFactory, ICurrencyDefaultFormFactory $currencyDefaultFormFactory) {
        parent::__construct();
        $this->currencyManager = $currencyManager;
        $this->currencyFormFactory = $currencyFormFactory;
        $this->currencyDefaultFormFactory = $currencyDefaultFormFactory;
    }

    /**
     * Nastavení layoutu pro backend. Metoda se volá před render metodami.
     * Všechny metody presneteru jsou součástí backendu - jeden layout
     */
    public function beforeRender() {
        parent::beforeRender();
        $this->setLayout('layoutBackend');

        $this->template->title = $this->translator->translate('eshop.settings.title');
        $this->template->headerManagementCurrencies = $this->translator->translate('eshop.currency.title');
        $this->template->headerDefaultCurrency = $this->translator->translate('eshop.currency.defaultCurrency');
        $this->template->currencies = $this->currencyManager->getAllCurrencies();

        $defaultCurrency = $this->currencyManager->getDefaultCurrency()[CurrencyManager::COLUMN_ID];
        $this['defaultCurrencyForm']->setDefaults(
                array(
                    CurrencyManager::COLUMN_DEFAULT_CURRENCY => $defaultCurrency
                )
        );
    }

    /**
     * @actionName management
     * @actionDescription Administrativní nastavení pro měnové kurzy - výpis i možnost editace.
     * Předá do šablony všechny měny a informace i nich.
     */
    public function renderManagement() {
        $this->template->headerNewCurrency = $this->translator->translate('eshop.currency.newCurrency');
    }

    /**
     * @actionName manage
     * @actionDescription Editace měny, šablona je stejná jako přehled, akorát tato akce zpracovává nastevení
     * Naplní formulář hodnotami, pokud editujeme existující měnu a nastaví jiný nadpis pro formulář
     * @param int $id ID měny, kterou editujeme
     */
    public function actionManage($id) {
        if ($id && ($currency = $this->currencyManager->getCurrency($id))) {
            // IRow je read-only, pro manipulaci s ním ho musíme převést na pole.
            $currencyArray = $currency->toArray();

            if ($currency[CurrencyManager::COLUMN_STATUS] !== Status::PUBLISHED) {
                // Zamezím editaci skryté měny - get product nic vrátí i skrytou měnu
                $this->redirect(':Eshop:Currency:management');
            } else {
                // Nastavení hodnot do formuláře
                $this['currencyForm']->setDefaults($currencyArray);
                $this->template->headerNewCurrency = $this->translator->translate('eshop.currency.editCurrency', ['currencyName' => $currencyArray[CurrencyManager::COLUMN_NAME]]);
            }
        }
    }

    /**
     * Editace měny
     * Použije se stejná šablona jako pro výpis všech měn, jen do formuláře se dosadí hodnoty.
     * Líší se pouze v nadpisu
     */
    public function renderManage() {
        $this->setView('management');
    }

    /**
     * @actionName deleteCurrency
     * @actionDescription Smazání jednotlivého nastavení pro měnu
     */
    public function handleDeleteCurrency($id) {
        try {
            $currencyName = $this->currencyManager->removeCurrency($id);
            // Zaznamení akce do DB
            $this->logAction('Smazání měny ' . $currencyName);
            $this->presenter->flashMessage($this->translator->translate('eshop.currency.flashes.deleteSucces', ['currencyName' => $currencyName]), MessageType::SUCCESS);
        } catch (Exception $e) {
            $this->presenter->flashMessage($e->getMessage(), MessageType::ERROR);
        }

        if ($this->presenter->isAjax()) {
            //$this->presenter->redrawControl('flashes');
            //$this['productDatagrid']->reload();
            $this->presenter->redrawControl('ajaxNotification');
            $this->redrawControl("currenciesSnippet");
        } else {
            $this->presenter->redirect('this');
        }
    }

    /**
     * @return CurrencyFormControl
     */
    protected function createComponentCurrencyForm() {
        return $this->currencyFormFactory->create();
    }

    /**
     * @return CurrencyDefaultFormControl
     */
    protected function createComponentDefaultCurrencyForm() {
        return $this->currencyDefaultFormFactory->create();
    }

}
