<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Presenters;

use App\EshopModule\Components\DataGrids\IOrderDataGridFactory;
use App\EshopModule\Components\Forms\IAddProductToOrderFormFactory;
use App\EshopModule\Components\Forms\IOrderFormFactory;
use App\Model\Enum\MessageType;
use App\Components\PersonFormControl;
use App\Components\IPersonFormFactory;
use Latte\Runtime\Filters;
use App\EshopModule\Model\OrderManager;
use App\Model\PersonManager;
use Nette\Application\Responses\TextResponse;
use Nette\Application\UI\Form;
use Nette\Bridges\ApplicationLatte\Template;
use Nette\Database\Table\IRow;
use Nette\Http\IResponse;
use Nette\InvalidArgumentException;
use Nette\Utils\ArrayHash;
use App\Services\OrderService;
use App\EshopModule\Components\Forms\OrderFormControl;

/**
 * @resourceDescription Zpracovává práci s objednávkami v systému.
 * Akce, které nejsou umístěny v presenteru, ale patří pod něj
 * @actionName pdf
 * @actionDescription Vyrenderuje PDF faktury.
 * @actionName setState
 * @actionDescription Změní stav objednávky, pokusí se odeslat mail a zpracovat nový stav (nastavit ho).
 * 
 * @package App\EshopModule\Presenters
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class OrderPresenter extends BaseEshopPresenter {

    /** @var OrderManager */
    private $orderManager;

    /** @var PersonManager Instance třídy modelu pro práci s osobami v e-shopu. */
    private $personManager;

    /** @var IPersonFormFactory Instance třídy továrničky na tvorbu formulářů pro správu osob. */
    private $personFormFactory;

    /** @var OrderService Služba pro OrderPresentery, která poskytne společné metody pro naplnění šablon */
    private $orderService;

    /** @var IOrderDataGridFactory */
    private $orderDataGridFactory;

    /** @var IAddProductToOrderFormFactory */
    private $addProductToOrderFormFactory;

    /** @var IOrderFormFactory */
    private $orderFormFactory;

    public function __construct(
    OrderManager $orderManager, PersonManager $personManager, IPersonFormFactory $personFormFactory, OrderService $orderService, IOrderDataGridFactory $orderDataGridFactory, IAddProductToOrderFormFactory $addProductToOrderFormFactory, IOrderFormFactory $orderFormFactory) {
        parent::__construct();
        $this->orderManager = $orderManager;
        $this->personManager = $personManager;
        $this->personFormFactory = $personFormFactory;
        $this->orderService = $orderService;
        $this->orderDataGridFactory = $orderDataGridFactory;
        $this->addProductToOrderFormFactory = $addProductToOrderFormFactory;
        $this->orderFormFactory = $orderFormFactory;
    }

    /**
     * Vytváří a vrací komponentu formuláře pro správu osoby.
     * @return PersonFormControl formulář pro správu osoby
     */
    protected function createComponentPersonForm() {
        $personForm = $this->personFormFactory->create();
        $personForm->formPath = self::FORM_PATH;
        return $personForm;
    }

    /**
     * @return OrderFormControl
     */
    protected function createComponentOrderForm() {
        return $this->orderFormFactory->create();
    }

    /**
     * @return \App\EshopModule\Components\Forms\AddProductToOrderFormControl
     */
    protected function createComponentAddProductToOrderForm() {
        return $this->addProductToOrderFormFactory->create();
    }

    /**
     * @return \App\EshopModule\Components\DataGrids\OrderDataGridControl
     */
    protected function createComponentOrderDataGrid() {
        return $this->orderDataGridFactory->create();
    }

    /**
     * @ctionName management
     * @actionDescription Výpis objednávek pro jejich administraci. 
     */
    public function renderManagement() {
        $this->setLayout('layoutBackend');
    }

    /**
     * @actionName manage
     * @actionDescription Správa objednávky.
     * @param int $e_order_id ID objednávky
     */
    public function actionManage($e_order_id) {
        // Získání příslušné objednávky.
        $order = $this->orderManager->getOrder($e_order_id);

        // Nastavení hodnot formulářům.
        $this['orderForm']->setDefaults([
            'e_order_id' => $order[OrderManager::COLUMN_ID],
            'issued' => Filters::date($order[OrderManager::COLUMN_ISSUED], 'j.n.Y'),
            'due_date' => Filters::date($order[OrderManager::COLUMN_DUE_DATE], 'j.n.Y')
        ]);

        $this['addProductToOrderForm']->setDefaults(['e_order_id' => $order[OrderManager::COLUMN_ID]]);

        // Odebereme standardní tlačítko pro odeslání formuláře.
        /** @var Form $form */
        $form = $this['personForm']['personForm'];
        $form->removeComponent($form['send']);

        // Doplníme vlastní zpracování k úspěšnému odeslání formuláře pro editaci osob.
        $this['personForm']->onSuccess[] = function (Form $form, ArrayHash $values) use ($order) {
            if (isset($values->person_id)) {
                // Po uložení nové osoby její údaje přiřadí objednávce a osoba samotná se odstraní.
                $this->orderManager->setPerson($values->person_id, $order[OrderManager::COLUMN_ID], $order[OrderManager::COLUMN_BUYER]);
                $this->personManager->deletePerson($values->person_id);
                // Zaznamení akce do DB
                 $this->logAction('Editace údajů osoby v objednávce ' . $order[OrderManager::COLUMN_NUMBER]);
                // Info o zprávě
                $this->flashMessage($this->translator->translate('eshop.order.flashes.personEditSaveSuccess'), MessageType::SUCCESS);
                // Překreslení snippetu
                $this->redrawControl('buyerDetail');
                $this->redrawControl('personForm');
            } else {
                $this->getHttpResponse()->setCode(IResponse::S500_INTERNAL_SERVER_ERROR);
                $this->sendPayload();
                // Info o zprávě
                $this->flashMessage($this->translator->translate('eshop.order.flashes.personEditSaveError'), MessageType::ERROR);
            }

            $this->redrawControl('ajaxNotification');
        };
    }

    /**
     * Správa konkrétní objednávky.
     * @param int $e_order_id ID objednávky
     */
    public function renderManage($e_order_id) {
        $this->setLayout('layoutBackend');
        /** @var Template $template */
        $template = $this->getTemplate();
        $order = $this->orderService->fillTemplateWithOrderDetail($template, $e_order_id);
        $this->template->title = $this->translator->translate('eshop.order.editOrder', ['orderNumber' => $order[OrderManager::COLUMN_NUMBER]]);

        // Získává celkovou cenu dané objednávky bez uživatelových slev.
        $template->addFilter('priceWithoutSales', function (IRow $order) {
            return $this->orderManager->getOrderSummary($order[OrderManager::COLUMN_ID], FALSE, FALSE, FALSE)[OrderManager::ATTR_PRICE];
        });
    }

    /**
     * @actionName editProduct
     * @actionDescription Editace počtu produktu v objednávce.
     * @param int $orderId   ID objednávky
     * @param int $productId ID produktu
     * @param int $quantity  počet kusů; pokud je 0, produkt je z objednávky odstraněn
     */
    public function handleEditProduct($orderId, $productId, $quantity) {
        try {
            $this->orderManager->updateProductInOrder($quantity, $productId, $orderId);
            // Zaznamení akce do DB
            $this->logAction('Editace počtu kusů produktu ' . $productId . ' v ojednávce ' . $orderId . ' na počet ' . $quantity);
            // Info o zprávě
            $this->flashMessage("Produkty v objednávce byly úspěšně změněny.", MessageType::SUCCESS);
            $this->redrawControl('products');
            $this->redrawControl('summary');
        } catch (InvalidArgumentException $e) {
            $this->getHttpResponse()->setCode(IResponse::S400_BAD_REQUEST);
            $this->sendResponse(new TextResponse($e->getMessage()));
            // Info o zprávě
            $this->flashMessage("Chybně zadaný počet produktů", MessageType::ERROR);
        }

        $this->redrawControl('ajaxNotification');
    }

}
