<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\EshopModule\Presenters;

use App\EshopModule\Components\DataGrids\ICustomersDataGridFactory;
use App\Model\Enum\MessageType;
use App\Model\PersonManager;
use App\Components\IPersonFormFactory;
use App\Components\PersonFormControl;
use App\EshopModule\Components\DataGrids\CustomerDataGridControl;
use App\EshopModule\Model\OrderManager;
use App\EshopModule\Model\SettingsManager;
use App\Model\UserManager;
use Nette\Database\Table\IRow;
use Nette\Forms\Form;
use Nette\Utils\ArrayHash;
use App\Services\FormatHelperService;

/**
 * @resourceDescription Zpracovává zákazníky v eshopu, případně nastavení prodejce, účetního pro eshop
 * @package App\EshopModule\Presenters
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class PersonPresenter extends BaseEshopPresenter {

    /** @var IPersonFormFactory Instance třídy továrničky na tvorbu formulářů pro správu osobami. */
    private $personFormFactory;

    /** @var PersonManager Instance třídy modelu pro práci s osobami. */
    private $personManager;

    /** @var OrderManager Instance třídy modelu pro práci s objednávkami. */
    private $orderManager;

    /** @var SettingsManager Instance třídy modelu pro práci s účetním nastavením. */
    private $settingsManager;

    /** @var FormatHelperService Pomocná služba pro formátování data, jména adresy atd. */
    private $formatHelperService;

    /** @var ICustomersDataGridFactory */
    private $customersDataGridFactory;

    public function __construct(
    IPersonFormFactory $personFormFactory, PersonManager $personManager, OrderManager $orderManager, SettingsManager $settingsManager, FormatHelperService $formatHelperService, ICustomersDataGridFactory $customersDataGridFactory) {
        parent::__construct();
        $this->personFormFactory = $personFormFactory;
        $this->personManager = $personManager;
        $this->orderManager = $orderManager;
        $this->settingsManager = $settingsManager;
        $this->formatHelperService = $formatHelperService;
        $this->customersDataGridFactory = $customersDataGridFactory;
    }

    /**
     * @actionName manage
     * @actionDescription Editace dat osoby uživatelem - nenaruší již existující objednávky a faktury.
     */
    public function actionManage() {
        $this->setLayout('layoutBackend');
        // Pokud jsem v adminu, chci formulář renderovat pomocí defaultního renderu
        $this['personForm']->formPath = self::FORM_PATH;

        // Aktulní nastavení, jinak budu editovat sám sebe
        if($settings = $this->settingsManager->getSettings()){
            //User id
            $userId = $this->personManager->getUserIdByPersonId($settings[SettingsManager::COLUMN_SELLER]);
            $this->template->userId = $userId;

            // Nasetoval jsem do formu ID prodejce s jeho ID uctu
            $this['personForm']->setDefaults(
                    [
                        UserManager::COLUMN_ID => $userId,
                        PersonManager::COLUMN_ID => $settings[SettingsManager::COLUMN_SELLER]
                    ]
            );
        }

        // Doplníme vlastní zpracování k úspěšnému odeslání formuláře pro správu osob.
        $this['personForm']->onSuccess[] = function (Form $form, ArrayHash $values) {
            if (isset($values->person_id)) {
                // Zaznamení akce do DB
                $this->logAction('Změna údajů prodejce.');
                $this->flashMessage($this->translator->translate('eshop.person.flashes.saveSuccess'), MessageType::SUCCESS);
            } else {
                $this->flashMessage($this->translator->translate('eshop.person.flashes.saveError'), MessageType::ERROR);
            }

            $this->redirect('this');
        };
    }

    /**
     * @actionName customerManagement
     * @actionDescription Výpis všech osob (persons), které již v eshopu nakoupili, nebo jsou registrovány - fyzické osoby
     */
    public function renderCustomerManagement() {
        $this->setLayout('layoutBackend');
        // Doplnění šablony o užitečné filtry
        $this->template->addFilter('name', [$this->formatHelperService, 'formatName']);
    }

    /**
     * @actionName personDetail
     * @actionDescription Detail kontrétní osoby (person)
     * @param int $person_id ID osoby
     */
    public function renderPersonDetail($person_id) {
        $this->setLayout('layoutBackend');

        $this->template->person = $this->personManager->getById(PersonManager::TABLE_NAME, PersonManager::COLUMN_ID, $person_id);
        $this->template->personOrders = $this->orderManager->getPersonOrders($person_id);
        $this->template->orderStates = $this->orderManager->getPrettyOrderStates();

        // Doplnění šablony o užitečné filtry
        $this->template->addFilter('name', [$this->formatHelperService, 'formatName']);
        $this->template->addFilter('address', [$this->formatHelperService, 'formatAddress']);

        // Získává celkovou cenu dané objednávky s uživatelovými slevami.
        $this->template->addFilter('priceWithSales', function (IRow $order) {
            return $this->orderManager->getOrderSummary($order[OrderManager::COLUMN_ID], FALSE)[OrderManager::ATTR_PRICE];
        });
    }

    /**
     * Vytváří a vrací komponentu formuláře pro správu osoby.
     * @return PersonFormControl formulář pro správu osoby
     */
    protected function createComponentPersonForm() {
        return $this->personFormFactory->create();
    }

    /**
     * @return CustomerDataGridControl
     */
    protected function createComponentCustomerPersonDataGrid() {
        return $this->customersDataGridFactory->create();
    }

}
