<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\FrontModule\Components;

use App\Model\Enum\MessageType;
use Nette\Application\Responses\TextResponse;
use Nette\Application\UI\Control;
use Nette\Application\UI\Form;
use App\Forms\BaseFormFactory;
use Nette\Http\Request;
use Nette\InvalidArgumentException;
use Nette\Utils\ArrayHash;
use App\EshopModule\Model\OrderManager;
use App\EshopModule\Model\ProductAttributeManager;
use App\EshopModule\Model\ProductManager;
use Nette\Http\Response;
use Nette\Application\Responses\JsonResponse;

/**
 * Komponenta pro formulář přidávání do košíku
 * @package App\FrontModule\Components
 *
 * @author Petr Vojáček <petr at ideatech.cz>
 */
class CartFormControl extends Control
{

    /** Cesta k souboru šablony pro tuto komponentu. */
    const TEMPLATE = '/../templates/components/CartForm/form.latte';

    /** @var BaseFormFactory Továrnička na formuláře. */
    private $formFactory;

    /** @var OrderManager */
    private $orderManager;

    /** @var ProductManager */
    private $productManager;

    /** @var ProductAttributeManager */
    private $productAttributeManager;

    /** @var Request */
    private $httpResponse;

    /**
     * @var int
     */
    private $defaultVariant;

    /**
     * CartFormControl constructor.
     * @param BaseFormFactory $formFactory
     * @param OrderManager $orderManager
     * @param ProductManager $productManager
     * @param ProductAttributeManager $productAttributeManager
     * @param Response $httpResponse
     */
    public function __construct(BaseFormFactory $formFactory, OrderManager $orderManager, ProductManager $productManager, ProductAttributeManager $productAttributeManager, Response $httpResponse)
    {
        parent::__construct();
        $this->formFactory = $formFactory;
        $this->orderManager = $orderManager;
        $this->productManager = $productManager;
        $this->productAttributeManager = $productAttributeManager;
        $this->httpResponse = $httpResponse;
    }

    /** Vykreslí komponentu formuláře pro správu osob. */
    public function render($product, $mode = null, $skin = null)
    {
        // Nastaví šablonu komponenty.
        $this->template->setFile(__DIR__ . self::TEMPLATE);
        $this->template->product = $product;

        $this->template->mode = $mode;
        $this->template->skin = $skin;

        if ($product->type == ProductManager::TYPE_VARIABLE) {
            $variants = $this->productManager->getVariants($product->product_id);
            $attributesArray = [];
            foreach ($variants as $variant) {
                $attributes = $this->productAttributeManager->getProductAttributes($variant->product_id);
                foreach ($attributes as $attribute) {
                    $attributesArray[$attribute->attribute->name][$variant->product_id] = $attribute->attribute_values->value;
                }
            }

            $this->template->attributes = $attributesArray;
        }

        if ($this->defaultVariant) {
            $this->template->defaultVariant = $this->defaultVariant;
        }

        // Vykreslí komponentu.
        $this->template->render();
    }

    public function setDefaultVariant($variantId)
    {
        $this->defaultVariant = $variantId;
    }

    /**
     * Vytváří a vrací komponentu formuláře pro přidání do košíku.
     * @return Form formulář pro přidání do košíku
     */
    protected function createComponentCartForm()
    {
        $form = $this->formFactory->create();

        $form->addHidden('product_id');
        $form->addText('quantity')
            ->setType('number')
            ->setRequired()
            ->addRule(Form::INTEGER)
            ->setValue(1);
        $form->addSubmit('add_to_cart');

        $form->onSuccess[] = function (Form $form, ArrayHash $values) {
            $this->cartFormSucceeded($form, $values);
        };

        return $form;
    }

    /**
     * Funkce se vykoná při úspěšném odeslání formuláře pro přidání do košíku.
     * @param Form $form formulář pro přidání do košíku
     * @param ArrayHash $values odeslané hodnoty formuláře
     */
    public function cartFormSucceeded($form, $values)
    {

        $formData = $form->getHttpData();
        if (isset($formData['variants'])) {
            $values->product_id = $formData['variants'];
        }

        $this->handleAddToCart($values->product_id, $values->quantity);
    }

    /**
     * @param $productId
     * @param $quantity
     */
    public function handleAddToCart($productId, $quantity)
    {
        $presenter = $this->presenter;
        $product = $this->productManager->getProductFromId($productId);
        if ($product[ProductManager::COLUMN_STOCK] <= 0) {
            $this->presenter->payload->message = (object)array('error' => 'stock');
        } else {
            try {
                $success = $this->orderManager->addProducts($productId, $quantity);

                if (!$success) {
                    if ($presenter->isAjax()) {
                        $presenter->sendResponse(new TextResponse('Daný produkt pravděpodobně není dostupný!'));
                    } else {
                        $presenter->flashMessage('Daný produkt pravděpodobně není dostupný!');
                    }
                } else {
                    //$presenter->flashMessage('Produkt byl přidán do košíku.');
                    $this->presenter->template->addedProduct = $this->productManager->getProductFromId($productId);
                    $this->presenter->redrawControl('addedProduct');
                    /* $presenter->sendResponse(new JsonResponse([
                      'productName' => $product[ProductManager::COLUMN_TITLE],
                      'productPrice' => $product[ProductManager::COLUMN_PRICE]
                      ])); */
                }
            } catch (InvalidArgumentException $ex) {
                $presenter->flashMessage($ex->getMessage(), MessageType::ERROR);
            }
        }

        if ($this->presenter->isAjax()) {
            $this->presenter->template->product_id = $productId;
            $this->presenter->template->quantity = $quantity;
            // Překreslení informtivního modalu, že produktu není dostupný
            $this->presenter->redrawControl('unavailabilityModal');
            // Ajaxové zpracování - překreslení snippetů košíku v headeru
            $this->presenter->redrawControl("basketSnippet");
            // Pokud přidávám produkt v košíku - je tam jiný snippet
            $this->presenter->redrawControl("cartFormSnippet");
            // Překreslím snippet s ajaxovýma zprávama
            $this->presenter->redrawControl("ajaxNotification");
        } else {
            // Není to ajax - překreslení stránky
            $this->presenter->redirect('this');
        }
    }

    /**
     * Metoda, která se volá po kliknutí na tlačítko v modálu, který upozorňuje na nedostupnonost produktu skladem
     * Metoda je podobná jako handleAddToCart, ale zde se už nekontroluje dostupnost produktu skladem
     * @param int $id ID produktu, který se přidává do košíku
     * @param int $amount Počrt kusů produktu
     */
    public function handleBuyWithFreeStore($id, $amount)
    {
        $presenter = $this->presenter;

        try {
            $success = $this->orderManager->addProducts($id, $amount);

            if (!$success) {
                if ($presenter->isAjax()) {
                    $presenter->sendResponse(new TextResponse('Daný produkt pravděpodobně není dostupný!'));
                } else {
                    $presenter->flashMessage('Daný produkt pravděpodobně není dostupný!');
                }
            } else {
                $this->presenter->template->addedProduct = $this->productManager->getProductFromId($id);
                $this->presenter->redrawControl('addedProduct');
            }
        } catch (InvalidArgumentException $ex) {
            $presenter->flashMessage($ex->getMessage(), MessageType::ERROR);
        }

        // Ajaxové zpracování - překreslení snippetů košíku a zpráv
        $presenter->redrawControl("basketSnippet");
        // Překreslím snippet s ajaxovýma zprávama
        $presenter->redrawControl("ajaxNotification");
    }

    public function handleUpdateStock($id = null)
    {
        $presenter = $this->presenter;
        $presenter->template->stock = $this->productManager->getProductFromId($id)[ProductManager::COLUMN_STOCK];
        $presenter->redrawControl('stock');
    }

}

/**
 * Interface ICartFormFactory
 * @package App\FrontModule\Components
 */
interface ICartFormFactory
{

    /**
     * @return CartFormControl
     */
    public function create();
}
