<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\FrontModule\Components;

use App\Model\Enum\MessageType;
use Nette\Application\UI\Control;
use Nette\Application\UI\Form;
use App\Forms\BaseFormFactory;
use Nette\InvalidArgumentException;
use Nette\InvalidStateException;
use Nette\Utils\ArrayHash;
use App\EshopModule\Model\OrderManager;
use App\EshopModule\Model\DiscountLevelsManager;
use App\Model\PersonManager;
use App\Components\IPriceControlFactory;

/**
 * Komponenta pro formulář přidávání do košíku
 * @package App\FrontModule\Components
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class CartManagementFormFactory extends Control {

    /** Cesta k souboru šablony pro tuto komponentu. */
    const TEMPLATE = '/../templates/components/CartManagementForm/form.latte';

    /** @var BaseFormFactory Továrnička na formuláře. */
    private $formFactory;

    /** @var OrderManager */
    private $orderManager;

    /** @var PersonManager */
    public $personManager;

    /** @var DiscountLevelsManager */
    public $discountLevels;

    /**
     * @var IPriceControlFactory
     */
    private $priceControlFactory;

    /**
     * CartFormControl constructor.
     * @param BaseFormFactory $formFactory
     * @param OrderManager $orderManager
     * @param PersonManager $personManager
     * @param DiscountLevelsManager $discountLevels
     * @param IPriceControlFactory $priceControlFactory
     */
    public function __construct(BaseFormFactory $formFactory, OrderManager $orderManager, PersonManager $personManager, DiscountLevelsManager $discountLevels, IPriceControlFactory $priceControlFactory) {
        parent::__construct();
        $this->formFactory = $formFactory;
        $this->orderManager = $orderManager;
        $this->personManager = $personManager;
        $this->discountLevels = $discountLevels;
        $this->priceControlFactory = $priceControlFactory;
    }

    /** Vykreslí komponentu formuláře pro správu osob. */
    public function render($couponDetail, $cart, $settings, $user) {
        // Nastaví šablonu komponenty.
        $this->template->setFile(__DIR__ . self::TEMPLATE);

        // Předání potřebných parametrů do šablony, hlavně pro komponentu price + coupon detail, jestli je už nastaven do objednávky
        $this->template->couponDetail = $couponDetail;
        $this->template->cart = $cart;
        $this->template->settings = $settings;
        $this->template->user = $user;

        // Vykreslí komponentu.
        $this->template->render();
    }

    /**
     * Vytváří a vrací komponentu formuláře pro správu košíku.
     * Pro každou položku košíku vytvoříme kontrolku, jejíž název bude unikátní ID produktu, ke kterému se váže.
     * (key => val - porductID => pocet). S tímto formátem pracuje metoda updateCart()
     * Průchod produkty realzován přes array_map
     * Kontrola rozsahu se realizuje i v modelu, ale pro jistotu kontola na straně uživatele s podbarvením špatných hodnot
     * @return Form formulář pro správu košíku
     */
    protected function createComponentCartManagementForm() {
        $form = $this->formFactory->create();

        array_map(function ($product) use ($form) {
            $form->addHidden($product->product_id)
                    ->setDefaultValue($product->quantity);
        }, $this->orderManager->getProducts());

        $form->addText('coupon_code', 'Kupón')
                ->setRequired(FALSE)
                ->setAttribute('placeholder', 'Kód kupónu')
                ->getControlPrototype()
                ->addClass('form-control');

        $form->addSubmit('refresh', 'Přepočítat');
        $form->addSubmit('continue', 'Pokračovat v nákupu');
        $form->addSubmit('checkout', 'Pokračovat');
        $form->addSubmit('useCoupone', 'Použít slevový kupón');

        $form->onSuccess[] = [$this, 'cartManagementFormSucceeded'];

        return $form;
    }

    /**
     * Funkce se vykonaná při úspěšném odeslání formuláře pro přidání do košíku.
     * @param Form $form formulář pro přidání do košíku
     */
    public function cartManagementFormSucceeded(Form $form, ArrayHash $values) {
        $presenter = $form->getPresenter();

        $couponCode = $values->coupon_code;
        $values->offsetUnset('coupon_code');
        $this->orderManager->updateCart($values);

        /** @var SubmitButton $sentButton */
        if (($sentButton = $form->isSubmitted())) {
            $sentButtonName = $sentButton->getName();

            switch ($sentButtonName) {
                case "continue":
                    $presenter->redirect(':Front:Page:', '');
                    break;
                case "checkout":
                    if ($presenter->getUser()->isLoggedIn()) {
                        try {
                            $this->orderManager->setPerson($this->personManager->getPersonId($presenter->getUser()->getId()));
                            $presenter->redirect(':Front:Person:manageInOrder');
                        } catch (InvalidArgumentException $e) {
                            $presenter->redirect(':Front:Person:register');
                        }
                    } else {
                        $presenter->redirect(':Front:Person:register');
                    }
                    break;
                case "useCoupone":
                    try {
                        $couponId = $this->discountLevels->useCoupon($couponCode);
                        // Nastavit objednávce id kupónu
                        $this->orderManager->setCouponDiscount($couponId);
                        $presenter->flashMessage('Kupón byl přičten k Vaší objednávce', MessageType::SUCCESS);
                    } catch (InvalidStateException $e) {
                        $presenter->flashMessage($e->getMessage(), MessageType::ERROR);
                    }
                    break;
                default:
                    break;
            }
        }

        if ($presenter->isAjax()) {
            $presenter->redrawControl("ajaxNotification");
            $presenter->redrawControl("cartFormSnippet");
            // Ajaxové zpracování - překreslení snippetů košíku v headeru
            $this->presenter->redrawControl("basketSnippet");
        } else {
            $presenter->redirect('this');
        }
    }

    /**
     * @return \App\Components\PriceControl
     */
    protected function createComponentPrice() {
        return $this->priceControlFactory->create();
    }

}

/**
 * Interface ICartManagementFormFactory
 * @package App\FrontModule\Components
 */
interface ICartManagementFormFactory {

    /**
     * @return CartManagementFormFactory
     */
    public function create();
}
