<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\FrontModule\Components;

use App\Model\Enum\MessageType;
use Nette\Application\UI\Control;
use Nette\Application\UI\Form;
use App\Forms\BaseFormFactory;
use Nette\Mail\Message;
use Nette\Mail\SendmailMailer;
use Nette\InvalidStateException;
use Nette\Utils\ArrayHash;
use App\CoreModule\Model\EmailSettingsManager;
use App\CoreModule\Model\ContactMessageManager;

/**
 * Komponenta pro kontaktni formular
 * @package App\FrontModule\Components
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class ContactFormControl extends Control {

    /** Cesta k souboru šablony pro tuto komponentu. */
    const TEMPLATE = '/../templates/components/ContactForm/form.latte';

    /** Cesta ke globální šabloně formulářů. */
    const FORM_PATH = __DIR__ . '/../../../templates/components/form.latte';

    /** @var BaseFormFactory Továrnička na formuláře. */
    private $formFactory;

    /** @var EmailSettingsManager Instance třídy modelu pro práci s nastavením emailu. */
    private $emailSettingsManager;
    
    /** @var ContactMessageManager Instance třídy modelu pro ukládání zpráv do databáze. */
    private $contactMessageManager;

    /**
     * ContactFormControl constructor.
     * @param BaseFormFactory $formFactory
     * @param EmailSettingsManager $emailSettingsManager automaticky injektovaná třída modelu pro práci s emaily
     * @param ContactMessageManager $contactMessageManager model pro ukládání zpráv do databáze
     */
    public function __construct(BaseFormFactory $formFactory, EmailSettingsManager $emailSettingsManager, ContactMessageManager $contactMessageManager) {
        parent::__construct();
        $this->formFactory = $formFactory;
        $this->emailSettingsManager = $emailSettingsManager;
        $this->contactMessageManager = $contactMessageManager;
    }

    /** Vykreslí komponentu formuláře pro správu osob. */
    public function render() {
        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE);

        $this->template->formPath = self::FORM_PATH;

        // Vykreslí komponentu.
        $this->template->render();
    }

    public function createComponentContactForm() {
        $form = $this->formFactory->create();
        $form->elementPrototype->addAttributes(['class' => 'ajax']);
        $form->addText('name', 'Jméno')
                ->setRequired();

        $form->addText('email', 'Email')
                ->setType('email')
                ->addRule(Form::EMAIL, 'Email musí být ve správném tvaru.')
                ->setRequired();

        $form->addText('phone', 'Telefon');

        $form->addTextArea('message', 'Zpráva')
                ->setRequired()
                ->addRule(Form::MIN_LENGTH, 'Zpráva musí být minimálně %d znaků dlouhá.', 10);
        $form->addSubmit('submit', 'Odeslat');
        $form->onSuccess[] = function (Form $form, ArrayHash $values) {
            $this->succeeded($form, $values);
        };
        return $form;
    }

    private function succeeded(Form $form, ArrayHash $values) {
        $presenter = $form->getPresenter();
        $contactEmail = $this->emailSettingsManager->getPublishedRowByCondition(EmailSettingsManager::TABLE_NAME, EmailSettingsManager::COLUMN_EMAIL_TYPE, EmailSettingsManager::TYPE_CONTACT_MAIL);

        try {
            $mail = new Message;
            $mail->setFrom($values->email)
                    ->addTo($contactEmail->email)
                    ->setSubject('Email z webu')
                    ->setBody($values->message);

            $mailer = new SendmailMailer;
            $mailer->send($mail);
            
            // Uložím zprvá do databáze - přidám k ní ještě i sekci, odkud přišla
            $values->offsetSet(ContactMessageManager::COLUMN_SECITON, 'contactFrom');
            $this->contactMessageManager->saveValues(ContactMessageManager::TABLE_NAME, ContactMessageManager::COLUMN_ID, $values);

            $presenter->flashMessage('Email byl úspěšně odeslán.', MessageType::SUCCESS);
            if ($presenter->isAjax()) {
                // Překreslím snippet s ajaxovýma zprávama
                $presenter->redrawControl("ajaxNotification");
            } else {
                // Není to ajax - překreslení stránky
                $presenter->redirect('this');
            }
        } catch (InvalidStateException $ex) {
            $presenter->flashMessage('Email se nepodařilo odeslat.', MessageType::ERROR);
            if ($presenter->isAjax()) {
                // Překreslím snippet s ajaxovýma zprávama
                $presenter->redrawControl("ajaxNotification");
            } else {
                // Není to ajax - překreslení stránky
                $presenter->redirect('this');
            }
        }
    }

}

/**
 * Interface IContactFormFactory
 * @package App\FrontModule\Components
 */
interface IContactFormFactory {

    /**
     * @return ContactFormControl
     */
    public function create();
}
