<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\FrontModule\Components;

use App\Forms\BaseFormFactory;
use App\Mailing\PasswordMail;
use App\MailSender;
use App\Model\Enum\MessageType;
use App\Model\UserManager;
use App\Services\MailFactory;
use Nette\Application\UI\Control;
use Nette\Application\UI\Form;
use Nette\InvalidStateException;
use Nette\Security\Passwords;
use Nette\Utils\ArrayHash;
use Nette\Utils\Random;

/**
 * Komponenta pro obnovení zapomenutého hesla
 * @package App\FrontModule\Components
 *
 * @author Petr Vojáček <petr at ideatech.cz>
 */
class ForgotPasswordFormControl extends Control {

    /** Cesta k souborům šablon pro tuto komponentu. */
    const TEMPLATE_PATH = '/../templates/components/ForgotPasswordForm/';

    /**
     * @var UserManager
     */
    private $userManager;

    /**
     * @var BaseFormFactory
     */
    private $baseFormFactory;

    /**
     * @var MailFactory
     */
    private $mailFactory;

    public function __construct(UserManager $userManager, BaseFormFactory $baseFormFactory, MailFactory $mailFactory) {
        parent::__construct();
        $this->userManager = $userManager;
        $this->baseFormFactory = $baseFormFactory;
        $this->mailFactory = $mailFactory;
    }

    /**
     * Výchozí metoda pro vykreslení komponenty
     */
    public function render($formPath) {
        // Nastaví šablonu komponenty.
        $this->template->setFile(dirname(__FILE__) . self::TEMPLATE_PATH . 'form.latte');

        //Předání do šablony cestu pro automatické vykreslení formuláře
        $this->template->formPath = $formPath;

        // Vykreslí komponentu.
        $this->template->render();
    }

    public function createComponentForgotPasswordForm() {
        $form = $this->baseFormFactory->create();
        $form->elementPrototype->addAttributes(['class' => 'ajax']);

        $form->addText('email', 'Email')
                ->setType('email')
                ->addRule(Form::EMAIL, 'Email musí být ve správném tvaru.')
                ->setRequired();


        $form->addSubmit('submit', 'Odeslat')
                ->setAttribute('id', 'forgotPasswordSubmit');
        $form->onSuccess[] = function (Form $form, ArrayHash $values) {
            $this->succeeded($form, $values);
        };
        return $form;
    }

    private function succeeded(Form $form, ArrayHash $values) {
        $presenter = $form->getPresenter();
        $password = $this->generatePassword(10);

        try {
            $user = $this->userManager->findByEmail($values->email)
                    ->update([UserManager::COLUMN_PASSWORD_HASH => Passwords::hash($password)]);

            /** @var PasswordMail $email */
            $email = $this->mailFactory->createByType(PasswordMail::class, ['password' => $password, 'recipient' => $values->email]);
            $email->send();

            $this->mailSender->setTemplate(MailSender::TEMPLATE_PASSWORD)
                    ->setParams(['password' => $password])
                    ->addTo($values->email)
                    ->send();

            $presenter->flashMessage('Email s novým heslem byl úspěšně odeslán.', MessageType::SUCCESS);
            if ($presenter->isAjax()) {
                // Překreslím snippet s ajaxovýma zprávama
                $presenter->redrawControl("ajaxNotification");
            } else {
                // Není to ajax - překreslení stránky
                $presenter->redirect('this');
            }
        } catch (InvalidStateException $ex) {
            $presenter->flashMessage('Email se nepodařilo odeslat.', MessageType::ERROR);
            if ($presenter->isAjax()) {
                // Překreslím snippet s ajaxovýma zprávama
                $presenter->redrawControl("ajaxNotification");
            } else {
                // Není to ajax - překreslení stránky
                $presenter->redirect('this');
            }
        }
    }

    private function generatePassword($length) {
        return Random::generate($length);
    }

}

/**
 * Interface IForgotPasswordFormFactory
 * @package App\FrontModule\Components
 */
interface IForgotPasswordFormFactory {

    /**
     * @return ForgotPasswordFormControl
     */
    public function create();
}
