<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\FrontModule\Components;

use App\Model\Enum\MessageType;
use Nette\Application\UI\Control;
use Nette\Application\UI\Form;
use App\Forms\BaseFormFactory;
use Nette\Utils\ArrayHash;
use App\EshopModule\Model\ProductReviewManager;
use App\EshopModule\Model\ProductManager;
use App\FrontModule\Components\RatingControl;
use Nette\Database\UniqueConstraintViolationException;

/**
 * Komponenta pro přidávání hodnocení k produktům
 * @package App\FrontModule\Components
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class ReviewFormControl extends Control {
    
    /** Cesta k souboru šablony pro tuto komponentu. */
    const TEMPLATE = '/../templates/components/ReviewForm/form.latte';

    /** @var BaseFormFactory Továrnička na formuláře. */
    private $formFactory;

    /** @var ProductReviewManager */
    private $reviewManager;

    /** @var ProductManager */
    private $productManager;

    /**
     * ReviewFormControl constructor.
     * @param BaseFormFactory $formFactory
     * @param ProductReviewManager $reviewManager
     * @param ProductManager $productManager
     */
    public function __construct(BaseFormFactory $formFactory, ProductReviewManager $reviewManager, ProductManager $productManager) {
        parent::__construct();
        $this->formFactory = $formFactory;
        $this->reviewManager = $reviewManager;
        $this->productManager = $productManager;
    }
    
    /** Vykreslí komponentu formuláře pro přidání hodnocení k produktu. */
    public function render($formPath, $ratingWidget) {
        // Nastaví šablonu komponenty.
        $this->template->setFile(__DIR__ . self::TEMPLATE);
        $this->template->formPath = $formPath;
        $this->template->ratingWidget = $ratingWidget;

        // Vykreslí komponentu.
        $this->template->render();
    }
    
    /**
     * Pomocná metoda pro předávání defaultních hodnot
     * Metoda se volá z presenteru úplně stejně jak předtím
     * Provolání metod
     * @param Array $values Defaultní hodnoty pro formulář
     */
    public function setDefaults(Array $values){
        // Nastavení výchozí hodnoty formuláře pro hodnocení produktu.
        $this['reviewForm']->setDefaults($values);
    }

    /**
     * Vytváří a vrací komponentu formuláře pro hodnocení produktu.
     * @return Form formulář pro hodnocení produktu
     */
    protected function createComponentReviewForm() {
        $form = $this->formFactory->create();

        $form->addHidden('rating')->setHtmlId('rating');
        $form->addHidden('product_id');
        $form->addTextArea('content', 'Jak jste spokojení?')->setRequired();
        $form->addSubmit('send', 'Odeslat')
                ->setAttribute('id', 'productReviewSubmit')
                ->setAttribute('class', 'productReviewSubmit');

        $form->onSuccess[] = function (Form $form, ArrayHash $values) {
            $this->reviewFormSucceeded($form, $values);
        };

        return $form;
    }

    /**
     * Funkce se vykonaná při úspěšném odeslání formuláře pro hodnocení produktu.
     * @param Form      $form   formulář pro hodnocení produktu
     * @param ArrayHash $values odeslané hodnoty formuláře
     */
    public function reviewFormSucceeded($form, $values) {
        $presenter = $form->getPresenter();

        if (!$presenter->getUser()->isLoggedIn()) {
            $presenter->flashMessage('Recenze mohou přidávat jen přihlášení uživatelé.', MessageType::ERROR);
            $presenter->redirect('this');
        }
        // K recenzi doplníme ID aktuálně přihlášeného uživatele.
        $values['user_id'] = $presenter->getUser()->getId();
        try { // Pokusíme se přidat recenzi.
            $this->reviewManager->addReview($values);
            $this->productManager->updateRatings($values->product_id, $values->rating);
            $presenter->flashMessage('Recenze byla úspěšně přidána, děkujeme.', MessageType::SUCCESS);
            $presenter->redirect('this');
        } catch (UniqueConstraintViolationException $ex) {
            $presenter->flashMessage('Tento produkt jsi již hodnotil.', MessageType::ERROR);
        }
    }
    
    /**
     * Vytváří a vrací komponentu pro vykreslení ratingu.
     * @return RatingControl komponenta pro rating
     */
    protected function createComponentRating() {
        return new RatingControl();
    }

}

/**
 * Interface IReviewFormFactory
 * @package App\FrontModule\Components
 */
interface IReviewFormFactory {

    /**
     * @return ReviewFormControl
     */
    public function create();
}
