<?php

namespace App\FrontModule\Presenters;

use App\BlogModule\Model\CategoryManager;
use App\CmsModule\Model\PageManager;
use App\EshopModule\Model\OrderManager;
use App\EshopModule\Model\SettingsManager;
use App\FrontModule\Components\ForgotPasswordFormControl;
use App\FrontModule\Components\ICartFormFactory;
use App\FrontModule\Components\IContactFormFactory;
use App\FrontModule\Components\IForgotPasswordFormFactory;
use App\FrontModule\Components\IHeaderFactory;
use App\FrontModule\Components\IFooterFactory;
use App\FrontModule\Components\IPageMenuFactory;
use App\MediaModule\Components\IMainImageFactory;
use App\FrontModule\Components\IAddedProductModalControlFactory;
use App\Presenters\BasePresenter;
use Detection\MobileDetect;

/**
 * Základní presenter pro všechny ostatní presentery v FrontModule.
 * @package App\CoreModule\Presenters
 *
 * @author Petr Vojáček <petr@ideatech.cz>
 */
class BaseFrontPresenter extends BasePresenter {

    /**
     * @var CategoryManager
     */
    private $categoryManager;

    /**
     * @var SettingsManager
     */
    private $settingsManager;

    /**
     * @var OrderManager
     */
    private $orderManager;

    /**
     * @var PageManager
     */
    private $pageManager;

    /**
     * @var IForgotPasswordFormFactory
     */
    private $forgotPasswordFormFactory;

    /**
     * @var ICartFormFactory
     */
    private $cartFormFactory;

    /**
     * @var IMainImageFactory
     */
    private $mainImageFactory;

    /**
     * @var IContactFormFactory
     */
    private $contactFormFactory;

    /**
     * @var IHeaderFactory
     */
    private $headerFactory;

    /**
     * @var IFooterFactory
     */
    private $footerFactory;

    /**
     * @var IPageMenuFactory
     */
    private $pageMenuFactory;

    /**
     * @var IAddedProductModalControlFactory
     */
    private $addedProductModal;



    /**
     * @var null|string Adresa presenteru pro logování uživatele v rámci FrontModule. 
     */
    protected $loginPresenter = ':Front:Sign:frontLogin';

    /**
     * BaseFrontPresenter constructor.
     * @param CategoryManager $categoryManager
     * @param SettingsManager $settingsManager
     * @param OrderManager $orderManager
     * @param PageManager $pageManager
     * @param IForgotPasswordFormFactory $forgotPasswordFormFactory
     * @param ICartFormFactory $cartFormFactory
     * @param IMainImageFactory $mainImageFactory
     * @param IContactFormFactory $contactFormFactory
     * @param IHeaderFactory $headerFactory
     * @param IFooterFactory $footerFactory
     * @param IPageMenuFactory $pageMenuFactory
     * @param IAddedProductModalControlFactory $addedProductModal
     */
    public function injectDependencies(CategoryManager $categoryManager, SettingsManager $settingsManager, OrderManager $orderManager, PageManager $pageManager, IForgotPasswordFormFactory $forgotPasswordFormFactory, ICartFormFactory $cartFormFactory, IMainImageFactory $mainImageFactory, IContactFormFactory $contactFormFactory, IHeaderFactory $headerFactory, IFooterFactory $footerFactory, IPageMenuFactory $pageMenuFactory, IAddedProductModalControlFactory $addedProductModal)
    {
        parent::__construct();
        $this->categoryManager = $categoryManager;
        $this->settingsManager = $settingsManager;
        $this->orderManager = $orderManager;
        $this->pageManager = $pageManager;
        $this->forgotPasswordFormFactory = $forgotPasswordFormFactory;
        $this->cartFormFactory = $cartFormFactory;
        $this->mainImageFactory = $mainImageFactory;
        $this->contactFormFactory = $contactFormFactory;
        $this->headerFactory = $headerFactory;
        $this->footerFactory = $footerFactory;
        $this->pageMenuFactory = $pageMenuFactory;
        $this->addedProductModal = $addedProductModal;
    }

    /** Volá se před každou akcí presenteru úplně na začátku */
    public function startup() {
        parent::startup();

        /**
         * Přidává nový filtr components, může sloužit jako shortcody ve Wordpressu. Syntaxe je [[nazev_komponenty]].
         */
        $this->template->getLatte()->addFilter('components', function ($text) {
            $presenter = $this;
            return preg_replace_callback('~\[\[(\w+)\]\]~', function ($matches) use ($presenter) {
                ob_start();
                if (!isset($this->filters[$matches[1]])) {
                    $presenter->getComponent($matches[1])->render();
                } else {
                    \Tracy\Debugger::barDump('Nedefinovaná komponenta');
                }
                return ob_get_clean();
            }, $text);
        });
    }

    /**
     * Add MobileDetect to use in templates
     * @return \Nette\Application\UI\ITemplate
     */
    protected function createTemplate() {
        // Init template
        $template = parent::createTemplate();
        $template->_mobileDetect = new MobileDetect();

        return $template;
    }

    /** Volá se před vykreslením každého presenteru a předává společné proměné do celkového layoutu webu. */
    protected function beforeRender() {
        parent::beforeRender();

        //$this->orderManager->getOrder();
        // Předá kategorií do šablony
        $this->template->categories = $this->categoryManager->getCategories(false /* $this->getUser()->isInRole('admin') */);

        // Košík - obsahuje počet položek v košíku a výslednou cenu objednávky
        $this->template->cart = $this->orderManager->getOrderSummary();

        /**
         * Pokud aktuální modul není Cms, zkusíme najít stránku, která obsahuje odkaz na náš modul a
         * pošleme do šablony její informace.
         * Využítí při nastavení v administraci, že stránka odkazuje na presenter
         */
        if (!strpos($this->name, 'Cms:Page')) {
            $page = $this->pageManager->findPageByUrlString($this->name);
            if ($page) {
                $this->template->page = $page;
            }
        }

        $this->template->settings = $this->settingsManager->getSettings();
    }

    /**
     * Metoda, která nastavuje SEO elementy do stránky - title, popisek, description
     * Hodnoty se berou z databáze, pokud stránka není uložená v administraci, berou se defaulní hodnoty z dílčích presenterů
     * @param String|null $pageURL url, podle kterého se stránka vyhledá - stránky mají unikátní url. Pokud je null, nenajdu stránku a do šablony dosadím defaultní hodnoty
     * @param String $alternativeTitle v případě, že se stránka nepodaří najít, nastaví defaultní titulek z presenteru
     * @param String $alternativeDescription v případě, že se stránka nepodaří najít, nastaví defaultní popisek z presenteru
     * @param String $alternativeKeywords v případě, že se stránka nepodaří najít, nastaví defaultní klíčová slova z presenteru
     */
    protected function setPageSeoElements($pageURL, $alternativeTitle, $alternativeDescription, $alternativeKeywords) {
        // Najdu si stránku aktuality, pokud ji chci vůbec hledat
        if ($pageURL) {
            $page = $this->pageManager->findPageByUrlString($pageURL);
        } else {
            $page = null;
        }

        if ($page) {
            // Pokud stránka existuje, předám jí popisky z adminu
            $this->template->page = new \stdClass();
            $this->template->page->seo_title = $page->seo_title;
            $this->template->page->seo_description = $page->seo_description;
            $this->template->page->seo_keywords = $page->seo_keywords;
        } else {
            // Pokud neexistuje, nasetuji defaultní popisky
            $this->template->page = new \stdClass();
            $this->template->page->seo_title = $alternativeTitle;
            $this->template->page->seo_description = $alternativeDescription;
            $this->template->page->seo_keywords = $alternativeKeywords;
        }
    }

    /**
     * @return ForgotPasswordFormControl
     */
    protected function createComponentForgotPasswordForm() {
        return $this->forgotPasswordFormFactory->create();
    }

    /**
     * @return \App\FrontModule\Components\CartFormControl
     */
    protected function createComponentCartForm() {
        return $this->cartFormFactory->create();
    }

    /**
     * @return \App\MediaModule\Components\MainImageControl
     */
    protected function createComponentMainImage() {
        return $this->mainImageFactory->create();
    }

    /**
     * @return \App\FrontModule\Components\ContactFormControl
     */
    protected function createComponentContactForm() {
        return $this->contactFormFactory->create();
    }

    /**
     * @return \App\FrontModule\Components\HeaderControl
     */
    protected function createComponentHeader() {
        return $this->headerFactory->create();
    }

    /**
     * @return \App\FrontModule\Components\FooterControl
     */
    protected function createComponentFooter() {
        return $this->footerFactory->create();
    }

    /**
     * @return AddedProductModalControl
     */
    protected function createComponentAddedProductModal() {
        return $this->addedProductModal->create();
    }

    /**
     * @return \App\FrontModule\Components\PageMenuControl
     */
    protected function createComponentPageMenu() {
        return $this->pageMenuFactory->create();
    }

}
