<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\FrontModule\Presenters;

use App\Components\PersonFormControl;
use App\EshopModule\Model\OrderManager;
use App\EshopModule\Model\ProductAttributeManager;
use App\EshopModule\Model\ProductManager;
use App\EshopModule\Model\DiscountLevelsManager;
use App\Components\IPersonFormFactory;
use App\FrontModule\Components\IOrderMethodsFormFactory;
use App\FrontModule\Components\ICartManagementFormFactory;
use App\Services\OrderService;
use App\Services\OrderEmailService;
use App\Model\Enum\MessageType;
use Nette\Bridges\ApplicationLatte\Template;
use Nette\InvalidStateException;
use Nette\Mail\SendException;
use Nette\Utils\ArrayHash;

/**
 * @resourceDescription Zpracovává požadavky na objednávku ve frontendové části.
 * Parametry ($email a $emailMessage) a jejich settery se budou používat pro inicializaci těchto parametrů s pomocí DI 
 * a budou se do nich dosazovat hodnoty z konfiguračního souboru. 
 * DI s využitím setterů se používá pro inicializaci hodnot, na kterých není daná část aplikace existenčně závislá.
 *
 * @package App\FrontModule\Presenters
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class OrderPresenter extends OrderStatePresenter {

    /** URL stránky z ROUTERU, který se na ni odkazuje a bude použita pro vyhledání stránky a nastavení SEO elementů */
    const PAGE_CART_URL = 'kosik';
    const PAGE_PAYMENT_URL = 'kosik/payment';
    const PAGE_SUMMARY_URL = 'kosik/summary';
    const PAGE_THANKS_URL = 'kosik/thanks';

    /** @var OrderManager */
    private $orderManager;

    /** @var ProductManager */
    private $productManager;

    /** @var ProductAttributeManager */
    private $productAttributeManager;

    /** @var DiscountLevelsManager */
    private $discountLevels;

    /** @var IPersonFormFactory */
    private $personFormFactory;

    /** @var IOrderMethodsFormFactory */
    private $orderMethodsFormFactory;

    /** @var ICartManagementFormFactory */
    private $cartManagementForm;

    /** @var OrderService Služba pro OrderPresentery, která poskytne společné metody pro naplnění šablon */
    private $orderService;
    
    /** @var OrderEmailService Služba pro OrderPresentery, která slouží pro odesílání e-mailů s informacemi o objednávce */
    private $orderEmailService;

    /**
     * OrderPresenter constructor.
     * @param OrderManager $orderManager
     * @param ProductManager $productManager
     * @param ProductAttributeManager $productAttributeManager
     * @param DiscountLevelsManager $discountLevels
     * @param IPersonFormFactory $personFormFactory
     * @param IOrderMethodsFormFactory $orderMethodsFormFactory
     * @param ICartManagementFormFactory $cartManagementForm
     * @param OrderService $orderService
     * @param OrderEmailService $orderEmailService 
     */
    public function __construct(OrderManager $orderManager, ProductManager $productManager, ProductAttributeManager $productAttributeManager, DiscountLevelsManager $discountLevels, IPersonFormFactory $personFormFactory, IOrderMethodsFormFactory $orderMethodsFormFactory, ICartManagementFormFactory $cartManagementForm, OrderService $orderService, OrderEmailService $orderEmailService) {
        parent::__construct();
        $this->orderManager = $orderManager;
        $this->productManager = $productManager;
        $this->productAttributeManager = $productAttributeManager;
        $this->discountLevels = $discountLevels;
        $this->personFormFactory = $personFormFactory;
        $this->orderMethodsFormFactory = $orderMethodsFormFactory;
        $this->cartManagementForm = $cartManagementForm;
        $this->orderService = $orderService;
        $this->orderEmailService = $orderEmailService;
    }

    /**
     * Vytváří a vrací komponentu formuláře pro správu osoby.
     * @return PersonFormControl formulář pro správu osoby
     */
    protected function createComponentPersonForm() {
        $personForm = $this->personFormFactory->create();
        $personForm->formPath = self::FORM_PATH;
        return $personForm;
    }

    /**
     * Vytváří a vrací komponentu pro nastavení dopravy a platby do objednávky
     * @return OrderMethodsFormControl komponenta pro vykreslení formuláře
     */
    protected function createComponentOrderMethodsForm() {
        return $this->orderMethodsFormFactory->create();
    }

    /**
     * Vytváří a vrací komponentu pro práci s produkty v košíku - kupón a přidávání/odebírání produktů do košíku
     * @return CartManagementFormFactory komponenta pro vykreslení formuláře
     */
    protected function createComponentCartManagementForm() {
        return $this->cartManagementForm->create();
    }

    /**
     * @actionName removeFromCart
     * @actionDescription Akce pro odstranění produktu z košíku
     * @param int $productId id produktu
     */
    public function actionRemoveFromCart($productId) {
        $this->orderManager->updateCart([$productId => 0]);
        $this->redirect('default');
    }

    /**
     * @actionName default
     * @actionDescription Výpis obsahu zákaznického košíku
     */
    public function renderDefault() {
        // Nastavím do šablony SEO elementy ze stránky z DB, kterou najdu podle URL, nebo výchozí, které do metody zadávám
        $this->setPageSeoElements(self::PAGE_CART_URL, 'Košík | Objednávka', 'Přehled uživatelské objednávky.', 'košík, objednávka, zboží, produkty');

        // Předání parametrů do šablony
        $this->template->products = $this->orderManager->getProducts();
        $this->template->order = $this->orderManager->getOrder($this->orderManager->getOrderId(false));
        $this->template->couponDetail = $this->discountLevels->getDiscount($this->orderManager->getOrderCoupon($this->orderManager->getOrderId(false)));
        // Předání atributů do šablony
        $attributes = [];
        foreach ($this->template->products as $product) {
            $attributes[$product->product_id] = $this->productAttributeManager->getProductAttributes($product->product_id);
        }
        $this->template->attributes = $attributes;

        $this->template->upsellProduct = $this->productManager->getProducts(['category_id' => 20])->fetch();

        if ($this->template->upsellProduct) {
            $this->template->upsellVariants = $this->productManager->getVariants($this->template->upsellProduct->product_id)->order(ProductManager::COLUMN_PRICE, 'DESC')->fetchAll();
        } else {
            $this->template->upsellVariants = null;
        }
    }

    /**
     * @actionName payment
     * @actionDescription Nastavení způsobu dopravy a platby v košíku - proces dokončení objednávky
     */
    public function renderPayment() {
        // Nastavím do šablony SEO elementy ze stránky z DB, kterou najdu podle URL, nebo výchozí, které do metody zadávám
        $this->setPageSeoElements(self::PAGE_PAYMENT_URL, 'Platba a doprava | Košík', 'Výběr dopravy a platby pro objednávku.', 'košík, objednávka, zboží, produkty, doprava, platba, metody');
    }

    /**
     * @actionName summary
     * @actionDescription Rekapitulace objednávky
     */
    public function renderSummary() {
        // Nastavím do šablony SEO elementy ze stránky z DB, kterou najdu podle URL, nebo výchozí, které do metody zadávám
        $this->setPageSeoElements(self::PAGE_SUMMARY_URL, 'Souhrn objednávky | Košík', 'Celkový souhrn uživatelské objednávky.', 'košík, objednávka, zboží, produkty, souhrn, přehled');

        /** @var Template $template */
        $template = $this->getTemplate();
        $this->orderService->fillTemplateWithOrderData($template, null, $this);
    }

    /**
     * @actionName thanks
     * @actionDescription Vykreslenení děkovné stránky po dokončení objednávky, které je tohoto procesu součást
     * Měla by být dotupná po dokončení objednávky, jinak ne.
     * Parametr id objednávky se předa a získa přes GET, otestuji, jeslti to nějakou objednávku najde a testuji její stav
     * Na stránku se dostanu jen pokud je stav objednávky vytvořeno - ten stav se bude rychle měnit v administraci
     * Stránka se stane dostupnout jen na chvíli
     * @param int $order_id ID objednávky z GET
     */
    public function renderThanks($order_id = null) {
        // Nastavím do šablony SEO elementy ze stránky z DB, kterou najdu podle URL, nebo výchozí, které do metody zadávám
        $this->setPageSeoElements(self::PAGE_THANKS_URL, 'Děkujeme | Košík', 'Děkovná stránka za provedení objednávky.', 'košík, objednávka, zboží, produkty');

        $orderId = $this->getParameter('order_id');

        if ($orderId) {
            $order = $this->orderManager->getOrder($orderId);
            if ($order && $order[OrderManager::COLUMN_STATE] == OrderManager::STATE_COMPLETED) {
                $this->template->lastOrder = ArrayHash::from([
                            'order' => $order,
                            'products' => $this->orderManager->getProducts($orderId),
                            'summaryWithShipping' => $this->orderManager->getOrderSummary($orderId, false),
                            'summary' => $this->orderManager->getOrderSummary($orderId, true)
                ]);
            } else {
                $this->flashMessage('Nelze zobrazit děkovnou stránku', MessageType::ERROR);
                $this->redirect(':Front:Page:');
            }
        } else {
            $this->flashMessage('Nelze zobrazit děkovnou stránku', MessageType::ERROR);
            $this->redirect(':Front:Page:');
        }
    }

    /**
     * @actionName setProductQuantity
     * @actionDescription Nastavuje množství produktu v košíku - sníží, zvýší, odebere
     * @param $productId
     * @param $quantity
     */
    public function handleSetProductQuantity($productId, $quantity) {
        $this->orderManager->updateCart([$productId => $quantity]);
        $this->flashMessage('Obsah košíku byl změněn', MessageType::SUCCESS);
        $this->redrawControl();
    }

    /**
     * @actionName buy
     * @actionDescription Potvrzení objednávky - slouží pro odeslání emailu pro shrnutí objednávky a potvrzení.
     */
    public function handleBuy() {
        try {
            // Zkusím použít kupón - přičtu u kupónu jedničku do počtu použití
            if ($couponId = $this->orderManager->getOrderCoupon($this->orderManager->getOrderId(false))) {
                $this->discountLevels->includeCouponUse($couponId);
            }

            // Pokud jsem použil kupón, můžu zkusit začít posílat mail a dokončit objednávku
            try {
                // Dokončení zkontrolované objednávky.
                $orderId = $this->orderManager->getOrderId(false);
                $this->orderManager->completeOrder();
                // Odeslání emailu zákazníkovi s rekapitulací - předám mu zprávu, šablonu, presenter a orderID
                $order = $this->orderEmailService->sendOrderEmail('eshop.order.changeStateAccepted', $this->createTemplate(), $this, $orderId);
                // Odeslání e-mailu o nové objednávce kompetentní osobě vybrané z administrace
                $this->orderEmailService->sendEmailToOrderEmail();

                $this->flashMessage('Děkujeme za Váš nákup, na email Vám bylo odesláno potvrzení objednávky.');
                $this->redirect(':Front:Order:thanks', $order->e_order_id);
            } catch (SendException $e) {
                $this->flashMessage($e->getMessage(), MessageType::ERROR);
            }
            $this->redirect(':Front:Order:summary');
        } catch (InvalidStateException $e) {
            $this->flashMessage($e->getMessage(), MessageType::ERROR);

            if ($this->isAjax()) {
                //$this->redirect(':Front:Order:summary');
                $this->redrawControl('ajaxNotification');
            }
        }
    }

    /**
     * @actionName unsetCoupon
     * @actionDescription Nastavím u objednávky hodnotu v coupon_id na null
     * @param int $couponId id slevového kupónu
     */
    public function handleUnsetCoupon($couponId) {
        // Odebrání kupónu
        $this->orderManager->unsetCouponDiscount($couponId);

        // Info o zprávě
        $this->flashMessage('Kupón byl odebrán z objednávky', MessageType::INFO);

        // Překrelsení komponent
        $this->redrawControl('ajaxNotification');
        // Ajaxové zpracování - překreslení snippetů košíku v headeru
        $this->presenter->redrawControl("basketSnippet");
        $this->redrawControl('cartFormSnippet');
    }

}
