<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\FrontModule\Presenters;

use App\Components\IPersonFormFactory;
use App\Components\PersonFormControl;
use App\FrontModule\Components\IChangePasswordFormFactory;
use App\Model\Enum\MessageType;
use Nette\Forms\Form;
use Nette\InvalidArgumentException;
use Nette\InvalidStateException;
use Nette\Mail\Message;
use Nette\Mail\SendException;
use Nette\Mail\SendmailMailer;
use Nette\Utils\ArrayHash;
use App\EshopModule\Model\OrderManager;

/**
 * @resourceDescription Zpracovává požadavky na osoby ve frontedové části.
 * @package App\FrontModule\Presenters
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class PersonPresenter extends OrderStatePresenter {

    /** URL stránky z ROUTERU, který se na ni odkazuje a bude použita pro vyhledání stránky a nastavení SEO elementů */
    const
            PAGE_REGISTRATION_URL = 'osoby/registrace-uctu',
            PAGE_ORDER_REGISTRATION_URL = 'osoby/registrace',
            PAGE_PASSWORD_CHANGE_URL = 'editace-hesla',
            PAGE_MANAGE_URL = 'osoby/sprava',
            PAGE_MANAGE_IN_ORDER_URL = 'osoby/dodaci-udaje';

    /** Cesta k šabloně registračních formulářů - je jiné než klasická cesta poděděná z BasePresenteru. */
    const PERSON_FORM_PATH = __DIR__ . '/../templates/components/PersonForm/form.latte';
    const REGISTER_FORM_PATH = __DIR__ . '/../templates/components/PersonForm/register.latte';

    /**
     * @var OrderManager
     */
    private $orderManager;

    /**
     * @var IPersonFormFactory Instance třídy továrničky na tvorbu formulářů pro správu osobami.
     */
    private $personFormFactory;

    /**
     * @var IChangePasswordFormFactory Instance třídy továrničky na tvorbu formulářů pro změnu hesla
     */
    private $changePasswordFormControl;

    /**
     * PersonPresenter constructor.
     * @param OrderManager $orderManager
     * @param IPersonFormFactory $personFormFactory
     * @param IChangePasswordFormFactory $changePasswordFormControl
     */
    public function __construct(OrderManager $orderManager, IPersonFormFactory $personFormFactory, IChangePasswordFormFactory $changePasswordFormControl) {
        parent::__construct();

        $this->orderManager = $orderManager;
        $this->personFormFactory = $personFormFactory;
        $this->changePasswordFormControl = $changePasswordFormControl;
    }

    /**
     * @actionName register
     * @actionDescription Registrace nové osoby během dokončení objednávky.
     * Metodu se volá ve chvíli, kdy neregistrovaný uživatel dorazí do fáze objednávky, kdy má zadat své kontaktní údaje.
     * Tehdy mu bude nabídnuta možnost vytvořit uživatelský účet, ale využít ji nemusí.
     * Nejedná se o formulář pro administrátora a měl by obsahovat antispam.
     * I když se uživatel nebude registrovat - jednorázový nákup -> stejně se jeho údaje uloži do tabulky PERSON a do dalších přidružených
     */
    public function actionRegister() {
        if ($this->getUser()->isLoggedIn()) {
            $this->flashMessage('Uživatel nemůže být přihlášen a zároveň vidět formulář s povinnou registrací.', MessageType::ERROR);
            $this->redirect('Person:manage');
        }

        // Doplní se vlastní zpracování k úspěšnému odeslání formuláře pro správu osob.
        $this['personForm']->onSuccess[] = function (Form $form, ArrayHash $values) {
            if (isset($values->user_id)) {
                $this->flashMessage('Byl jste úspěšně registrován.', MessageType::SUCCESS);
            }

            try {
                $this->orderManager->setPerson($values->person_id);

                // Odeslání emailu novému uživateli
                $this->sendEmailToNewUser($values);
            } catch (InvalidArgumentException $ex) {
                $this->flashMessage('Nepodařilo se vytvořit osobu se zadanými údaji.', MessageType::ERROR);
            }

            $this->redirect(':Front:Order:payment');
        };
    }

    /**
     * @actionName registerAccount
     * @actionDescription Registrace nového uživatelského účtu spolu s osobou - registrace není nijak vázáná na objednávku = obyč registrace. 
     */
    public function actionRegisterAccount() {
        if ($this->getUser()->isLoggedIn()) {
            $this->flashMessage('Uživatel nemůže být přihlášen a zároveň vidět formulář s povinnou registrací.', MessageType::ERROR);
            $this->redirect('Person:manage');
        }

        $this['personForm']->formPath = self::REGISTER_FORM_PATH;

        // Doplní se vlastní zpracování k úspěšnému odeslání formuláře pro správu osob.
        $this['personForm']->onSuccess[] = function (Form $form, ArrayHash $values) {
            if (isset($values->user_id) && isset($values->person_id)) {
                // Odeslání emailu novému uživateli
                $this->sendEmailToNewUser($values);

                $this->flashMessage('Byl jste úspěšně registrován, přihlaste se níže.', MessageType::SUCCESS);
                $this->redirect($this->loginPresenter);
            } else {
                $this->flashMessage('Nepodařilo se zaregistrovat nový účet.', MessageType::ERROR);
                $this->redirect('this');
            }
        };
    }

    /**
     * @actionName manageInOrder
     * @actionDescription Editace dat osoby uživatelem - uživatel edituje sám sebe, jako druhy krok objednavky
     */
    public function actionManageInOrder()
    {
        // Doplníme vlastní zpracování k úspěšnému odeslání formuláře pro správu osob.
        $this['personForm']->onSuccess[] = function (Form $form, ArrayHash $values) {
            $this->redirect(':Front:Order:payment');
        };
    }

    /**
     * @actionName manage
     * @actionDescription Editace dat osoby uživatelem - uživatel edituje sám sebe 
     */
    public function actionManage() {
        // Doplníme vlastní zpracování k úspěšnému odeslání formuláře pro správu osob.
        $this['personForm']->onSuccess[] = function (Form $form, ArrayHash $values) {
            if (isset($values->person_id)) {
                $this->flashMessage('Osoba byla úspěšně uložena.', MessageType::SUCCESS);
                $this->redirect('this');
            } else {
                $this->flashMessage('Nepodařilo se uložit změny osoby.', MessageType::ERROR);
                $this->redirect('this');
            }
        };
    }

    /**
     * @actionName changePassword
     * @actionDescription Změna hesla přihlášené osoby
     */
    public function renderChangePassword() {
        // Nastavím do šablony SEO elementy ze stránky z DB, kterou najdu podle URL, nebo výchozí, které do metody zadávám
        $this->setPageSeoElements(self::PAGE_PASSWORD_CHANGE_URL, 'Změna hesla', 'Stránka pro změnu uživatelského hesla', 'heslo, změna, uživatel');
    }

    /** Registrace nové osoby během dokončení objednávky. */
    public function renderRegister() {
        $this->template->loginPresenter = $this->loginPresenter;
        // Nastavím do šablony SEO elementy ze stránky z DB, kterou najdu podle URL, nebo výchozí, které do metody zadávám
        $this->setPageSeoElements(self::PAGE_ORDER_REGISTRATION_URL, 'Registrace | Objednávka', 'Stránka pro registraci do stránky', 'registrace, účet');
    }

    /** Registrace nové osoby, nesouvisi s dokončení objednávky. */
    public function renderRegisterAccount() {
        // Nastavím do šablony SEO elementy ze stránky z DB, kterou najdu podle URL, nebo výchozí, které do metody zadávám
        $this->setPageSeoElements(self::PAGE_REGISTRATION_URL, 'Registrace', 'Stránka pro registraci na stránku', 'registrace, účet');
    }

    public function renderManageInOrder()
    {
        // Nastavím do šablony SEO elementy ze stránky z DB, kterou najdu podle URL, nebo výchozí, které do metody zadávám
        $this->setPageSeoElements(self::PAGE_MANAGE_IN_ORDER_URL, 'Úprava dodacích údajů', 'Stránka pro editování uživatelských údajů', 'osoba, změna, uživatelské údaje');
    }

    /** Správa uživatelských údajů. */
    public function renderManage() {
        // Nastavím do šablony SEO elementy ze stránky z DB, kterou najdu podle URL, nebo výchozí, které do metody zadávám
        $this->setPageSeoElements(self::PAGE_MANAGE_URL, 'Správa osoby', 'Stránka pro editování uživatelských údajů', 'osoba, změna, uživatelské údaje');
    }

    /**
     * Vytváří a vrací komponentu formuláře pro správu osoby.
     * @return PersonFormControl formulář pro správu osoby
     */
    protected function createComponentPersonForm() {
        $personForm = $this->personFormFactory->create();
        $personForm->formPath = self::PERSON_FORM_PATH;

        return $personForm;
    }

    /**
     * @return \App\FrontModule\Components\ChangePasswordFormControl
     */
    protected function createComponentChangePasswordForm() {
        return $this->changePasswordFormControl->create();
    }

    private function sendEmailToNewUser($values) {
        try {
            $mail = new Message;
            $mail->setFrom('info@ideatech.cz')
                    ->addTo($values->person_detail['email'])
                    ->setSubject('Registrace na webu ....')
                    ->setBody("Děkujeme Vám za Vaši registraci. Vaše přihlašovací jméno je: " . $values->person_detail['email'] . "Přihlášení zde: ...");

            $mailer = new SendmailMailer;
            $mailer->send($mail);
        } catch (InvalidStateException $ex) {
            throw new SendException('Bohužel se nepodařilo odeslat potvrzovací email.', $e->getCode(), $e);
        }
    }

}
