<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\FrontModule\Presenters;

use App\EshopModule\Model\CategoryManager;
use App\EshopModule\Model\ProductManager;
use App\CmsModule\Model\PageManager;
use App\FrontModule\Components\EshopCategoryMenuControl;
use App\FrontModule\Components\IPaginationControlFactory;
use App\FrontModule\Components\IProductBlockFactory;
use App\FrontModule\Components\IProductDetailFactory;
use App\FrontModule\Components\PaginationControl;
use App\FrontModule\Components\IFilterFormFactory;
use App\Model\Enum\MessageType;
use Nette\Application\BadRequestException;
use Nette\Application\UI\Multiplier;
use Nette\Database\Table\IRow;

/**
 * @resourceDescription Zpracováná požadavky na produkty ve frontendové části.
 * @package App\FrontModule\Presenters
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class ProductPresenter extends BaseFrontPresenter {

    /** URL stránky z ROUTERU, který se na ni odkazuje a bude použita pro vyhledání stránky a nastavení SEO elementů */
    const PAGE_URL = 'produkty';

    /**
     * @var ProductManager
     */
    private $productManager;

    /**
     * @var CategoryManager
     */
    private $categoryManager;
    
    /**
     * @var PageManager
     */
    private $pageManager;

    /**
     * @var IPaginationControlFactory
     */
    private $paginationControlFactory;

    /**
     * @var IFilterFormFactory
     */
    private $filterFormFactory;

    /**
     * @var IProductDetailFactory
     */
    private $productDetailFactory;

    /**
     * @var IProductBlockFactory
     */
    private $productBlockFactory;

    /**
     * @var IRow
     */
    private $product;

    const CATEGORY_ACTION_SETS = 'akcni-sety';

    /**
     * ProductPresenter constructor.
     * @param ProductManager $productManager
     * @param CategoryManager $categoryManager
     * @param PageManager $pageManager
     * @param IPaginationControlFactory $paginationControlFactory
     * @param IFilterFormFactory $filterFormFactory
     * @param IProductDetailFactory $productDetailFactory
     * @param IProductBlockFactory $productBlockFactory
     */
    public function __construct(ProductManager $productManager, CategoryManager $categoryManager, PageManager $pageManager, IPaginationControlFactory $paginationControlFactory, IFilterFormFactory $filterFormFactory, IProductDetailFactory $productDetailFactory, IProductBlockFactory $productBlockFactory) {
        parent::__construct();
        $this->productManager = $productManager;
        $this->categoryManager = $categoryManager;
        $this->pageManager = $pageManager;
        $this->paginationControlFactory = $paginationControlFactory;
        $this->filterFormFactory = $filterFormFactory;
        $this->productDetailFactory = $productDetailFactory;
        $this->productBlockFactory = $productBlockFactory;
    }

    /**
     * @actionName default
     * @actionDescription Výpis produktů na webu.
     * @param null|string $url URL kategorie, do které mají produkty spadat nebo null pro všechny produkty
     */
    public function actionDefault($url) {
        if ($url == self::CATEGORY_ACTION_SETS) {
            if (!$this->user->isLoggedIn()) {
                $this->flashMessage('Pro nákup akčních setů musíte být přihlášen/a', MessageType::INFO);
                $this->redirect($this->loginPresenter);
            }
        }

        // Spojení formuláře pro filtrování s vyhledáváním.
        // Předáváme do skrytého pole formuláře pro filtrování vyhledávací frázi
        // Umožní filtrovat produkty, které jsme předtím vyhledali pomocí této fráze.
        $this['filterForm']->setDefaults(['phrase' => $this->getParameter('phrase')]);
    }

    /**
     * Výpis produktů.
     * Tvar URL: phrase=&order_by=lowest_price&start_price=100&end_price=999&filter=Filtrovat&do=filterForm-submit
     * @param null|string $url URL kategorie, do které mají produkty spadat nebo null pro všechny produkty
     */
    public function renderDefault($url) {
        // Najdu si stránku aktuality
        $page = $this->pageManager->findPageByUrlString(self::PAGE_URL);

        // Nastavím do šablony SEO elementy ze stránky z DB, kterou najdu podle URL, nebo výchozí, které do metody zadávám
        $this->setPageSeoElements(self::PAGE_URL, 'Produkty', 'Stránka s produkty z E-shopu', 'produkty, e-shop');

        // Získání parametrů dotazu.
        $parameters = $this->getParameters();

        // Pokusí se získat kategorii podle jejího URL, pokud bylo zadáno.
        if ($url && ($category = $this->categoryManager->getCategory($url))) {
            // Přidá ID kategorie do parametrů
            $parameters['category_id'] = $category->category_id;
        } elseif ($url) {
            // Pokud byla zadána URL adresa kategorie, která ale neexistuje.
            // Oznámí to uživateli.
            $this->flashMessage('Zvolená kategorie neexistuje.');
            // Přesměruje na výpis všech produktů bez kategorie.
            $this->redirect(':Front:Product:');
        }

        // Získání vyfiltrovaných produktů.
        $products = $this->productManager->getProducts($parameters);

        // Nastavení stránkování.
        // Nezapomeňte předat referencí!
        $paginator = &$this['pagination']->getPaginator();
        $paginator->setItemCount(count($products));
        // Pokud parametr PAGE nenajde, vrací 1
        $paginator->setPage($this->getParameter('page', 1));

        // Omezení počtu vypisovaných produktů podle aktuálního stránkování - získám přesně ty produkty na konkrétní stránku.
        $products->limit($paginator->getLength(), $paginator->getOffset());

        // Předání proměnných do šablony.
        $this->template->title = isset($category) ? $category->title : 'Produkty';
        $this->template->products = $products;
    }

    /**
     * @actionName news
     * @actionDescription Výpis novinek - produktů na webu
     */
    public function renderNews() {
        // Získání vyfiltrovaných produktů.
        $products = $this->productManager->getProducts(['new_product' => 1]);

        // Předání proměnných do šablony.
        $this->template->title = isset($category) ? $category->title : 'Produkty';
        $this->template->products = $products;
    }

    /**
     * @actionName detail
     * @actionDescription Detail produktu.
     * @param string $url URL produktu
     * @throws BadRequestException Jestliže produkt s danou URL nebyl nalezen.
     */
    public function actionDetail($url) {
        // Pokusí se načíst produkt s danou URL a pokud nebude nalezen, vyhodí chybu 404.
        if (!($product = $this->productManager->getProduct($url))) {
            throw new BadRequestException();
        } else {
            $this->product = $product;
            $this->template->product = $product;
        }
    }

    protected function createComponentProductDetail() {
        return $this->productDetailFactory->create($this->product);
    }

    protected function createComponentProductBlock() {
        return new Multiplier(function ($itemId) {
            $productBlock = $this->productBlockFactory->create($this->productManager->getProductFromId($itemId));
            return $productBlock;
        });
    }

    /**
     * Vytváří a vrací komponentu pro filtrování produktů.
     * @return \App\FrontModule\Components\FilterFormControl
     */
    protected function createComponentFilterForm() {
        return $this->filterFormFactory->create();
    }

    /**
     * Vytváří a vrací komponentu pro správu a vykreslování stránkování.
     * @return PaginationControl komponenta pro správu a vykreslování stránkování
     */
    protected function createComponentPagination() {
        return $this->paginationControlFactory->create();
    }

    /**
     * @return \App\FrontModule\Components\EshopCategoryMenuControl
     */
    protected function createComponentCategoryMenu() {
        return new EshopCategoryMenuControl();
    }

}
