<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\FrontModule\Presenters;

use App\Forms\UserForms;
use App\Model\Enum\MessageType;
use Nette\Application\UI\Form;
use Nette\Utils\ArrayHash;
use App\Model\UserLogManager;

/**
 * @resourceDescription Zpracovává vykreslování přihlašování a registrace - jak pro frontendovou část, tak i přihlášeni do adminu
 * @package App\FrontModule\Presenters
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class SignPresenter extends BaseFrontPresenter {

    /** URL stránky z ROUTERU, který se na ni odkazuje a bude použita pro vyhledání stránky a nastavení SEO elementů */
    const PAGE_URL = 'prihlaseni';
    
    /** @var UserLogManager Instance třídy modelu pro ukládání přihlašovacích logů do db. */
    private $userLogManager;
    
    /** @var UserForms Továrnička na uživatelské formuláře. */
    private $userFormsFactory;

    /** @var array Společné instrukce pro přihlašovací a registrační formuláře. */
    private $instructions;

    /**
     * Konstruktor s injektovanou továrničkou na uživatelské formuláře.
     * @param UserForms $userForms automaticky injektovaná třída továrničky na uživatelské formuláře
     * @param UserLogManager $userLogManager Instace modelu pro ukládání uživatelských logů
     */
    public function __construct(UserForms $userForms, UserLogManager $userLogManager) {
        parent::__construct();
        $this->userFormsFactory = $userForms;
        $this->userLogManager = $userLogManager;
    }

    /**
     * Volá se před každou akcí presenteru a inicializuje společné proměnné. 
     */
    public function startup() {
        parent::startup();
        $this->instructions = array(
            'message' => null
        );
    }

    /**
     * @actionName login
     * @actionDescription Login a přesměrování do administrace, pokud je uživatel již přihlášen. 
     * Speciální layout pro přihlašování
     */
    public function actionLogin() {
        $this->setLayout('layoutLogin');

        // Pokud přijdu jako přihlášeny uživatel s přístupem do adminu - redirect do adminu ihned
        if ($this->getUser()->isLoggedIn() && $this->getUser()->isAllowed('Core:Administration', 'default')) {
            $this->redirect(':Core:Administration:default');
        }

        // Doplní se vlastní zpracování k úspěšnému odeslání formuláře pro přihlášení.
        $this['loginForm']->onSuccess[] = function (Form $form, ArrayHash $values) {
            // Kontrola, jestli se uživatel úspěšně přihlásil a má oprávnění
            if ($this->getUser()->isLoggedIn() && $this->getUser()->isAllowed('Core:Administration', 'default')) {
                // Pokud se úspěšně přihlásil do amdinu a má oprávnění - zaeviduji jeho login do DB
                $this->userLogManager->saveValues(UserLogManager::TABLE_NAME, UserLogManager::COLUMN_ID, array(
                    UserLogManager::COLUMN_USER => $this->getUser()->getId()
                ));
            }

            // Na závěr redirect po provedení loginu, nemusím kontrolovat oprávnění - přesměruji, i když je nemá, ale lognul se
            // Hodí mu to info, že se přihlásil, ale do amdinu ho to nepustí, protože nemá oprávnění
            $this->redirect(':Core:Administration:default');
        };
    }

    /**
     * @actionName frontLogin
     * @actionDescription Login a přesměrování na frontu (webu), pokud je uživatel již přihlášen. 
     */
    public function actionFrontLogin() {
        // Nastavím do šablony SEO elementy ze stránky z DB, kterou najdu podle URL, nebo výchozí, které do metody zadávám
        $this->setPageSeoElements(self::PAGE_URL, 'Přihlášení', 'Stránka pro uživatelské přihlášení', 'login, přihlášení, učet');

        $this->instructions = array(
            'redirection' => ':Front:Page:default'
        );
        if ($this->getUser()->isLoggedIn()) {
            $this->redirect(':Front:Page:default');
        }
    }

    /**
     *  Vykreslení front přihlašování s klasickým layoutem. 
     */
    public function renderFrontLogin() {
        $this->template->title = 'Přihlášení';
    }

    /**
     * @actionName logout
     * @actionDescription Odhlášení uživatele. 
     */
    public function actionLogout() {
        $this->getUser()->logout();
        $this->flashMessage('Byl jste úspěšně odhlášen.', MessageType::SUCCESS);
        $this->redirect($this->loginPresenter);
    }

    /**
     * Vrací komponentu přihlašovacího formuláře z továrničky.
     * @return Form přihlašovací formulář
     */
    protected function createComponentLoginForm() {
        $this->instructions['message'] = 'Byl jste úspěšně přihlášen.';
        $this->instructions['msg_type'] = MessageType::SUCCESS;
        return $this->userFormsFactory->createLoginForm(ArrayHash::from($this->instructions));
    }

}
