<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\Presenters;

use App\Components\IAdminMenuFactory;
use App\Components\IPriceControlFactory;
use App\Components\PriceControl;
use App\FrontModule\Components\EshopCategoryMenuControl;
use App\EshopModule\Model\SettingsManager;
use App\EshopModule\Model\CurrencyManager;
use App\CoreModule\Model\ModuleManager;
use App\Forms\BaseFormFactory;
use App\MediaModule\Components\IMainImageFactory;
use App\Model\Enum\MessageType;
use App\Model\UserManager;
use App\Model\UserLogManager;
use Kdyby\Translation\Translator;
use Nette\Application\BadRequestException;
use Nette\Application\UI\Presenter;

/**
 * Výchozí presenter, od kterého všechni ostatní budou dědit.
 * @package App\Presenters
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
abstract class BasePresenter extends Presenter {

    /** Cesta ke globální šabloně formulářů. */
    const FORM_PATH = __DIR__ . '/../templates/components/form.latte';

    /** Cesta ke globální šabloně formulářů. */
    const FORM_BOOTSTRAP_PATH = __DIR__ . '/../templates/components/bootstrap-form.latte';

    /** @var null|string Adresa presenteru pro logování uživatele. */
    protected $loginPresenter = null;

    /** @var BaseFormFactory Továrnička na formuláře. */
    protected $formFactory;

    /** @var IPriceControlFactory Instance třídy továrničky na tvorbu komponent pro vykreslování ceny. */
    private $priceControlFactory;

    /** @var IAdminMenuFactory Instance třídy továrničky na tvorbu komponent pro vykreslování admin menu. */
    private $adminMenuFactory;

    /** @var SettingsManager Instance třídy modelu pro práci s účetním nastavením. */
    private $settingsManager;

    /** @var CurrencyManager Instance třídy modelu pro práci s měnami. */
    private $currencyManager;

    /** @var UserManager Instance třídy modelu pro práci s uživateli. */
    private $userManager;

    /** @var UserLogManager Instance třídy modelu pro ukládání přihlašovacích logů do db. */
    private $userLogManager;

    /** @var ModuleManager Instance třídy modelu pro práci s moduly. */
    private $moduleManager;

    /** @var IMainImageFactory */
    private $mainImageFactory;

    /**
     * @var Translator
     * @inject
     */
    public $translator;

    /** @persistent */
    public $locale = 'cs';

    public function injectManagerDependencies(SettingsManager $settingsManager, CurrencyManager $currencyManager, UserManager $userManager, UserLogManager $userLogManager, ModuleManager $moduleManager) {
        $this->settingsManager = $settingsManager;
        $this->currencyManager = $currencyManager;
        $this->userManager = $userManager;
        $this->userLogManager = $userLogManager;
        $this->moduleManager = $moduleManager;
    }

    public function injectMainImageFactory(IMainImageFactory $mainImageFactory) {
        $this->mainImageFactory = $mainImageFactory;
    }

    /**
     * Speciální setter pro injectování továrničky na formuláře se společným nastavením.
     * Dostanu do vsech presenteru, ktere dedi od tohoto presenteru BaseFormFactory, ktery budou moct vyuzivat
     * Vyhoda tohoto zpusobu predani pred klasickym DI = nemusi pri volani rodicovskeho construktoru (tohoto) predavat promenou => setri kod
     * @param BaseFormFactory $baseFormFactory automaticky injektovaná továrnička na formuláře se společným nastavením
     */
    public function injectFormFactory(BaseFormFactory $baseFormFactory) {
        $this->formFactory = $baseFormFactory;
    }

    /**
     * Speciální setter pro injectování továrničky na tvorbu komponent pro vykreslování ceny.
     * @param IPriceControlFactory $priceControlFactory automaticky injectovaná továrnička na tvorbu komponent pro vykreslování ceny
     */
    public function injectPriceControlFactory(IPriceControlFactory $priceControlFactory) {
        $this->priceControlFactory = $priceControlFactory;
    }

    /**
     * Speciální setter pro injectování továrničky na tvorbu komponent pro vykreslování admin menu
     * @param IAdminMenuFactory $adminMenuFactory automaticky injectovaná továrnička na tvorbu komponent pro vykreslování admin menu
     */
    public function injectAdminMenuControlFactory(IAdminMenuFactory $adminMenuFactory) {
        $this->adminMenuFactory = $adminMenuFactory;
    }

    /**
     * Volá se na začátku každé akce a kontroluje uživatelská oprávnění k této akci a zda je její modul aktivní.
     * @throws BadRequestException Jestliže je uživatel přihlášen, ale nemá oprávnění k této akci.
     */
    protected function startup() {
        parent::startup();

        if (!$this->getUser()->isAllowed($this->getName(), $this->getAction())) {
            $this->flashMessage($this->translator->translate('core.common.noPermission'), MessageType::ERROR);
            if ($this->loginPresenter) {
                $this->redirect($this->loginPresenter);
            }
        } else if (!$this->moduleManager->checkModulActivity(strstr($this->getName(), ':', true))) {
            $this->flashMessage($this->translator->translate('core.common.noModulActivity'), MessageType::ERROR);
            if ($this->loginPresenter) {
                $this->redirect($this->loginPresenter);
            }
        }
    }

    /** Volá se před vykreslením každého presenteru a předává společné proměné do celkového layoutu webu. */
    protected function beforeRender() {
        parent::beforeRender();

        // Test, jestli má uživatel admin oprávnění
        $this->template->admin = $this->getUser()->isInRole('admin') || $this->getUser()->isInRole('seperadmin');
        // Předá jméno domény do šablony.
        $this->template->domain = $this->getHttpRequest()->getUrl()->getHost();
        // Předá cestu ke globální šabloně formulářů do šablony.
        $this->template->formPath = self::FORM_PATH;
        // Výchozí nastavení
        $this->template->settings = $this->settingsManager->getSettings();

        // Předání měn a kurzů do šablony
        $this->template->defaultCurrency = $this->currencyManager->getDefaultCurrency()[CurrencyManager::COLUMN_ID]; // V adminu je potřeba defaultní měna mapříklad pro ceny doprav
    }

    /**
     * Pomocná funkce pro zaznamenávání uživatelských akcí
     * Předpokladem je přihlášený uživatel, jelikož se zaznamenávají akce jen v administraci
     * @param String $description Popisek zaznamenávané akce
     */
    public function logAction($description) {
        $values = array(
            UserLogManager::COLUMN_USER => $this->getUser()->getId(),
            UserLogManager::COLUMN_RESOURCE => $this->getPresenter()->name,
            UserLogManager::COLUMN_ACTIVITY => $this->getAction(),
            UserLogManager::COLUMN_DESCRIPTION => $description
        );
        $this->userLogManager->saveValues(UserLogManager::TABLE_ACTIVITY_NAME, UserLogManager::COLUMN_ACTIVITY_ID, $values);
    }

    /**
     * Vytváří komponentu, která sestavuje admin menu z menu.neon souborů v jednotlivých modulech
     * @return \App\Components\AdminMenuControl
     */
    protected function createComponentAdminMenu() {
        return $this->adminMenuFactory->create();
    }

    /**
     * Vytváří a vrací komponentu pro vykreslení ceny.
     * @return PriceControl komponenta pro vykreslení ceny
     */
    protected function createComponentPrice() {
        return $this->priceControlFactory->create();
    }

    /**
     * Vytváří komponentu, která se stará o rekurzivní vykreslení stromového navigačního menu kategorií.
     * @return EshopCategoryMenuControl komponenta menu kategorií
     */
    protected function createComponentMenu() {
        return new EshopCategoryMenuControl();
    }

    protected function createComponentMainImage() {
        return $this->mainImageFactory->create();
    }

}
