<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\Services;

use DateTime;
use Nette\Utils\Random;
use Nette\Database\Table\IRow;
use App\Model\PersonManager;
use Nette\Localization\ITranslator;

/**
 * Pomocná služba pro formátování data, jména adresy atd...
 * 
 * @package App\Services
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class FormatHelperService {

    /** var array České názvy měsíců */
    private static $months = array('ledna', 'února', 'března', 'dubna', 'května', 'června', 'července', 'srpna', 'září', 'října', 'listopadu', 'prosince');

    /** var array České zkratky měsíců */
    private static $chartMonths = array('Led.', 'Úno.', 'Bře.', 'Dub.', 'Kvě.', 'Čvn', 'Čvc.', 'Srp.', 'Zář.', 'Říj.', 'Lis.', 'Pros.');
    
    /**
     * @var ITranslator
     */
    private $translator;

    /**
     * FormatHelperService constructor.
     * @param ITranslator $translator
     */
    public function __construct(ITranslator $translator) {
        $this->translator = $translator;
    }

    /**
     * Formátuje adresu osoby.
     * @param IRow $address data adresy osoby
     * @return string formátovaná adresa
     */
    public function formatAddress(IRow $address) {
        $html = $address[PersonManager::COLUMN_ADDRESS_STREET] . ' ';

        if ($address[PersonManager::COLUMN_ADDRESS_REGISTRY_NUMBER] && $address[PersonManager::COLUMN_ADDRESS_HOUSE_NUMBER]) {
            $html .= $address[PersonManager::COLUMN_ADDRESS_REGISTRY_NUMBER] . '/' . $address[PersonManager::COLUMN_ADDRESS_HOUSE_NUMBER];
        } else {
            $html .= $address[PersonManager::COLUMN_ADDRESS_REGISTRY_NUMBER] ? $address[PersonManager::COLUMN_ADDRESS_REGISTRY_NUMBER] : $address[PersonManager::COLUMN_ADDRESS_HOUSE_NUMBER];
        }

        $html .= '<br />' . $address[PersonManager::COLUMN_ADDRESS_CITY] . '<br />' . $address[PersonManager::COLUMN_ADDRESS_ZIP];
        return $html;
    }

    /**
     * Formátuje jméno fyzické osoby, případně název firmy (právnické osoby).
     * @param IRow $personDetail data osoby
     * @return string formátované jméno osoby, případně název firmy (právnické osoby)
     */
    public function formatName(IRow $personDetail) {
        $name = PersonManager::getName($personDetail);
        return empty($name) ? $this->translator->translate('eshop.common.nameFormat') : $name;
    }

    /**
     * Formátuje datum do databázového formátu.
     * Konstaty a fce je stejná jak v edtiaci nastavení - duplicitní
     * Pro práci s daty lze využít etimepicker
     * @param string $date datum v textové podobě
     * @return bool|string formátované datum
     */
    public function formatDate($date) {
        return date('Y-m-d', strtotime(str_replace('.', '-', $date)));
    }

    /**
     * Zformátuje datum a čas z libovolné stringové podoby na tvar např. "Dnes 15:21".
     * @param string $date datum ke zformátování
     * @return string zformátované datum
     */
    public function prettyDateTime($date) {
        if (ctype_digit($date)) {
            $date = '@' . $date;
        }
        $dateTime = new DateTime($date);
        return $this->getPrettyDate($dateTime) . $dateTime->format(' H:i:s');
    }

    /**
     * Zformátuje instanci DateTime na formát např. "Dnes".
     * @param DateTime $dateTime instance DateTime
     * @return string zformátovaná hodnota
     */
    public function getPrettyDate($dateTime) {
        $now = new DateTime();
        if ($dateTime->format('Y') != $now->format('Y')) {
            return $dateTime->format('j.n.Y');
        }
        $dayMonth = $dateTime->format('d-m');
        if ($dayMonth == $now->format('d-m')) {
            return "Dnes";
        }
        $now->modify('-1 DAY');
        if ($dayMonth == $now->format('d-m')) {
            return "Včera";
        }
        $now->modify('+2 DAYS');
        if ($dayMonth == $now->format('d-m')) {
            return "Zítra";
        }
        return $dateTime->format('j. ') . self::$months[$dateTime->format('n') - 1];
    }
    
    /**
     * Vráti pole zkratek českých měsíců
     * @return array Pole zkratek měsíců
     */
    public function getChartMonths(){
        return self::$chartMonths;
    }

    /**
     * Vráti náhodně vygenerované heslo podle zadané délky
     * @param int length délka hesl
     * @return string vygenerované heslo
     */
    public function generatePassword($length) {
        return Random::generate($length);
    }

}
