<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\Services;

use mPDF;
use Nette\Mail\Message;
use Nette\Mail\SendException;
use Nette\Mail\SendmailMailer;
use Nette\InvalidStateException;
use Nette\Bridges\ApplicationLatte\Template;
use Nette\Database\Table\IRow;
use App\EshopModule\Model\SettingsManager;
use App\EshopModule\Model\OrderManager;
use App\CoreModule\Model\EmailSettingsManager;
use App\Services\OrderService;
use Nette\Application\UI\Presenter;
use Nette\Localization\ITranslator;

/**
 * Pomocná služba pro odesílání objednávakových e-mailů
 * Odesílá se e-mail při nové objednávce a taky když se změní její stav z adminsitrace
 * 
 * Parametry ($email a $emailMessage) a jejich settery se budou používat pro inicializaci těchto parametrů s pomocí DI 
 * a budou se do nich dosazovat hodnoty z konfiguračního souboru. 
 * DI s využitím setterů se používá pro inicializaci hodnot, na kterých není daná část aplikace existenčně závislá.
 * 
 * @package App\Services
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class OrderEmailService {

    /** Cesta k šabloně emailu objednávky. */
    const EMAIL_TEMPLATE = '/../modules/EshopModule/templates/Order/email.latte';

    /** Cesta k šabloně PDF faktury objednávky. */
    const PDF_TEMPLATE = '/../modules/EshopModule/templates/Order/pdf.latte';

    /** @var OrderManager */
    private $orderManager;

    /** @var SettingsManager Instance třídy modelu pro práci s účetním nastavením v e-shopu. */
    private $settingsManager;

    /** @var EmailSettingsManager */
    private $emailSettingsManager;

    /** @var OrderService Služba pro OrderPresentery, která poskytne společné metody pro naplnění šablon */
    private $orderService;
    
    /** @var ITranslator */
    private $translator;

    /**
     * OrderEmailService contructor
     */
    public function __construct(OrderManager $orderManager, SettingsManager $settingsManager, EmailSettingsManager $emailSettingsManager, OrderService $orderService, ITranslator $translator) {
        $this->orderManager = $orderManager;
        $this->settingsManager = $settingsManager;
        $this->emailSettingsManager = $emailSettingsManager;
        $this->orderService = $orderService;
        $this->translator = $translator;
    }

    /**
     * Odešle objednávku emailem spolu se zprávou.
     * @param string $message zpráva
     * @param Template $template šablona
     * @param Presenter $presenter presenter, který vykováná akci
     * @param int|null $orderId Id objednávky nebo null, pokud se má vzít aktuální objednávka
     * @return bool|mixed|IRow aktuální zkontrolovaná objednávka
     * @throws SendException Jestliže se nepodařilo email odeslat.
     */
    public function sendOrderEmail($message, $template, $presenter, $orderId = null) {
        $pdfTemplate = $template;
        $template->setFile(dirname(__FILE__) . self::EMAIL_TEMPLATE);
        // Naplním šablonu daty a vrátím si komplet data pro danou objednávku
        $order = $this->orderService->fillTemplateWithOrderData($template, $orderId, $presenter);
        $template = $this->fillTemplate($template, $message, $order);

        if (($email = $this->orderManager->getOrderEmail($order->getPrimary()))) {
            return $this->send($email, $orderId, $template, $pdfTemplate);
        } else {
            throw new SendException($this->translator->translate('eshop.order.flashes.emailSendNoEmaiAddress'));
        }
    }

    /**
     * Odesíla informační email o nové objednávce na zadaný mail z databáze
     * @throws SendException Jestliže se nepodařilo email odeslat.
     */
    public function sendEmailToOrderEmail() {
        $orderEmail = $this->emailSettingsManager->getPublishedRowByCondition(EmailSettingsManager::TABLE_NAME, EmailSettingsManager::COLUMN_EMAIL_TYPE, EmailSettingsManager::TYPE_ORDER_MAIL);

        try {
            $mail = $this->createMessage($orderEmail[EmailSettingsManager::COLUMN_EMAIL], $this->translator->translate('eshop.order.email.newOrder'), $this->translator->translate('eshop.order.email.newOrder'));

            $mailer = new SendmailMailer;
            $mailer->send($mail);
        } catch (InvalidStateException $ex) {
            throw new SendException($this->translator->translate('eshop.order.flashes.emailSendError'), $e->getCode(), $e);
        }
    }

    /**
     * Pomocná fce pro naplnění šablony pro email.
     * @param Template $template sablona pro naplnění
     * @param ArrayHAsh $order objednávka
     * @return Template $template
     */
    private function fillTemplate($template, $message, $order) {
        // Přidám do šablony variabilní symbol, settings a emailou zprávu
        $template->orderNumber = $order[OrderManager::COLUMN_NUMBER];
        $template->settings = $this->settingsManager->getSettings();

        if (is_array($message)) {
            $template->message = $this->translator->translate($message[1], ['state' => $message[0]]);
        } else {
            $template->message = $this->translator->translate($message);
        }
        return $template;
    }

    /**
     * Pomocná funkce pro odeslání rekapitulaci objednávky emailem.
     * @param strin $email email, na který se má zpráva odeslat
     * @param int $orderId id objednávaky
     * @param Template $template šablona pro email
     * @param Template $pdfTemplate dalsi sablona pro PDF
     * @return IRow objednávka
     */
    private function send($email, $orderId, $template, $pdfTemplate) {
        $mail = $this->createMessage($email, $this->translator->translate('eshop.order.email.subject'), $template);
        if ($orderId !== null) {
            $pdfTemplate->setFile(dirname(__FILE__) . self::PDF_TEMPLATE);
            $order = $this->orderService->fillTemplateWithOrderDetail($pdfTemplate, $orderId);

            // Render PDF.
            $mPdf = new mPDF();
            // parametr 1 (předání CSS)
            $mPdf->WriteHTML(file_get_contents(dirname(__FILE__) . '/../../www/css/global/invoice.css'), 1);
            // parametr 2 (předání Latte šablony)
            $mPdf->WriteHTML($pdfTemplate, 2);
            // PDF se vygeneruje metodou Output(), kde 2. parametr - 'D' = vynutí stažení faktury na místo zobrazení v prohlížeči.
            $mail->addAttachment($this->translator->translate('eshop.order.invoice') . '.pdf', $mPdf->Output('', 'S'));
        }

        try {
            (new SendmailMailer)->send($mail);
        } catch (SendException $e) {
            throw new SendException($this->translator->translate('eshop.order.flashes.emailSendError'), $e->getCode(), $e);
        }

        return $order;
    }

    /**
     * Pomocnná funkce pro vytvoření zprávy
     * @param string $addTo emailová adresa, kam se má zpráva poslat
     * @param string $subject předmět mailu
     * @param string $body zpráva mailu
     * @return Message message
     */
    private function createMessage($addTo, $subject, $body) {
        $senderEmail = $this->emailSettingsManager->getPublishedRowByCondition(EmailSettingsManager::TABLE_NAME, EmailSettingsManager::COLUMN_EMAIL_TYPE, EmailSettingsManager::TYPE_SENDER_MAIL);

        $mail = new Message;
        return $mail->setFrom($senderEmail[EmailSettingsManager::COLUMN_EMAIL])
                        ->addTo($addTo)
                        ->setSubject($subject)
                        ->setHtmlBody($body);
    }

}
