<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\Services;

use App\Model\Enum\MessageType;
use Nette\Application\BadRequestException;
use Nette\Database\Table\IRow;
use Nette\Bridges\ApplicationLatte\Template;
use App\EshopModule\Model\OrderManager;
use App\Model\PersonManager;
use App\EshopModule\Model\ProductAttributeManager;
use App\EshopModule\Model\DiscountLevelsManager;
use App\EshopModule\Model\ShippingMethodsManager;
use App\EshopModule\Model\PaymentMethodsManager;
use App\EshopModule\Model\CurrencyManager;
use App\EshopModule\Model\SettingsManager;
use Nette\Localization\ITranslator;
use App\Services\FormatHelperService;
use Nette\Application\UI\Presenter;

/**
 * Služba pro OrderPresentery, která poskytne metody pro naplnění šablon
 * Tyto metody jsou společné pro oba OrderPresentery (FrontModule, EshopModule)
 * Nelze vytvořit společného předka, proto se vytváří služba
 * 
 * @package App\Services
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class OrderService {

    /**
     * @var OrderManager
     */
    private $orderManager;

    /**
     * @var ShippingMethodsManager
     */
    private $shippingMethodsManager;

    /**
     * @var PaymentMethodsManager
     */
    private $paymentMethodsManager;

    /**
     * @var DiscountLevelsManager Instance třídy modelu pro práci se slevami v e-shopu. 
     */
    private $discountLevels;

    /**
     * @var PersonManager Instance třídy modelu pro práci s osobami v e-shopu. 
     */
    private $personManager;

    /**
     * @var ProductAttributeManager Instance třídy modelu pro správu atributu produktu.
     */
    private $productAttributeManager;
    
     /** @var SettingsManager Instance třídy modelu pro práci s účetním nastavením. */
    private $settingsManager;

    /**
     * @var ITranslator
     */
    private $translator;

    /**
     * @var FormatHelperService Pomocná služba pro formátování data, jména adresy atd.
     */
    private $formatHelperService;

    /**
     * OrderService constructor.
     * @param OrderManager $orderManager
     * @param ShippingMethodsManager $shippingMethodsManager
     * @param PaymentMethodsManager $paymentMethodsManager
     * @param DiscountLevelsManager $discountLevels
     * @param PersonManager $personManager
     * @param ProductAttributeManager $productAttributeManager
     * @param SettingsManager $settingsManager
     * @param ITranslator $translator
     * @param FormatHelperService $formatHelperService
     */
    public function __construct(OrderManager $orderManager, ShippingMethodsManager $shippingMethodsManager, PaymentMethodsManager $paymentMethodsManager, DiscountLevelsManager $discountLevels, PersonManager $personManager, ProductAttributeManager $productAttributeManager, SettingsManager $settingsManager, ITranslator $translator, FormatHelperService $formatHelperService) {
        $this->orderManager = $orderManager;
        $this->shippingMethodsManager = $shippingMethodsManager;
        $this->paymentMethodsManager = $paymentMethodsManager;
        $this->discountLevels = $discountLevels;
        $this->personManager = $personManager;
        $this->productAttributeManager = $productAttributeManager;
        $this->settingsManager = $settingsManager;
        $this->translator = $translator;
        $this->formatHelperService = $formatHelperService;
    }

    /**
     * Vyplní danou šablonu (šablona pro rekapitulaci objednávky a rekapitulačni mail) daty aktuální objednávky a zároveň je zkontroluje.
     * @param Template $template šablona
     * @param null|int $orderId  ID objednávky nebo null, pokud se má použít aktuální objednávka, přičemž kontrola
     *                           údajů se provádí pouze pro aktuální objednávku
     * @param Presenter $presenter presenter, který zpracovává akci
     * @return bool|mixed|IRow aktuální zkontrolovaná objednávka
     */
    public function fillTemplateWithOrderData(Template $template, $orderId = null, Presenter $presenter) {
        // Aktuální objednávka.
        $order = $this->orderManager->getOrder($orderId);
        
        // Produkty.
        $products = $this->orderManager->getProducts($order->getPrimary());

        if (empty($products) && !$orderId) {
            $presenter->flashMessage($this->translator->translate('eshop.order.flashes.emptyBasket'), MessageType::ERROR);
            $presenter->redirect(':Front:Page:', '');
        }

        // Způsob dopravy.
        if (!($deliveryProduct = $this->shippingMethodsManager->getById(ShippingMethodsManager::TABLE_NAME, ShippingMethodsManager::COLUMN_ID, $order[OrderManager::COLUMN_DELIVERY_PRODUCT])) && !$orderId) {
            $presenter->flashMessage($this->translator->translate('eshop.order.flashes.transportMethodError'), MessageType::ERROR);
            $presenter->redirect(':Front:Order:payment');
        }

        // Metody platby.
        if (!($paymentProduct = $this->paymentMethodsManager->getById(PaymentMethodsManager::TABLE_NAME, PaymentMethodsManager::COLUMN_ID, $order[OrderManager::COLUMN_PAYMENT_PRODUCT])) && !$orderId) {
            $presenter->flashMessage($this->translator->translate('eshop.order.flashes.paymentMethodError'), MessageType::ERROR);
            $presenter->redirect(':Fron:Order:payment');
        }

        // Osoba.
        if ($presenter->getUser()->isLoggedIn() && !$orderId) {
            // Reset adresy doručení pro případ, že ji uživatel změnil
            // Znovu nasetuju osobu k objednávce předtím, než si ji budu používat, protože uživatel od prvotního vyplnění mohl něco změnit
            $personId = $this->personManager->getPersonId($presenter->getUser()->getId());
            $this->orderManager->setPerson($personId, $order->getPrimary());
        }

        // Detail osoby.
        if ($order->buyer_id) {
            $person = $this->personManager->getPerson($order->buyer_id);
            $personDetail = $person->ref(PersonManager::TABLE_DETAIL_NAME);
        } else {
            $presenter->flashMessage('Vyplňte prosím údaje zákazníka.', MessageType::ERROR);
            $presenter->redirect(':Front:Person:register');
        }

        // Kupon pro objednavku
        $coupon = $this->discountLevels->getDiscount($this->orderManager->getOrderCoupon($order->getPrimary()));
        // Předám do šablony uživatelskou slevovou hladinu pro konkrétní objednávku
        $buyerUser = $this->personManager->getUserIdByPersonId($order[OrderManager::COLUMN_BUYER]);

        $discountLevel = $this->discountLevels->getDiscount($this->orderManager->getOrderUserDiscount($order->getPrimary()));

        /*
         * Souhrn objednávky - kdyz se odesilal email, nemohl najit objednavku, takze mu predam ID 
         * Vse ostatni je nastaveno defaultne TRUE, chci pocitat se vsemi slevami a nechci zapocitat v summary metody platby a dopravy
         * Metody prictu az v komponente price - $aditionalOrderMethodsPrice
         */
        $summary = $this->orderManager->getOrderSummary($orderId);
        $aditionalOrderMethodsPrice = $deliveryProduct[ShippingMethodsManager::COLUMN_SHIPPING_PRICE] + $paymentProduct[PaymentMethodsManager::COLUMN_PAYMENT_PRICE];

        $template->products = $products;
        $attributes = [];
        foreach ($template->products as $product) {
            $attributes[$product->product_id] = $this->productAttributeManager->getProductAttributes($product->product_id);
        }
        $template->attributes = $attributes;
        $template->deliveryProduct = $deliveryProduct;
        $template->paymentProduct = $paymentProduct;
        $template->person = $person;
        $template->personDetail = $personDetail;
        $template->summary = $summary;
        $template->coupon = $coupon ? $coupon : null;
        $template->discount = $discountLevel ? $discountLevel : null;
        $template->aditionalOrderMethodsPrice = $aditionalOrderMethodsPrice;
        $template->buyerUser = $buyerUser ? $buyerUser : null;
        $template->currency = $order[CurrencyManager::COLUMN_ID];
        $template->orderSettings = $this->settingsManager->getById(SettingsManager::TABLE_NAME, SettingsManager::COLUMN_ID, $order[OrderManager::COLUMN_SETTINGS]);

        // Doplnění šablony o užitečné filtry.
        // Doplnění šablony o užitečné filtry.
        $template->addFilter('address', [$this->formatHelperService, 'formatAddress']);
        $template->addFilter('name', [$this->formatHelperService, 'formatName']);

        return $order;
    }

    /**
     * Vyplní danou šablonu detaily aktuální objednávky - pro order manage a PDF.
     * @param Template $template šablona
     * @param int      $orderId  ID objednávky
     * @return bool|mixed|IRow objednávka
     * @throws BadRequestException Jestliže objednávka s daným ID neexistuje.
     */
    public function fillTemplateWithOrderDetail(Template $template, $orderId) {
        // Získání příslušné objednávky.
        if (!($order = $this->orderManager->getOrder($orderId))) {
            throw new BadRequestException($this->translator->translate('eshop.order.flashes.noOrderError'));
        }

        // Kupon pro objednavku
        $coupon = $this->discountLevels->getDiscount($this->orderManager->getOrderCoupon($order->getPrimary()));
        // Předám do šablony uživatelskou slevovou hladinu pro konkrétní objednávku
        $buyerUser = $this->personManager->getUserIdByPersonId($order[OrderManager::COLUMN_BUYER]);
        $discountLevel = $this->discountLevels->getDiscount($this->orderManager->getOrderUserDiscount($order->getPrimary()));
        
        // Nastavení pro danou objednávku
        $orderSettings = $this->settingsManager->getById(SettingsManager::TABLE_NAME, SettingsManager::COLUMN_ID, $order[OrderManager::COLUMN_SETTINGS]);

        // Vyplnění šablony daty.
        $template->order = $order;
        $template->orderSettings = $orderSettings;
        $template->buyer = $order->ref(PersonManager::TABLE_DETAIL_NAME, OrderManager::COLUMN_BUYER_PERSON_DETAIL);
        $template->buyerAddress = $order->ref(PersonManager::TABLE_ADDRESS_NAME, OrderManager::COLUMN_BUYER_ADDRESS);
        $template->buyerDeliveryAddress = $order->ref(PersonManager::TABLE_ADDRESS_NAME, OrderManager::COLUMN_BUYER_DELIVERY_ADDRESS); // Toto je zde nově navíc!
        $template->seller = $order->ref(PersonManager::TABLE_DETAIL_NAME, OrderManager::COLUMN_SELLER_PERSON_DETAIL);
        $template->sellerAddress = $order->ref(PersonManager::TABLE_ADDRESS_NAME, OrderManager::COLUMN_SELLER_ADDRESS);
        $template->sellerBankAccount = $sellerBankAccount = $order->ref(PersonManager::TABLE_BANK_ACCOUNT_NAME);
        $template->sellerBank = $this->personManager->getBank($sellerBankAccount[PersonManager::COLUMN_BANK_CODE]);
        $template->accountant = $this->personManager->getById(PersonManager::TABLE_DETAIL_NAME, PersonManager::COLUMN_PERSON_DETAIL_ID, $orderSettings[SettingsManager::COLUMN_ACCOUNTANT]);
        $template->products = $this->orderManager->getProducts($orderId);
        $attributes = [];
        foreach ($template->products as $product) {
            $attributes[$product->product_id] = $this->productAttributeManager->getProductAttributes($product->product_id);
        }
        $template->attributes = $attributes;
        $template->summary = $this->orderManager->getOrderSummary($orderId, FALSE);
        $template->coupon = $coupon ? $coupon : null;
        $template->discount = $discountLevel ? $discountLevel : null;
        $template->buyerUser = $buyerUser ? $buyerUser : null;

        // Doplnění šablony o užitečné filtry.
        $template->addFilter('address', [$this->formatHelperService, 'formatAddress']);
        $template->addFilter('name', [$this->formatHelperService, 'formatName']);
        // Získává celkovou cenu dané objednávky bez uživatelových slev.
        $template->addFilter('priceWithoutSales', function (IRow $order) {
            return $this->orderManager->getOrderSummary($order[OrderManager::COLUMN_ID], FALSE, FALSE)[OrderManager::ATTR_PRICE];
        });

        return $order;
    }

}
