<?php

/*
 * (C) Ideatech.cz
 * 2016
 * https://www.ideatech.cz/
 * Tento kód nesmí být šířen ani používán v open-source projektech.
 */

namespace App\Services;

use App\CoreModule\Model\UserRoleManager;
use App\CoreModule\Model\ModuleManager;
use Nette\Database\UniqueConstraintViolationException;

/**
 * Pomocná služba pro procházení souborů a hledání konrétních anotací, které potom uloží do databází
 * Hledá názvy presenterů (reserouces) a render funkcí (actions)
 * 
 * @package App\Services
 *
 * @author Jan Vodák <jan at ideatech.cz>
 */
class RBACLoader {

    const
            INIT_PATH = '../app/modules/',
            SUBSTRING_RESOURCE_NAME = 'Presenter.php',
            SUBSTRING_MODULE_NAME = 'Module/',
            SUBSTRING_MODULE_NAME_WINDOWS = 'Module\\',
            ANOTATION_NAME = '@actionName',
            ANOTATION_DESCRIPTION = '@actionDescription',
            ANOTATION_RESOURCE_DESCRIPTION = '@resourceDescription';

    /**
     *  @var UserRoleManager Instance třídy modelu pro práci se s uživatelskými rolemi. 
     */
    private $userRoleManager;
    
    /** 
     * @var ModuleManager Instance třídy modelu pro práci s moduly. 
     */
    private $moduleManager;

    /**
     * RBACLoader contructor
     * @param UserRoleManager $userRoleManager
     * @param ModuleManager $moduleManager
     */
    public function __construct(UserRoleManager $userRoleManager, ModuleManager $moduleManager) {
        $this->userRoleManager = $userRoleManager;
        $this->moduleManager = $moduleManager;
    }

    /**
     * Jediná přístupná funkce z venku, která vše spustí
     */
    public function run() {
        $this->listFolderFiles(self::INIT_PATH);
    }

    /**
     * Metoda vrací jméno zdroje ve tvaru Modul:Presenter
     * @param String $singleFileName jméno souboru
     * @param String $singleFilePath celá cesta i s jménem k souboru
     * @return String název zdroje pro práci s uživatelskými rolemi
     */
    private function getResourceName($singleFileName, $singleFilePath) {
        // Ořízne od klíčového stringu vše až po konec
        if (strstr($singleFilePath, self::SUBSTRING_MODULE_NAME, true)) {
            // Unix cesta
            $substrModule = strstr($singleFilePath, self::SUBSTRING_MODULE_NAME, true);
            // V oříznutém stringu hledám poslední výskyt lomítka a vše po něj oříznu - zbyde název modulu
            $module = substr($substrModule, strrpos($substrModule, '/') + 1);
        } else {
            // Windows cesta
            $substrModule = strstr($singleFilePath, self::SUBSTRING_MODULE_NAME_WINDOWS, true);
            // V oříznutém stringu hledám poslední výskyt lomítka a vše po něj oříznu - zbyde název modulu
            $module = substr($substrModule, strrpos($substrModule, '\\') + 1);
        }
        
        // Z názvu souboru oříznu koncovku Presenter.php, zbyde mi jen název presenteru
        $resource = strstr($singleFileName, self::SUBSTRING_RESOURCE_NAME, true);
        return trim(preg_replace('/\s\s+/', ' ', $module)) . ':' . trim(preg_replace('/\s\s+/', ' ', $resource));
    }

    /**
     * Metoda vrací pole pro akci - název akce a pokud je zadán tak i popisek akce
     * @param String $line konrétní řádek
     * @param String $lines všechny řádky
     * @param int $iterator Iterátor aktuálního řádku - slouží pro kontrolu, jeslti nejsem na konci souboru
     * @return ArrayHash hodnoty pro akci, které se uloží do DB
     */
    private function getActionValues($line, $lines, $iterator) {
        $actionValues = array();

        if (strpos($line, self::ANOTATION_NAME) !== false) {
            // Pokud je na řádku zadána anotace pro název akce (funkcue) - oříznu si její název
            // Ořezávám na řádku, který je string od pozice nálezu + délka toho slova + 1 kvůli mezeře
            $actionName = substr($line, (strrpos($line, self::ANOTATION_NAME) + strlen(self::ANOTATION_NAME) + 1));
            // Předám do pole
            $actionValues[UserRoleManager::COLUMN_ACTION_NAME] = trim(preg_replace('/\s\s+/', ' ', $actionName));

            // Kontroluji, jestli se na následujícím řádku nevyskytuje popisek k akci
            if ($iterator + 1 < sizeof($lines) && strpos($lines[$iterator + 1], self::ANOTATION_DESCRIPTION) !== false) {
                // Pokud ano, oříznuji si jen ten popisek
                // Ořezávám na řádku, který je string od pozice nálezu + délka toho slova + 1 kvůli mezeře
                $actionDescription = substr($lines[$iterator + 1], (strrpos($lines[$iterator + 1], self::ANOTATION_DESCRIPTION) + strlen(self::ANOTATION_DESCRIPTION) + 1));
                // Přidám do výsledného pole
                $actionValues[UserRoleManager::COLUMN_ACTION_DESRCIPTION] = trim(preg_replace('/\s\s+/', ' ', $actionDescription));
            }
        }

        return $actionValues;
    }

    /**
     * Metoda kontroluje a případně vrací popisek zdroje, který se nachází hned na začátku souboru
     * @param String $line řádek souboru
     * @return String|null konkrétní popisek, nebo null, pokud se na řádku nevyskytuje
     */
    private function getResourceDescription($line) {
        if (strpos($line, self::ANOTATION_RESOURCE_DESCRIPTION) !== false) {
            return substr($line, (strlen(self::ANOTATION_RESOURCE_DESCRIPTION) + 4));
        }

        return null;
    }

    /**
     * Hlavní metoda pro procházení adresářů a souborů v něm umístěných
     * @param String $dir výchozí adresář, ve kterém se začne prohledávat
     */
    private function listFolderFiles($dir) {
        // Získám si všechny soubory v aktuálním adresáři
        $allFiles = scandir($dir);

        // Projdi všechny soubory
        foreach ($allFiles as $singleFileName) {
            // Soubor i s cestou
            $singleFilePath = realpath($dir . DIRECTORY_SEPARATOR . $singleFileName);

            // Funkce pro práci s konrétním souborem
            $this->testFile($singleFileName, $singleFilePath);
        }
    }

    /**
     * Funkce pro rozhodování se o práci se souborem
     * @param String $singleFileName Jméno souboru
     * @param String $singleFilePath Celá cesta i s názvem souboru
     */
    private function testFile($singleFileName, $singleFilePath) {
        // Jen konkrétni soubory - v allFiles muze byt vlivem scandiru i tecky, které značím adresáře
        if ($singleFileName != '.' && $singleFileName != '..') {

            // Podmínka pro kontrolu php souboru
            if (substr($singleFileName, strlen($singleFileName) - 4, strlen($singleFileName)) == ".php") {
                // Soubor po řádcích
                $lines = file($singleFilePath);
                // Čti soubor
                $this->readFile($lines, $singleFileName, $singleFilePath);
            }

            // Pokud je soubor adresář, rekurzivní zanoření do něj a zase procházím soubory v něm
            if (is_dir($singleFilePath)) {
                $this->listFolderFiles($singleFilePath);
            }
        }
    }

    /**
     * Funkce pro čtení souboru
     * @param array() $lines Description
     * @param String $singleFileName jméno souboru
     * @param String $singleFilePath celá cesta i s jménem k souboru
     */
    private function readFile($lines, $singleFileName, $singleFilePath) {
        $i = 0;
        // Na začátku je popisek zdorje null, ale je lepší si ho později pamatovat, než jej dodatečně zase hledat
        $resourceDescription = null;

        // Projdu všechny řádky
        foreach ($lines as $line) {
            if (!$resourceDescription) {
                // Kontroluji, jestli popisek zdroje není vyplněn, případně jej vyplním, pokud na něj v souboru narazím
                $resourceDescription = $this->getResourceDescription($line);
            }

            // Hledám na řádku anotace pro název a popisek akce
            $actionValues = $this->getActionValues($line, $lines, $i);

            // Pokud jsem našel akci a případně i její popisek, najdu si název zdroje případně i jeho popsiek
            if (!empty($actionValues)) {
                $resourceValues[UserRoleManager::COLUMN_RESOURCE_NAME] = $this->getResourceName($singleFileName, $singleFilePath);
                $resourceValues[UserRoleManager::COLUMN_RESOURCE_DESRCIPTION] = $resourceDescription;

                // Pokusím se všechny věci uložit
                $this->saveAllComponents($resourceValues, $actionValues);
            }

            // Potřebuji iterátor ve foreach
            $i++;
        }
    }

    /**
     * Funkce pro ukládání hodnot - pokusí se najít, případně uložit zdroj, pokud nebyl nalezen
     * Ke zdroji se pokusí uložit i nalezené akce (pokud už nejsou uloženy)
     * @param ArrayHash() $resourceValues Pole pro uložení do zdroj
     * @param ArrayHash() $actionValues Pole pro uložení pro akce
     */
    private function saveAllComponents($resourceValues, $actionValues) {
        // Zkusím si najít resource
        $resourceID = $this->userRoleManager->getById(UserRoleManager::TABLE_NAME_RESOURCE, UserRoleManager::COLUMN_RESOURCE_NAME, $resourceValues[UserRoleManager::COLUMN_RESOURCE_NAME]);

        if (!$resourceID) {
            $moduleID = $this->moduleManager->getById(ModuleManager::TABLE_NAME, ModuleManager::COLUMN_SLUG, strstr($resourceValues[UserRoleManager::COLUMN_RESOURCE_NAME], ':', true))[ModuleManager::COLUMN_ID];
            $resourceValues[ModuleManager::COLUMN_ID] = $moduleID;
            $resourceID = $this->saveFunction(UserRoleManager::TABLE_NAME_RESOURCE, UserRoleManager::COLUMN_RESOURCE_ID, $resourceValues)[UserRoleManager::COLUMN_RESOURCE_ID];
        }

        if ($resourceID) {
            // Předám si id pro resource do action values - obsahuje i toto idecko, kvuli filtrovani akci pro presentery
            $actionValues[UserRoleManager::COLUMN_RESOURCE_ID] = $resourceID;

            // Zkusím si najít záznam pro akci v DB, pokud už je v DB, vůbec se nepokouším ukládat hodnoty
            // Vyjímka by mi to odchytila a neuložilo by se to, ale autoincrementovalo by se ID
            $actionID = $this->userRoleManager->getRowByMoreConditons(UserRoleManager::TABLE_ACTION_NAME, $actionValues)[UserRoleManager::COLUMN_ACTION_ID];
            
            if (!$actionID) {
                $this->saveFunction(UserRoleManager::TABLE_ACTION_NAME, UserRoleManager::COLUMN_ACTION_ID, $actionValues);
            }
        }
    }

    private function saveFunction($tableName, $id, $values) {
        try {
            // Zkusím uložit hodnoty, pokud ještě nejsou uloženy - pokud jsou, klíče mi vyhodí vyjímku
            $this->userRoleManager->saveValues($tableName, $id, $values);
        } catch (UniqueConstraintViolationException $ex) {
            // Neprovede se nic, jen se odchytí vyjímka
        }
    }

}
